@@ -54,22 +54,21 @@
  * use as firmware manifest key. If $SIGID starts with 'sig-id-in-*' then we
  * have to replace it by VPD value 'custom_label_tag' as '$MODEL-$CLTAG'.
  *
- * The current implementation is to first look at `setvars.sh` first, and then
- * fallback to `signer_config.csv` if needed.
+ * The current implementation is to try `signer_config.csv` approach first, and
+ * then fallback to `setvars.sh` on failure.
  */
 
 static const char * const SETVARS_IMAGE_MAIN = "IMAGE_MAIN",
 		  * const SETVARS_IMAGE_EC = "IMAGE_EC",
 		  * const SETVARS_SIGNATURE_ID = "SIGNATURE_ID",
 		  * const SIG_ID_IN_VPD_PREFIX = "sig-id-in",
-		  * const DIR_KEYSET = "keyset",
 		  * const DIR_MODELS = "models",
 		  * const DEFAULT_MODEL_NAME = "default",
 		  * const VPD_CUSTOM_LABEL_TAG = "custom_label_tag",
 		  * const VPD_CUSTOM_LABEL_TAG_LEGACY = "whitelabel_tag",
 		  * const VPD_CUSTOMIZATION_ID = "customization_id",
 		  * const ENV_VAR_MODEL_DIR = "${MODEL_DIR}",
-		  * const PATH_STARTSWITH_KEYSET = "keyset/",
+		  * const PATH_KEYSET_FOLDER = "keyset/",
 		  * const PATH_SIGNER_CONFIG = "signer_config.csv",
 		  * const PATH_ENDSWITH_SETVARS = "/setvars.sh";
 
@@ -314,7 +313,7 @@ static void find_patches_for_model(struct model_config *model,
 
 	assert(ARRAY_SIZE(names) == ARRAY_SIZE(targets));
 	for (i = 0; i < ARRAY_SIZE(names); i++) {
-		ASPRINTF(&path, "%s/%s.%s", DIR_KEYSET, names[i], signature_id);
+		ASPRINTF(&path, "%s%s.%s", PATH_KEYSET_FOLDER, names[i], signature_id);
 		if (archive_has_entry(archive, path))
 			*targets[i] = path;
 		else
@@ -354,8 +353,6 @@ static int manifest_scan_entries(const char *name, void *arg)
 	struct model_config model = {0};
 	char *slash;
 
-	if (str_startswith(name, PATH_STARTSWITH_KEYSET))
-		manifest->has_keyset = 1;
 	if (!str_endswith(name, PATH_ENDSWITH_SETVARS))
 		return 0;
 
@@ -457,6 +454,8 @@ static int manifest_from_signer_config(struct manifest *manifest)
 	uint8_t *data;
 	char *s, *tok_ptr = NULL;
 
+	VB2_DEBUG("Try to build the manifest from %s\n", PATH_SIGNER_CONFIG);
+
 	if (!archive_has_entry(archive, PATH_SIGNER_CONFIG))
 		return -1;
 
@@ -572,6 +571,8 @@ static int manifest_from_simple_folder(struct manifest *manifest)
 	struct firmware_image image = {0};
 	struct model_config model = {0};
 
+	VB2_DEBUG("Try to build the manifest from a simple folder\n");
+
 	/* Try to load from current folder. */
 	if (archive_has_entry(archive, old_host_image_name))
 		image_name = old_host_image_name;
@@ -811,37 +812,78 @@ int model_apply_custom_label(
 	return r;
 }
 
+/*
+ * b/251040363: Checks if the archive must be parsed using setvars.sh.
+ */
+static bool manifest_must_enforce_setvars(struct manifest *manifest)
+{
+	int i;
+	const char *setvars_list[] = {
+		"setvars_sh_only",
+	};
+
+	for (i = 0; i < ARRAY_SIZE(setvars_list); i++) {
+		if (archive_has_entry(manifest->archive, setvars_list[i])) {
+			INFO("Detected %s, will use *%s.\n",
+			     setvars_list[i], PATH_ENDSWITH_SETVARS);
+			return true;
+		}
+	}
+	return false;
+}
+
+static int manifest_from_setvars_sh(struct manifest *manifest) {
+	VB2_DEBUG("Try to build the manifest from *%s\n", PATH_ENDSWITH_SETVARS);
+	return archive_walk(manifest->archive, manifest, manifest_scan_entries);
+}
+
+static int manifest_from_build_artifacts(struct manifest *manifest) {
+	VB2_DEBUG("Try to build the manifest from a */firmware folder\n");
+	return archive_walk(manifest->archive, manifest, manifest_scan_raw_entries);
+}
+
 /*
  * Creates a new manifest object by scanning files in archive.
  * Returns the manifest on success, otherwise NULL for failure.
  */
 struct manifest *new_manifest_from_archive(struct u_archive *archive)
 {
+	int i;
 	struct manifest manifest = {0}, *new_manifest;
+	bool try_builders = true;
+	int (*manifest_builders[])(struct manifest *) = {
+		manifest_from_signer_config,
+		manifest_from_setvars_sh,
+		manifest_from_build_artifacts,
+		manifest_from_simple_folder,
+	};
 
 	manifest.archive = archive;
 	manifest.default_model = -1;
+	if (archive_has_entry(archive, PATH_KEYSET_FOLDER))
+		manifest.has_keyset = 1;
+	VB2_DEBUG("Has keyset: %s\n", manifest.has_keyset ? "True" : "False");
 
-	VB2_DEBUG("Try to build a manifest from *%s\n", PATH_ENDSWITH_SETVARS);
-	archive_walk(archive, &manifest, manifest_scan_entries);
+	if (manifest_must_enforce_setvars(&manifest)) {
+		try_builders = false;
+		manifest_from_setvars_sh(&manifest);
+	}
 
-	if (manifest.num == 0) {
-		VB2_DEBUG("Try to build a manifest from %s\n",
-			  PATH_SIGNER_CONFIG);
-		manifest_from_signer_config(&manifest);
-	}
-	if (manifest.num == 0) {
-		VB2_DEBUG("Try to build a manifest from a */firmware folder\n");
-		archive_walk(archive, &manifest, manifest_scan_raw_entries);
-	}
-	if (manifest.num == 0) {
-		VB2_DEBUG("Try to build a manifest from a simple folder\n");
-		manifest_from_simple_folder(&manifest);
+	for (i = 0; try_builders && i < ARRAY_SIZE(manifest_builders); i++) {
+		/*
+		 * For archives manually updated (for testing), it is possible a
+		 * builder can successfully scan the archive but no valid models
+		 * were found, so here we don't need to check the return value.
+		 * Only stop when manifest.num is non-zero.
+		 */
+		(void) manifest_builders[i](&manifest);
+		if (manifest.num)
+			try_builders = false;
 	}
 
 	VB2_DEBUG("%d model(s) loaded.\n", manifest.num);
 	if (!manifest.num) {
-		ERROR("No valid configurations found from archive.\n");
+		ERROR("No valid configurations found from the archive.\n");
 		return NULL;
 	}
 
@@ -898,9 +940,17 @@ static void print_json_image(
 		return;
 	if (!is_first)
 		printf(",\n");
-	printf("%*s\"%s\": { \"versions\": { \"ro\": \"%s\", \"rw\": \"%s\" },",
-	       indent, "", name, image.ro_version, image.rw_version_a);
+	printf("%*s\"%s\": {", indent, "", name);
 	indent += 2;
+	printf("\n%*s\"versions\": {", indent, "");
+	indent += 2;
+	printf("\n%*s\"ro\": \"%s\"", indent, "", image.ro_version);
+	printf(",\n%*s\"rw\": \"%s\"", indent, "", image.rw_version_a);
+	if (is_host && image.ecrw_version_a[0] != '\0')
+		printf(",\n%*s\"ecrw\": \"%s\"", indent, "",
+		       image.ecrw_version_a);
+	indent -= 2;
+	printf("\n%*s},", indent, "");
 	if (is_host) {
 		if (patch_image_by_model(&image, m, archive))
 			ERROR("Failed to patch images by model: %s\n", m->name);
@@ -916,7 +966,10 @@ static void print_json_image(
 		       get_gbb_key_hash(gbb, gbb->recovery_key_offset,
 					gbb->recovery_key_size));
 	}
-	printf("\n%*s\"image\": \"%s\" }", indent, "", fpath);
+	printf("\n%*s\"image\": \"%s\"", indent, "", fpath);
+	indent -= 2;
+	printf("\n%*s}", indent, "");
+	check_firmware_versions(&image);
 	free_firmware_image(&image);
 }
 
