@@ -16,6 +16,7 @@
 #endif
 
 #include "2common.h"
+#include "cbfstool.h"
 #include "host_misc.h"
 #include "util_misc.h"
 #include "updater.h"
@@ -70,54 +71,6 @@ int save_file_from_stdin(const char *output)
 	return 0;
 }
 
-/*
- * Returns 1 if a given file (cbfs_entry_name) exists inside a particular CBFS
- * section of an image file, otherwise 0.
- */
-int cbfs_file_exists(const char *image_file,
-		     const char *section_name,
-		     const char *cbfs_entry_name)
-{
-	char *cmd;
-	int r;
-
-	ASPRINTF(&cmd,
-		 "cbfstool '%s' print -r %s 2>/dev/null | grep -q '^%s '",
-		 image_file, section_name, cbfs_entry_name);
-	r = system(cmd);
-	free(cmd);
-	return !r;
-}
-
-/*
- * Extracts files from a CBFS on given region (section) of image_file.
- * Returns the path to a temporary file on success, otherwise NULL.
- */
-const char *cbfs_extract_file(const char *image_file,
-			      const char *cbfs_region,
-			      const char *cbfs_name,
-			      struct tempfile *tempfiles)
-{
-	const char *output = create_temp_file(tempfiles);
-	char *command, *result;
-
-	if (!output)
-		return NULL;
-
-	ASPRINTF(&command, "cbfstool \"%s\" extract -r %s -n \"%s\" "
-		 "-f \"%s\" 2>&1", image_file, cbfs_region,
-		 cbfs_name, output);
-
-	result = host_shell(command);
-	free(command);
-
-	if (!*result)
-		output = NULL;
-
-	free(result);
-	return output;
-}
-
 /*
  * Loads the firmware information from an FMAP section in loaded firmware image.
  * The section should only contain ASCIIZ string as firmware version.
@@ -158,6 +111,68 @@ static int load_firmware_version(struct firmware_image *image,
 	return 0;
 }
 
+/*
+ * Loads the version of "ecrw" CBFS file within `section_name` of `image_file`.
+ * Returns the version string on success; otherwise an empty string.
+ */
+static char *load_ecrw_version(const struct firmware_image *image,
+			       const char *image_file,
+			       const char *section_name)
+{
+	char *version = NULL;
+	struct tempfile tempfile_head = {0};
+
+	/* EC image or older AP images may not have the section. */
+	if (!firmware_section_exists(image, section_name))
+		goto done;
+
+	const char *ecrw_version_file = create_temp_file(&tempfile_head);
+	if (!ecrw_version_file)
+		goto done;
+
+	/* "ecrw.version" doesn't exist in old images. */
+	const char *ecrw_version_name = "ecrw.version";
+	if (!cbfstool_file_exists(image_file, section_name, ecrw_version_name))
+		goto done;
+
+	if (cbfstool_extract(image_file, section_name, ecrw_version_name,
+			     ecrw_version_file)) {
+		ERROR("Failed to extract %s from %s\n",
+		      ecrw_version_name, section_name);
+		goto done;
+	}
+
+	uint8_t *data;
+	uint32_t size;
+	if (vb2_read_file(ecrw_version_file, &data, &size) != VB2_SUCCESS)
+		goto done;
+
+	version = strndup((const char *)data, size);
+
+done:
+	if (!version)
+		version = strdup("");
+	remove_all_temp_files(&tempfile_head);
+	return version;
+}
+
+/* Loads the version of "ecrw" CBFS file for FW_MAIN_A and FW_MAIN_B. */
+static void load_ecrw_versions(struct firmware_image *image)
+{
+	struct tempfile tempfile_head = {0};
+	const char *image_file = get_firmware_image_temp_file(
+			image, &tempfile_head);
+
+	if (image_file) {
+		image->ecrw_version_a = load_ecrw_version(
+				image, image_file, FMAP_RW_FW_MAIN_A);
+		image->ecrw_version_b = load_ecrw_version(
+				image, image_file, FMAP_RW_FW_MAIN_B);
+	}
+
+	remove_all_temp_files(&tempfile_head);
+}
+
 /*
  * Fills in the other fields of image using image->data.
  * Returns IMAGE_LOAD_SUCCESS or IMAGE_PARSE_FAILURE.
@@ -198,16 +213,11 @@ static int parse_firmware_image(struct firmware_image *image)
 	load_firmware_version(image, section_a, &image->rw_version_a);
 	load_firmware_version(image, section_b, &image->rw_version_b);
 
+	load_ecrw_versions(image);
+
 	return ret;
 }
 
-/*
- * Loads a firmware image from file.
- * If archive is provided and file_name is a relative path, read the file from
- * archive.
- * Returns IMAGE_LOAD_SUCCESS on success, IMAGE_READ_FAILURE on file I/O
- * failure, or IMAGE_PARSE_FAILURE for non-vboot images.
- */
 int load_firmware_image(struct firmware_image *image, const char *file_name,
 			struct u_archive *archive)
 {
@@ -233,6 +243,19 @@ int load_firmware_image(struct firmware_image *image, const char *file_name,
 	return parse_firmware_image(image);
 }
 
+void check_firmware_versions(const struct firmware_image *image)
+{
+	if (strcmp(image->rw_version_a, image->rw_version_b))
+		WARN("Different versions in %s (%s) and %s (%s).\n",
+		     FMAP_RW_FWID_A, image->rw_version_a,
+		     FMAP_RW_FWID_B, image->rw_version_b);
+	if (image->ecrw_version_a && image->ecrw_version_b &&
+	    strcmp(image->ecrw_version_a, image->ecrw_version_b))
+		WARN("Different ecrw versions in %s (%s) and %s (%s).\n",
+		     FMAP_RW_FW_MAIN_A, image->ecrw_version_a,
+		     FMAP_RW_FW_MAIN_B, image->ecrw_version_b);
+}
+
 /*
  * Generates a temporary file for snapshot of firmware image contents.
  *
@@ -270,10 +293,18 @@ void free_firmware_image(struct firmware_image *image)
 	free(image->ro_version);
 	free(image->rw_version_a);
 	free(image->rw_version_b);
+	free(image->ecrw_version_a);
+	free(image->ecrw_version_b);
 	memset(image, 0, sizeof(*image));
 	image->programmer = programmer;
 }
 
+int reload_firmware_image(const char *file_path, struct firmware_image *image)
+{
+	free_firmware_image(image);
+	return load_firmware_image(image, file_path, NULL);
+}
+
 /*
  * Finds a firmware section by given name in the firmware image.
  * If successful, return zero and *section argument contains the address and
@@ -358,13 +389,35 @@ const struct vb2_gbb_header *find_gbb(const struct firmware_image *image)
 }
 
 /*
- * Returns true if the write protection is enabled on current system.
+ * Different settings may have different SWWP programmers.
  */
-int is_write_protection_enabled(struct updater_config *cfg)
+static bool is_write_protection_enabled(struct updater_config *cfg,
+					const char *programmer,
+					enum dut_property_type swwp_type)
 {
 	/* Assume HW/SW WP are enabled if -1 error code is returned */
-	return dut_get_property(DUT_PROP_WP_HW, cfg) &&
-	       dut_get_property(DUT_PROP_WP_SW, cfg);
+	bool hwwp = !!dut_get_property(DUT_PROP_WP_HW, cfg);
+	bool swwp = !!dut_get_property(swwp_type, cfg);
+	bool wp_enabled = hwwp && swwp;
+	STATUS("Write protection (%s): %d (%s; HW=%d, SW=%d).\n", programmer,
+	       wp_enabled, wp_enabled ? "enabled" : "disabled", hwwp, swwp);
+	return wp_enabled;
+}
+
+/*
+ * Returns true if the AP write protection is enabled on current system.
+ */
+inline bool is_ap_write_protection_enabled(struct updater_config *cfg)
+{
+	return is_write_protection_enabled(cfg, cfg->image.programmer, DUT_PROP_WP_SW_AP);
+}
+
+/*
+ * Returns true if the EC write protection is enabled on current system.
+ */
+inline bool is_ec_write_protection_enabled(struct updater_config *cfg)
+{
+	return is_write_protection_enabled(cfg, cfg->ec_image.programmer, DUT_PROP_WP_SW_EC);
 }
 
 /*
@@ -421,6 +474,7 @@ char *host_detect_servo(const char **prepare_ctrl_name)
 {
 	const char *servo_port = getenv(ENV_SERVOD_PORT);
 	const char *servo_name = getenv(ENV_SERVOD_NAME);
+	const char *servo_id = servo_port, *servo_id_type = ENV_SERVOD_PORT;
 	char *servo_type = host_shell("dut-control -o servo_type 2>/dev/null");
 	const char *programmer = NULL;
 	char *ret = NULL;
@@ -429,35 +483,26 @@ char *host_detect_servo(const char **prepare_ctrl_name)
 	static const char * const raiden_debug_spi = "raiden_debug_spi";
 	static const char * const cpu_fw_spi = "cpu_fw_spi";
 	static const char * const ccd_cpu_fw_spi = "ccd_cpu_fw_spi";
+	const char *serial_cmd = "dut-control -o serialname 2>/dev/null";
 
 	/* By default, no control is needed. */
 	*prepare_ctrl_name = NULL;
 	VB2_DEBUG("servo_type: %s\n", servo_type);
 
-	/* Get serial name if servo port is provided. */
-	if ((servo_port && *servo_port) || (servo_name && *servo_name)) {
-		const char *cmd = "dut-control -o serialname 2>/dev/null";
-
-		VB2_DEBUG("Select servod using port: %s or name: %s\n",
-			  servo_port, servo_name);
-		if (strstr(servo_type, "with_servo_micro"))
-			cmd = ("dut-control -o servo_micro_serialname"
-			       " 2>/dev/null");
-		else if (strstr(servo_type, "with_c2d2"))
-			cmd = ("dut-control -o c2d2_serialname"
-			       " 2>/dev/null");
-		else if (strstr(servo_type, "with_ccd"))
-			cmd = "dut-control -o ccd_serialname 2>/dev/null";
-
-		servo_serial = host_shell(cmd);
-		VB2_DEBUG("Servo SN=%s (serial cmd: %s)\n", servo_serial, cmd);
+	/* dut-control defaults to port 9999, or non-empty servo_name. */
+	if (!servo_id || !*servo_id) {
+		if (servo_name && *servo_name) {
+			servo_id = servo_name;
+			servo_id_type = ENV_SERVOD_NAME;
+		} else {
+			servo_id = "9999";
+		}
 	}
+	assert(servo_id && *servo_id);
 
 	/* servo_type names: chromite/lib/firmware/servo_lib.py */
 	if (!*servo_type) {
 		ERROR("Failed to get servo type. Check servod.\n");
-	} else if (servo_serial && !*servo_serial) {
-		ERROR("Failed to get serial at servo port %s.\n", servo_port);
 	} else if (strcmp(servo_type, "servo_v2") == 0) {
 		VB2_DEBUG("Selected Servo V2.\n");
 		programmer = "ft2232_spi:type=google-servo-v2";
@@ -466,23 +511,42 @@ char *host_detect_servo(const char **prepare_ctrl_name)
 		VB2_DEBUG("Selected Servo Micro.\n");
 		programmer = raiden_debug_spi;
 		*prepare_ctrl_name = cpu_fw_spi;
-	} else if (strstr(servo_type, "c2d2")) {
-		VB2_DEBUG("Selected C2D2.\n");
-		programmer = raiden_debug_spi;
-		*prepare_ctrl_name = cpu_fw_spi;
+		serial_cmd = ("dut-control -o servo_micro_serialname"
+			" 2>/dev/null");
 	} else if (strstr(servo_type, "ccd_cr50") ||
 		   strstr(servo_type, "ccd_gsc") ||
 		   strstr(servo_type, "ccd_ti50")) {
 		VB2_DEBUG("Selected CCD.\n");
 		programmer = "raiden_debug_spi:target=AP,custom_rst=true";
 		*prepare_ctrl_name = ccd_cpu_fw_spi;
+		serial_cmd = "dut-control -o ccd_serialname 2>/dev/null";
+	} else if (strstr(servo_type, "c2d2")) {
+		/* Most C2D2 devices don't support flashing AP, so this must
+		 * come after CCD.
+		 */
+		VB2_DEBUG("Selected C2D2.\n");
+		programmer = raiden_debug_spi;
+		*prepare_ctrl_name = cpu_fw_spi;
+		serial_cmd = ("dut-control -o c2d2_serialname"
+			" 2>/dev/null");
 	} else {
 		WARN("Unknown servo: %s\nAssuming debug header.\n", servo_type);
 		programmer = raiden_debug_spi;
 		*prepare_ctrl_name = cpu_fw_spi;
 	}
 
-	if (programmer) {
+	/*
+	 * To support "multiple servos connected but only one servod running" we
+	 * should always try to get the serial number.
+	 */
+	VB2_DEBUG("Select servod by %s=%s\n", servo_id_type, servo_id);
+	servo_serial = host_shell(serial_cmd);
+	VB2_DEBUG("Servo SN=%s (serial cmd: %s)\n", servo_serial, serial_cmd);
+	if (!(servo_serial && *servo_serial)) {
+		ERROR("Failed to get serial: %s=%s\n", servo_id_type, servo_id);
+		/* If there is no servo serial, undo the prepare_ctrl_name. */
+		*prepare_ctrl_name = NULL;
+	} else if (programmer) {
 		if (!servo_serial) {
 			ret = strdup(programmer);
 		} else {
@@ -520,6 +584,9 @@ static int is_the_same_programmer(const struct firmware_image *image1,
 int load_system_firmware(struct updater_config *cfg,
 			 struct firmware_image *image)
 {
+	if (!strcmp(image->programmer, FLASHROM_PROGRAMMER_INTERNAL_EC))
+		WARN("%s: flashrom support for CrOS EC is EOL.\n", __func__);
+
 	int r, i;
 	const int tries = 1 + get_config_quirk(QUIRK_EXTRA_RETRIES, cfg);
 
@@ -547,6 +614,10 @@ int write_system_firmware(struct updater_config *cfg,
 			  const char * const regions[],
 				const size_t regions_len)
 {
+	if (!strcmp(image->programmer, FLASHROM_PROGRAMMER_INTERNAL_EC)) {
+		WARN("%s: flashrom support for CrOS EC is EOL.\n", __func__);
+	}
+
 	int r = 0, i;
 	const int tries = 1 + get_config_quirk(QUIRK_EXTRA_RETRIES, cfg);
 	struct firmware_image *flash_contents = NULL;
@@ -663,7 +734,7 @@ const char *get_firmware_rootkey_hash(const struct firmware_image *image)
  * given values.
  * Returns 0 on success, otherwise failure.
  */
-static int overwrite_section(struct firmware_image *image,
+int overwrite_section(struct firmware_image *image,
 			     const char *fmap_section, size_t offset,
 			     size_t size, const uint8_t *new_values)
 {
@@ -683,70 +754,3 @@ static int overwrite_section(struct firmware_image *image,
 	memcpy(section.data + offset, new_values, size);
 	return 0;
 }
-
-/*
- * Unlock the Flash Master values in SI_DESC.
- * Returns 0 on success, otherwise failure.
- *
- * TODO(b/270275115): Replace with a call to ifdtool.
- */
-int unlock_flash_master(struct firmware_image *image)
-{
-	const int flash_master_offset = 0x80;
-	const uint8_t flash_master[] = {
-		0x00, 0xff, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff, 0x00, 0xff,
-		0xff, 0xff
-	};
-
-	if (overwrite_section(image, FMAP_SI_DESC, flash_master_offset,
-			      ARRAY_SIZE(flash_master), flash_master)) {
-		ERROR("Failed unlocking Flash Master values\n");
-		return -1;
-	}
-
-	INFO("Changed Flash Master Values to unlocked.\n");
-	return 0;
-}
-
-/*
- * Disable GPR0 (Global Protected Range). When enabled, it provides
- * write-protection to part of the SI_ME region, specifically CSE_RO and
- * part of CSE_DATA, so it must be disabled to allow updating SI_ME.
- * Returns 0 on success, otherwise failure.
- *
- * TODO(b/270275115): Replace with a call to ifdtool.
- */
-static int disable_gpr0(struct firmware_image *image)
-{
-	const int gpr0_offset = 0x154;
-	const uint8_t gpr0_value_disabled[] = { 0x00, 0x00, 0x00, 0x00 };
-
-	if (overwrite_section(image, FMAP_SI_DESC, gpr0_offset,
-			      ARRAY_SIZE(gpr0_value_disabled),
-			      gpr0_value_disabled)) {
-		ERROR("Failed disabling GPR0.\n");
-		return -1;
-	}
-
-	INFO("Disabled GPR0.\n");
-	return 0;
-}
-
-/*
- * Unlock the Intel ME by:
- * - Unlocking the FLMSTR values in the descriptor.
- * - Disabling GPR0 in the descriptor.
- * This allows the SI_DESC and SI_ME regions to be updated.
- * Returns 0 on success, otherwise failure.
- */
-int unlock_me(struct firmware_image *image)
-{
-	if (unlock_flash_master(image))
-		return -1;
-
-	if (disable_gpr0(image))
-		return -1;
-
-	INFO("Unlocked Intel ME.\n");
-	return 0;
-}
