@@ -30,9 +30,8 @@
 /* Filename for kernel command line */
 #define KERNEL_CMDLINE_PATH "/proc/cmdline"
 
-/* Filename for the TPM simulator NV data */
-#define TPM_SIMULATOR_NVCHIP_PATH \
-	"/mnt/stateful_partition/unencrypted/tpm2-simulator/NVChip"
+/* Filename for the tpm_clear_request executable. */
+#define TPM_CLEAR_REQUEST_EXEC_NAME "/usr/sbin/tpm_clear_request"
 
 /* Fields that GetVdatString() can get */
 typedef enum VdatStringField {
@@ -54,11 +53,12 @@ typedef enum VdatIntField {
 
 	VDAT_INT_FW_VERSION_TPM,      /* Current firmware version in TPM */
 	VDAT_INT_KERNEL_VERSION_TPM,  /* Current kernel version in TPM */
-	VDAT_INT_TRIED_FIRMWARE_B,    /* Tried firmware B due to fwb_tries */
 	VDAT_INT_KERNEL_KEY_VERIFIED, /* Kernel key verified using
 				       * signature, not just hash */
 	VDAT_INT_RECOVERY_REASON,     /* Recovery reason for current boot */
-	VDAT_INT_FW_BOOT2             /* Firmware selection by vboot2 */
+	VDAT_INT_FW_BOOT2,            /* Firmware selection by vboot2 */
+	VDAT_INT_FW_VERSION_ACT,      /* Current active firmware version */
+	VDAT_INT_KERNEL_VERSION_ACT,  /* Current active kernel version */
 } VdatIntField;
 
 
@@ -187,6 +187,7 @@ int vb2_set_nv_storage(enum vb2_nv_param param, int value)
 			free(sh);
 			return -1;
 		}
+		ctx->flags &= ~VB2_CONTEXT_NVDATA_CHANGED;
 	}
 
 	/* Success */
@@ -269,12 +270,14 @@ static int GetVdatLoadFirmwareDebug(char *dest, int size,
 		 "Check A result=%d\n"
 		 "Check B result=%d\n"
 		 "Firmware index booted=0x%02x\n"
-		 "TPM combined version at start=0x%08x\n"
+		 "Active firmware version=0x%08x\n"
+		 "Firmware version in TPM =0x%08x\n"
 		 "Lowest combined version from firmware=0x%08x\n",
 		 sh->check_fw_a_result,
 		 sh->check_fw_b_result,
 		 sh->firmware_index,
-		 sh->fw_version_tpm_start,
+		 sh->fw_version_act,
+		 sh->fw_version_tpm,
 		 sh->fw_version_lowest);
 	return 0;
 }
@@ -317,6 +320,23 @@ static int GetVdatString(char *dest, int size, VdatStringField field)
 	return value;
 }
 
+static int FwidMajorVersion(void)
+{
+	char fwid[VB_MAX_STRING_PROPERTY];
+	int version;
+
+	if (VbGetSystemPropertyString("fwid", fwid, sizeof(fwid)) != 0)
+		return -1;
+
+	if (sscanf(fwid, "%*[^.].%d", &version) != 1 || version <= 0) {
+		fprintf(stderr, "WARNING: Cannot parse major version from %s\n",
+			fwid);
+		return -1;
+	}
+
+	return version;
+}
+
 static int GetVdatInt(VdatIntField field)
 {
 	VbSharedDataHeader* sh = VbSharedDataRead();
@@ -333,21 +353,26 @@ static int GetVdatInt(VdatIntField field)
 		case VDAT_INT_HEADER_VERSION:
 			value = sh->struct_version;
 			break;
-		case VDAT_INT_TRIED_FIRMWARE_B:
-			value = (sh->flags & VBSD_FWB_TRIED ? 1 : 0);
-			break;
 		case VDAT_INT_KERNEL_KEY_VERIFIED:
 			value = (sh->flags & VBSD_KERNEL_KEY_VERIFIED ? 1 : 0);
 			break;
 		case VDAT_INT_FW_VERSION_TPM:
-			value = (int)sh->fw_version_tpm;
+			/* b/269204332#comment5: Before CL:2054270 and CL:2056343,
+			   fw_version_tpm was always 0. */
+			if (sh->struct_version <= 2 && FwidMajorVersion() < 12935)
+				value = (int)sh->fw_version_act;
+			else
+				value = (int)sh->fw_version_tpm;
 			break;
 		case VDAT_INT_KERNEL_VERSION_TPM:
 			value = (int)sh->kernel_version_tpm;
 			break;
 		case VDAT_INT_FW_BOOT2:
 			value = (sh->flags & VBSD_BOOT_FIRMWARE_VBOOT2 ? 1 : 0);
-			VBOOT_FALLTHROUGH;
+			break;
+		case VDAT_INT_FW_VERSION_ACT:
+			value = (int)sh->fw_version_act;
+			break;
 		default:
 			break;
 	}
@@ -375,6 +400,17 @@ static int GetVdatInt(VdatIntField field)
 		}
 	}
 
+	/* Fields added in struct version 3 */
+	if (sh->struct_version >= 3) {
+		switch(field) {
+			case VDAT_INT_KERNEL_VERSION_ACT:
+				value = (int)sh->kernel_version_act;
+				break;
+			default:
+				break;
+		}
+	}
+
 	free(sh);
 	return value;
 }
@@ -408,18 +444,21 @@ int VbGetSystemPropertyInt(const char *name)
 	} else if (!strcasecmp(name,"disable_dev_request")) {
 		value = vb2_get_nv_storage(VB2_NV_DISABLE_DEV_REQUEST);
 	} else if (!strcasecmp(name,"clear_tpm_owner_request")) {
-		if (TPM2_SIMULATOR)
-			/* Check TPM simulator NVChip status */
-			value = access(TPM_SIMULATOR_NVCHIP_PATH, F_OK) != 0;
-		else
-			value = vb2_get_nv_storage(
-				VB2_NV_CLEAR_TPM_OWNER_REQUEST);
+		if (EXTERNAL_TPM_CLEAR_REQUEST) {
+			const char *const argv[] = {
+				TPM_CLEAR_REQUEST_EXEC_NAME,
+				NULL,
+			};
+			value = subprocess_run(argv, &subprocess_null, &subprocess_null,
+					       &subprocess_null);
+		} else {
+			value = vb2_get_nv_storage(VB2_NV_CLEAR_TPM_OWNER_REQUEST);
+		}
 	} else if (!strcasecmp(name,"clear_tpm_owner_done")) {
 		value = vb2_get_nv_storage(VB2_NV_CLEAR_TPM_OWNER_DONE);
 	} else if (!strcasecmp(name,"tpm_rebooted")) {
 		value = vb2_get_nv_storage(VB2_NV_TPM_REQUESTED_REBOOT);
-	} else if (!strcasecmp(name,"fwb_tries") ||
-		   !strcasecmp(name,"fw_try_count")) {
+	} else if (!strcasecmp(name,"fw_try_count")) {
 		value = vb2_get_nv_storage(VB2_NV_TRY_COUNT);
 	} else if (!strcasecmp(name,"fw_vboot2")) {
 		value = GetVdatInt(VDAT_INT_FW_BOOT2);
@@ -482,8 +521,10 @@ int VbGetSystemPropertyInt(const char *name)
 		value = GetVdatInt(VDAT_INT_FW_VERSION_TPM);
 	} else if (!strcasecmp(name,"tpm_kernver")) {
 		value = GetVdatInt(VDAT_INT_KERNEL_VERSION_TPM);
-	} else if (!strcasecmp(name,"tried_fwb")) {
-		value = GetVdatInt(VDAT_INT_TRIED_FIRMWARE_B);
+	} else if (!strcasecmp(name,"act_fwver")) {
+		value = GetVdatInt(VDAT_INT_FW_VERSION_ACT);
+	} else if (!strcasecmp(name,"act_kernver")) {
+		value = GetVdatInt(VDAT_INT_KERNEL_VERSION_ACT);
 	} else if (!strcasecmp(name,"recovery_reason")) {
 		value = GetVdatInt(VDAT_INT_RECOVERY_REASON);
 	} else if (!strcasecmp(name, "boot_on_ac_detect")) {
@@ -625,20 +666,14 @@ static int VbSetSystemPropertyIntInternal(const char *name, int value)
 	} else if (!strcasecmp(name,"disable_dev_request")) {
 		return vb2_set_nv_storage(VB2_NV_DISABLE_DEV_REQUEST, value);
 	} else if (!strcasecmp(name,"clear_tpm_owner_request")) {
-		if (TPM2_SIMULATOR) {
-			/* We don't support to set clear_tpm_owner_request to 0
-			 * on simulator */
-			if (value == 0)
-				return -1;
-			/* Check TPM simulator data status */
-			if (!access(TPM_SIMULATOR_NVCHIP_PATH, F_OK)) {
-				/* Remove the TPM2.0 simulator data */
-				return remove(TPM_SIMULATOR_NVCHIP_PATH);
-			} else {
-				/* Return success when the file is already
-				 * removed */
-				return 0;
-			}
+		if (EXTERNAL_TPM_CLEAR_REQUEST) {
+			const char *const argv[] = {
+				TPM_CLEAR_REQUEST_EXEC_NAME,
+				value ? "1" : "0",
+				NULL,
+			};
+			return subprocess_run(argv, &subprocess_null, &subprocess_null,
+					      &subprocess_null);
 		} else {
 			return vb2_set_nv_storage(
 				VB2_NV_CLEAR_TPM_OWNER_REQUEST, value);
@@ -646,8 +681,7 @@ static int VbSetSystemPropertyIntInternal(const char *name, int value)
 	} else if (!strcasecmp(name,"clear_tpm_owner_done")) {
 		/* Can only clear this flag; it's set by firmware. */
 		return vb2_set_nv_storage(VB2_NV_CLEAR_TPM_OWNER_DONE, 0);
-	} else if (!strcasecmp(name,"fwb_tries") ||
-		   !strcasecmp(name,"fw_try_count")) {
+	} else if (!strcasecmp(name,"fw_try_count")) {
 		return vb2_set_nv_storage(VB2_NV_TRY_COUNT, value);
 	} else if (!strcasecmp(name,"display_request")) {
 		return vb2_set_nv_storage(VB2_NV_DISPLAY_REQUEST, value);
@@ -807,18 +841,20 @@ int VbSetSystemPropertyString(const char *name, const char *value)
 }
 
 /**
- * Get index of the last valid VBNV entry in an EEPROM.
+ * Get index of the last valid VBNV entry.
  *
- * @param buf		Pointer to the beginning of the EEPROM.
- * @param buf_sz	Size of the EEPROM.
+ * @param buf		Pointer to the buffer containing VBNV entries.
+ * @param buf_sz	Size of the buffer.
  * @param vbnv_size	The size of a single VBNV entry for this device.
  *
- * @return The index of the last valid VBNV entry on success, or -1 on
- * failure.
+ * @return The index of the last valid VBNV entry found by binary search,
+ * or -1 if not found. When the FMAP region is corrupted (used entries occurring
+ * after blank ones), the returned index may not point to the last VBNV
+ * entry.
  */
 static int vb2_nv_index(const uint8_t *buf, uint32_t buf_sz, int vbnv_size)
 {
-	int index;
+	int used_below, blank_above;
 	uint8_t blank[VB2_NVDATA_SIZE_V2];
 
 	/* The size of the buffer should be an even multiple of the
@@ -830,17 +866,54 @@ static int vb2_nv_index(const uint8_t *buf, uint32_t buf_sz, int vbnv_size)
 	}
 
 	memset(blank, 0xff, sizeof(blank));
-	for (index = 0; index < buf_sz / vbnv_size; index++) {
-		if (!memcmp(blank, &buf[index * vbnv_size], vbnv_size))
-			break;
+
+	/* To match the searching algorithm in firmware, perform binary search
+	   instead of linear search to find the last used index. */
+	used_below = 0;
+	blank_above = buf_sz / vbnv_size;
+	while (used_below + 1 < blank_above) {
+		int mid = (used_below + blank_above) / 2;
+		if (!memcmp(blank, &buf[mid * vbnv_size], vbnv_size))
+			blank_above = mid;
+		else
+			used_below = mid;
 	}
 
-	if (!index) {
+	/* Check the all blank case. */
+	if (used_below == 0 &&
+	    !memcmp(blank, &buf[used_below * vbnv_size], vbnv_size)) {
 		fprintf(stderr, "VBNV is uninitialized.\n");
 		return -1;
 	}
 
-	return index - 1;
+	return used_below;
+}
+
+/**
+ * Check whether the VBNV entries are corrupted.
+ *
+ * @param buf		Pointer to the buffer containing VBNV entries.
+ * @param buf_sz	Size of the buffer.
+ * @param vbnv_size	The size of a single VBNV entry for this device.
+ *
+ * @return True if there are used entries occurring after blank ones, or false
+ * otherwise.
+ */
+static bool is_corrupted(const uint8_t *buf, uint32_t buf_sz, int vbnv_size)
+{
+	uint8_t blank[VB2_NVDATA_SIZE_V2];
+	bool found_blank = false;
+
+	memset(blank, 0xff, sizeof(blank));
+
+	for (int i = 0; i < buf_sz / vbnv_size; i++) {
+		if (!memcmp(blank, &buf[i * vbnv_size], vbnv_size))
+			found_blank = true;
+		else if (found_blank)
+			return true;
+	}
+
+	return false;
 }
 
 #define VBNV_FMAP_REGION "RW_NVRAM"
@@ -870,8 +943,8 @@ int vb2_read_nv_storage_flashrom(struct vb2_context *ctx)
 int vb2_write_nv_storage_flashrom(struct vb2_context *ctx)
 {
 	int rv = 0;
-	int current_index;
-	int next_index;
+	int index;
+	bool corrupted;
 	int vbnv_size = vb2_nv_get_size(ctx);
 
 	struct firmware_image image = {
@@ -880,20 +953,19 @@ int vb2_write_nv_storage_flashrom(struct vb2_context *ctx)
 	if (flashrom_read(&image, VBNV_FMAP_REGION))
 		return -1;
 
-	current_index = vb2_nv_index(image.data, image.size, vbnv_size);
-	if (current_index < 0) {
-		rv = -1;
-		goto exit;
-	}
+	index = vb2_nv_index(image.data, image.size, vbnv_size) + 1;
+	corrupted = is_corrupted(image.data, image.size, vbnv_size);
 
-	next_index = current_index + 1;
-	if (next_index * vbnv_size == image.size) {
-		/* VBNV is full.  Erase and write at beginning. */
+	if (corrupted || index * vbnv_size == image.size) {
+		/* VBNV is corrupted or full.  Erase and write at beginning. */
+		if (corrupted)
+			fprintf(stderr, "VBNV is corrupted; erasing %s\n",
+				VBNV_FMAP_REGION);
 		memset(image.data, 0xff, image.size);
-		next_index = 0;
+		index = 0;
 	}
 
-	memcpy(&image.data[next_index * vbnv_size], ctx->nvdata, vbnv_size);
+	memcpy(&image.data[index * vbnv_size], ctx->nvdata, vbnv_size);
 	if (flashrom_write(&image, VBNV_FMAP_REGION)) {
 		rv = -1;
 		goto exit;
