@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (c) 2012 The Chromium OS Authors. All rights reserved.
+# Copyright 2012 The ChromiumOS Authors
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 #
@@ -61,6 +61,15 @@ DEFINE_boolean force "$FLAGS_FALSE" \
   "Skip validity checks and make the change" "f"
 DEFINE_boolean default_rw_root "${FLAGS_TRUE}" \
   "When --remove_rootfs_verification is set, change root mount option to RW." ""
+DEFINE_boolean enable_proc_mem_ptrace "${FLAGS_TRUE}" \
+  "When --remove_rootfs_verification is set, allow ptracers to write proc/mem" \
+  ""
+# If neither --enable_kdump nor --disable_kdump was provided, the enablement
+# status won't be changed.
+DEFINE_boolean enable_kdump "${FLAGS_FALSE}" \
+  "Enable Kdump." ""
+DEFINE_boolean disable_kdump "${FLAGS_FALSE}" \
+  "Disable Kdump." ""
 
 # Parse command line
 FLAGS "$@" || exit 1
@@ -90,7 +99,14 @@ remove_rootfs_verification() {
     rw_root_opt="s| rw | ro |"
   fi
 
-  echo "$*" | sed '
+  local ptracer_opt="proc_mem.restrict_write=ptracer proc_mem.restrict_foll_force=ptracer"
+  if [ "${FLAGS_enable_proc_mem_ptrace}" = "${FLAGS_FALSE}" ]; then
+    # we could set proc_mem.restrict_write=all, however that's already default
+    # via Kconfig and we don't want to clutter the cmdline with redundant params
+    ptracer_opt=""
+  fi
+
+  echo "$* chromiumos.allow_overlayfs ${ptracer_opt}" | sed '
     s| dm=\"2 [^"]*bootcache[^"]* vroot | dm=\"1 vroot |
     s| root=/dev/dm-[0-9] | root='"$new_root"' |
     s| dm_verity.dev_wait=1 | dm_verity.dev_wait=0 |
@@ -132,8 +148,8 @@ remove_parameter() {
   local cmdline="$1"
   local param="$2"
 
-  cmdline=$(echo "${cmdline}" | sed '
-    s/'"${param} "'//g')
+  cmdline=$(echo "${cmdline}" | \
+    sed -E 's/(^|\s)\s*'"${param}"'(=\S*|="(\\"|[^"])*")?\s*/\1/g');
 
   echo "${cmdline}"
 }
@@ -169,7 +185,7 @@ find_valid_kernel_partitions() {
   for part_id in $*; do
     local name="$(cros_kernel_name $part_id)"
     local kernel_part="$(make_partition_dev "$FLAGS_image" "$part_id")"
-    if [ -z "$(dump_kernel_config "$kernel_part" 2>"$EXEC_LOG")" ]; then
+    if [ -z "$(futility dump_kernel_config "$kernel_part" 2>"$EXEC_LOG")" ]; then
       info "${name}: no kernel boot information, ignored." >&2
     else
       [ -z "$valid_partitions" ] &&
@@ -213,7 +229,7 @@ resign_ssd_kernel() {
 
     debug_msg "Checking if $name is valid"
     local kernel_config
-    if ! kernel_config="$(dump_kernel_config "$old_blob" 2>"$EXEC_LOG")"; then
+    if ! kernel_config="$(futility dump_kernel_config "$old_blob" 2>"$EXEC_LOG")"; then
       debug_msg "dump_kernel_config error message: $(cat "$EXEC_LOG")"
       info "${name}: no kernel boot information, ignored."
       continue
@@ -244,19 +260,7 @@ resign_ssd_kernel() {
       local new_config_file="$(make_temp_file)"
       echo "${kernel_config}" >"${new_config_file}"
       local old_md5sum="$(md5sum "${new_config_file}")"
-      local editor="${VISUAL:-${EDITOR:-vi}}"
-      info "${name}: Editing kernel config:"
-      # On ChromiumOS, some builds may come with broken EDITOR that refers to
-      # nano so we want to check again if the editor really exists.
-      if type "${editor}" >/dev/null 2>&1; then
-        "${editor}" "${new_config_file}"
-      else
-        # This script runs under dash but we want readline in bash to support
-        # editing in in console.
-        bash -c "read -e -i '${kernel_config}' &&
-                 echo \"\${REPLY}\" >${new_config_file}" ||
-          die "Failed to run editor. Please specify editor name by VISUAL."
-      fi
+      "${EDITOR}" "${new_config_file}"
       kernel_config="$(cat "${new_config_file}")"
       if [ "$(md5sum "${new_config_file}")" = "${old_md5sum}" ]; then
         info "${name}: Config not changed."
@@ -288,7 +292,7 @@ resign_ssd_kernel() {
 
     if [ "${FLAGS_enable_console}" = "${FLAGS_TRUE}" ]; then
       debug_msg "Enabling serial console"
-      kernel_config="$(remove_parameter "${kernel_config}" "console=")"
+      kernel_config="$(remove_parameter "${kernel_config}" "console")"
       debug_msg "New kernel config: ${kernel_config}"
     elif [ "${FLAGS_disable_console}" = "${FLAGS_TRUE}" ]; then
       debug_msg "Disabling serial console"
@@ -296,12 +300,23 @@ resign_ssd_kernel() {
       debug_msg "New kernel config: ${kernel_config}"
     fi
 
-    local new_kernel_config_file="$(make_temp_file)"
-    echo -n "$kernel_config"  >"$new_kernel_config_file"
+    if [ "${FLAGS_enable_kdump}" = "${FLAGS_TRUE}" ]; then
+      debug_msg "Enabling kdump"
+      kernel_config="$(insert_parameter "${kernel_config}" "crashkernel=256M")"
+      debug_msg "New kernel config: ${kernel_config}"
+    elif [ "${FLAGS_disable_kdump}" = "${FLAGS_TRUE}" ]; then
+      debug_msg "Disabling kdump"
+      kernel_config="$(remove_parameter "${kernel_config}" "crashkernel")"
+      debug_msg "New kernel config: ${kernel_config}"
+    fi
+
+    local new_kernel_config_file
+    new_kernel_config_file="$(make_temp_file)"
+    printf %s "${kernel_config}" >"${new_kernel_config_file}"
 
     debug_msg "Re-signing $name from $old_blob to $new_blob"
     debug_msg "Using key: $KERNEL_DATAKEY"
-    vbutil_kernel \
+    futility vbutil_kernel \
       --repack "$new_blob" \
       --keyblock "$KERNEL_KEYBLOCK" \
       --config "$new_kernel_config_file" \
@@ -322,7 +337,7 @@ resign_ssd_kernel() {
     fi
 
     debug_msg "Verifying new kernel and keys"
-    vbutil_kernel \
+    futility vbutil_kernel \
       --verify "$new_kern" \
       --signpubkey "$KERNEL_PUBKEY" --verbose >"$EXEC_LOG" 2>&1 ||
       die "Failed to verify new ${name}. Message: $(cat "${EXEC_LOG}")"
@@ -406,7 +421,7 @@ validity_check_crossystem_flags() {
   you really want to make this change, allow the firmware to boot self-signed
   images by running:
 
-    sudo crossystem dev_boot_signed_only=0
+    sudo -E crossystem dev_boot_signed_only=0
 
   before re-executing this command.
   "
@@ -432,7 +447,7 @@ validity_check_live_partitions() {
   change the partition you have booted with. To do that, re-execute this command
   as:
 
-    sudo $ORIGINAL_CMD $ORIGINAL_PARAMS --partitions $ROOTDEV_KERNEL
+    sudo -E $ORIGINAL_CMD $ORIGINAL_PARAMS --partitions $ROOTDEV_KERNEL
 
   If you are sure to modify other partition, please invoke the command again and
   explicitly assign only one target partition for each time  (--partitions N )
@@ -447,11 +462,9 @@ validity_check_live_firmware() {
     return
   fi
   debug_msg "Loading firmware to check root key..."
-  local bios_image="$(make_temp_file)"
   local rootkey_file="$(make_temp_file)"
   info "checking system firmware..."
-  sudo flashrom -p host -i GBB -r "$bios_image" >/dev/null 2>&1
-  futility gbb -g --rootkey="$rootkey_file" "$bios_image" >/dev/null 2>&1
+  futility gbb -g --flash --rootkey="$rootkey_file" >/dev/null 2>&1
   if [ ! -s "$rootkey_file" ]; then
     debug_msg "failed to read root key from system firmware..."
   else
@@ -477,17 +490,17 @@ validity_check_live_firmware() {
   You need to either install developer firmware, or change system root key.
 
    - To install developer firmware: type command
-     sudo chromeos-firmwareupdate --mode=todev
+     sudo -E chromeos-firmwareupdate --mode=todev
 
    - To change system rootkey: disable firmware write protection (a hardware
      switch) and then type command:
-     sudo $SCRIPT_BASE/make_dev_firmware.sh
+     sudo -E $SCRIPT_BASE/make_dev_firmware.sh
 
   If you are sure that you want to make such image without developer
   firmware or you've already changed system root keys, please run this
   command again with --force paramemeter:
 
-     sudo $ORIGINAL_CMD --force $ORIGINAL_PARAMS
+     sudo -E $ORIGINAL_CMD --force $ORIGINAL_PARAMS
   "
   return $FLAGS_FALSE
 }
@@ -500,6 +513,10 @@ validity_check() {
     debug_msg "Device is a VM, skipping firmware checks"
     return $FLAGS_TRUE
   fi
+  if crossystem 'mainfw_type?nonchrome'; then
+    debug_msg "Device is nonchrome, skipping firmware checks"
+    return $FLAGS_TRUE
+  fi
 
   validity_check_live_firmware || return $FLAGS_FALSE
   validity_check_crossystem_flags || return $FLAGS_FALSE
@@ -531,7 +548,12 @@ main() {
     exit 1
 
   # checks for running on a live system image.
-  if [ "$FLAGS_image" = "$ROOTDEV_DISK" ]; then
+  if [ "${FLAGS_image}" = "${ROOTDEV_DISK}" ]; then
+    # Log the operartions and the parameters passed to the script
+    # when the device is changing itself
+    logger -p NOTICE "executing ${ORIGINAL_CMD}" \
+                     "with the following parameters: ${ORIGINAL_PARAMS}"
+
     debug_msg "check valid kernel partitions for live system"
     local valid_partitions="$(find_valid_kernel_partitions $FLAGS_partitions)"
     [ -n "$valid_partitions" ] ||
