@@ -7,12 +7,13 @@
 . "$(dirname "$0")/common.sh"
 . "$(dirname "$0")/lib/sign_android_lib.sh"
 load_shflags || exit 1
+SCRIPT_DIR="${SCRIPT_DIR:-$(dirname "$0")}"
 
 DEFINE_boolean use_apksigner "${FLAGS_FALSE}" \
   "Use apksigner instead of signapk for APK signing"
 
 FLAGS_HELP="
-Usage: $PROG /path/to/cros_root_fs/dir /path/to/keys/dir
+Usage: $PROG /path/to/cros_root_fs/dir /path/to/keys/dir [--use_apksigner]
 
 Re-sign framework apks in an Android system image.  The image itself does not
 need to be signed since it is shipped with Chrome OS image, which is already
@@ -23,6 +24,7 @@ keys, depends on the purpose of the apk.  During development, apks are signed
 with the debug one.  This script is to re-sign those apks with corresponding
 release key.  It also handles some of the consequences of the key changes, such
 as sepolicy update.
+It can also sign using cloud kms keys based on env variables present for it.
 "
 
 # Parse command line.
@@ -41,6 +43,26 @@ sign_framework_apks() {
   local flavor_prop=""
   local keyset=""
 
+  # Generate key config directory.
+  local gen_key_config_dir
+  gen_key_config_dir="$(make_temp_dir)"
+  local gcloud_provider_class="sun.security.pkcs11.SunPKCS11"
+
+  if [[ -n ${KEYCFG_ANDROID_CLOUD_KEY_PREFIX} ]]; then
+    info "Using cloud signing as cloud key prefix is present."
+
+    if [[ "${FLAGS_use_apksigner}" == "${FLAGS_FALSE}" ]]; then
+      die "use_apksigner flag is required when using gcloud to sign."
+    fi
+
+    # Generate config file needed for gcloud to run the KMS signing.
+    if ! "${SCRIPT_DIR}/lib/generate_android_cloud_config.py" \
+      -o="${gen_key_config_dir}/";
+    then
+      die "Unable to generate config for cloud signing, exiting."
+    fi
+  fi
+
   if ! flavor_prop=$(android_get_build_flavor_prop \
     "${system_mnt}/system/build.prop"); then
     die "Failed to extract build flavor property from \
@@ -52,7 +74,11 @@ sign_framework_apks() {
   fi
   info "Expecting signing keyset '${keyset}'."
 
-  info "Start signing framework apks"
+  if [[ "${FLAGS_use_apksigner}" == "${FLAGS_FALSE}" ]]; then
+    info "Using signapk to sign the Framework apk's."
+  else
+    info "Using apksigner to sign the Framework apk's."
+  fi
 
   if ! image_content_integrity_check "${system_mnt}" "${working_dir}" \
                                      "Prepare apks signing"; then
@@ -109,18 +135,40 @@ build flavor '${flavor_prop}'."
       # apksigner rotate --out media.lineage --old-signer --key old-media.pk8
       # --cert old-media.x509.pem --new-signer --key new-media.pk8 --cert
       # new-media.x509.pem
-      #
-      # TODO(b/132818552): disable v1 signing once a check is removed.
 
       local extra_flags
-      local lineage_file="${key_dir}/$keyname.lineage}"
-      if [ -f ${lineage_file} ]; then
+      local lineage_file="${key_dir}/${keyname}.lineage"
+      local apksigner_min_sdk_version=28
+      local temp_zipaligned_apk="${temp_dir}/temp_zipaligned.apk"
+
+      # If using apksigner zipalign needs to be done before signing.
+      if ! zipalign -p -f 4 "${temp_apk}" "${temp_zipaligned_apk}"; then
+        die "Zipalign failed to align the apk."
+      fi
+
+      if [[ -f ${lineage_file} ]]; then
         extra_flags="--lineage ${lineage_file}"
       fi
-      apksigner sign --v1-signing-enabled true --v2-signing-enabled false \
-        --key "${key_dir}/$keyname.pk8" --cert "${key_dir}/$keyname.x509.pem" \
-        --in "${temp_apk}" --out "${signed_apk}" \
-        ${extra_flags}
+
+      if [[ -n ${KEYCFG_ANDROID_CLOUD_KEY_PREFIX} ]]; then
+        KMS_PKCS11_CONFIG="${key_dir}/${keyname}_config.yaml" \
+          apksigner sign --v3-signing-enabled true \
+          --min-sdk-version="${apksigner_min_sdk_version}" \
+          --provider-class "${gcloud_provider_class}" \
+          --provider-arg "${gen_key_config_dir}/pkcs11_java.cfg" --ks "NONE" \
+          --ks-type "PKCS11" \
+          --ks-key-alias "${KEYCFG_ANDROID_CLOUD_KEY_PREFIX}${keyname}" \
+          --ks-pass pass:\"\" \
+          --in "${temp_zipaligned_apk}" --out "${signed_apk}"  \
+          ${extra_flags}
+      else
+        # b/349826228: explicitly disabling v1/v2 signing due to lineage error
+        apksigner sign --key "${key_dir}/${keyname}.pk8" \
+          --cert "${key_dir}/${keyname}.x509.pem" \
+          --v1-signing-enabled false --v2-signing-enabled false \
+          --in "${temp_zipaligned_apk}" --out "${signed_apk}" \
+          ${extra_flags}
+      fi
     fi
     if ! image_content_integrity_check "${system_mnt}" "${working_dir}" \
                                        "sign apk ${signed_apk}"; then
@@ -128,6 +176,7 @@ build flavor '${flavor_prop}'."
     fi
 
     # Copy the content instead of mv to avoid owner/mode changes.
+    #TODO(b/331944273): Fail when copy of apk fails.
     sudo cp "${signed_apk}" "${apk}" && rm -f "${signed_apk}"
 
     # Set timestamp rounded to second since squash file system has resolution
@@ -141,7 +190,7 @@ build flavor '${flavor_prop}'."
                                        "update re-signed apk ${apk}"; then
       return 1
     fi
-  done < <(find "${system_mnt}/system" -type f -name '*.apk' -print0)
+  done < <(sudo find "${system_mnt}/system" -type f -name '*.apk' -print0)
 
   info "Found ${counter_platform} platform APKs."
   info "Found ${counter_media} media APKs."
@@ -173,6 +222,20 @@ get_cert() {
   echo "${cert}"
 }
 
+# Extract certificate from the provided certificate yaml file.
+get_cert_from_yaml() {
+  # Full path to the yaml file. It must exist, and must have exactly 1 cert.
+  local public_key=$1
+  local cert
+  cert=$(awk '/(BEGIN CERTIFICATE)/,/(END CERTIFICATE)/' \
+    "${public_key}" | sed -E '/(BEGIN|END) CERTIFICATE/d' | \
+    tr -d ' ' | tr -d '\n' | base64 --decode | hexdump -v -e '/1 "%02x"')
+  if [[ -z "${cert}" ]]; then
+    die "Unable to get the public platform key"
+  fi
+  echo "${cert}"
+}
+
 # Replaces particular certificate in mac_permissions xml file with new one.
 # Note, this does not fail if particular entry is not found. For example
 # network_stack does not exist in P.
@@ -193,16 +256,29 @@ update_sepolicy() {
   local system_mnt=$1
   local key_dir=$2
 
-  # Only platform is used at this time.
-  local public_platform_key="${key_dir}/platform.x509.pem"
-  local public_media_key="${key_dir}/media.x509.pem"
-  local public_network_stack_key="${key_dir}/networkstack.x509.pem"
-
   info "Start updating sepolicy"
 
-  local new_platform_cert=$(get_cert "${public_platform_key}")
-  local new_media_cert=$(get_cert "${public_media_key}")
-  local new_network_stack_cert=$(get_cert "${public_network_stack_key}")
+  local new_platform_cert
+  local new_media_cert
+  local new_network_stack_cert
+
+  if [[ -n ${KEYCFG_ANDROID_CLOUD_KEY_PREFIX} ]]; then
+    local platform_cert_yaml="${key_dir}/platform_config.yaml"
+    local media_cert_yaml="${key_dir}/media_config.yaml"
+    local network_cert_yaml="${key_dir}/networkstack_config.yaml"
+
+    new_platform_cert=$(get_cert_from_yaml "${platform_cert_yaml}")
+    new_media_cert=$(get_cert_from_yaml "${media_cert_yaml}")
+    new_network_stack_cert=$(get_cert_from_yaml "${network_cert_yaml}")
+  else
+    local public_platform_key="${key_dir}/platform.x509.pem"
+    local public_media_key="${key_dir}/media.x509.pem"
+    local public_network_stack_key="${key_dir}/networkstack.x509.pem"
+
+    new_platform_cert=$(get_cert "${public_platform_key}")
+    new_media_cert=$(get_cert "${public_media_key}")
+    new_network_stack_cert=$(get_cert "${public_network_stack_key}")
+  fi
 
   shopt -s nullglob
   local xml_list=( "${system_mnt}"/system/etc/**/*mac_permissions.xml )
@@ -226,6 +302,34 @@ update_sepolicy() {
   fi
 }
 
+replace_ota_cert_cloud() {
+  local system_mnt=$1
+  local key_dir=$2
+  local keyname=$3
+
+  info "Replacing OTA cert for cloud"
+
+  local temp_cert
+  temp_cert=$(make_temp_file)
+  local cert
+  cert=$(awk '/(BEGIN CERTIFICATE)/,/(END CERTIFICATE)/' \
+    "${key_dir}/${keyname}_config.yaml" | sed 's/^\s*//')
+  echo "${cert}" > "${temp_cert}"
+
+  local ota_zip="${system_mnt}/system/etc/security/otacerts.zip"
+
+  local temp_dir
+  temp_dir=$(make_temp_dir)
+  pushd "${temp_dir}" > /dev/null
+  cp "${temp_cert}" "${keyname}.x509.pem"
+  local temp_zip
+  temp_zip=$(make_temp_file)
+  zip -q -r "${temp_zip}.zip" .
+  # Copy the content instead of mv to avoid owner/mode changes.
+  sudo cp "${temp_zip}.zip" "${ota_zip}"
+  popd > /dev/null
+}
+
 # Replace the debug key in OTA cert with release key.
 replace_ota_cert() {
   local system_mnt=$1
@@ -353,9 +457,9 @@ sign_android_internal() {
   fsck_erofs=$(which fsck.erofs)
   mkfs_erofs=$(which mkfs.erofs)
 
-  if [[ $# -ne 2 ]]; then
+  if [[ $# -lt 2 || $# -gt 3 ]]; then
     flags_help
-    die "command takes exactly 2 args"
+    die "command requires 2 input args and can take 1 optional arguments."
   fi
 
   if [[ ! -f "${system_img}" ]]; then
@@ -375,28 +479,45 @@ sign_android_internal() {
   fi
   info "Detected ARC system image type: ${image_type}"
 
-  local squashfs_comp_flags=()
-  if [[ "${image_type}" == "squashfs" ]]; then
-    # NOTE: Keep squashfs_comp_flags aligned with
-    # project-cheets-private/scripts/board_specific_setup.sh
-    local compression
-    compression="$("${unsquashfs}" -s "${system_img}" \
-      | awk '$1 == "Compression" {print $2}')"
-    case "${compression}" in
-      "gzip")
-        squashfs_comp_flags=(-comp gzip)
-        ;;
-      "lz4")
-        squashfs_comp_flags=(-comp lz4 -Xhc -b 256K)
-        ;;
-      "zstd")
-        squashfs_comp_flags=(-comp zstd -b 256K)
-        ;;
-      *)
-        die "Unexpected compression type: ${compression}"
-        ;;
-    esac
-    info "Detected squashfs compression type: ${compression}"
+  # Detect image compression flags.
+  local compression_flags=()
+  local compression_flags_path="${android_dir}/image_compression_flags"
+  if [[ -f "${compression_flags_path}" ]]; then
+    # The file contains a line such as:
+    # mkfs.erofs -z lz4hc -C32768
+    read -r -a tokens <"${compression_flags_path}"
+    if [[ ("${image_type}" == "squashfs" && "${tokens[0]}" != "mksquashfs") ||
+          ("${image_type}" == "erofs" && "${tokens[0]}" != "mkfs.erofs") ]]; then
+      die "Compression tool '${tokens[0]}' found in image_compression_flags" \
+          "doesn't match detected image type '${image_type}'"
+    fi
+    compression_flags=("${tokens[@]:1}")
+    info "Detected compression flags: ${compression_flags[*]}"
+  else
+    if [[ "${image_type}" == "erofs" ]]; then
+      compression_flags=(-z lz4hc -C32768)
+    elif [[ "${image_type}" == "squashfs" ]]; then
+      local compression
+      compression="$("${unsquashfs}" -s "${system_img}" \
+        | awk '$1 == "Compression" {print $2}')"
+      case "${compression}" in
+        "gzip")
+          compression_flags=(-comp gzip)
+          ;;
+        "lz4")
+          compression_flags=(-comp lz4 -Xhc -b 256K)
+          ;;
+        "zstd")
+          compression_flags=(-comp zstd -b 256K)
+          ;;
+        *)
+          die "Unexpected compression type: ${compression}"
+          ;;
+      esac
+    fi
+    info "image_compression_flags does not exist." \
+         "(This is expected for versions older than R121-15679.)" \
+         "Using default compression flags: ${compression_flags[*]}"
   fi
 
   if ! type -P zipalign &>/dev/null || ! type -P signapk &>/dev/null \
@@ -445,6 +566,14 @@ sign_android_internal() {
     sudo find "${system_mnt}" > "${working_dir}/image_file_list.orig"
   fi
 
+  # Override apksigner flag if file is available, ref b/307968835.
+  local use_apksigner_file_path
+  use_apksigner_file_path="${system_mnt}/system/etc/signing_use_apksigner"
+  if [[ -f "${use_apksigner_file_path}" ]]; then
+    info "Changing apksigner flag from ${FLAGS_use_apksigner} to True"
+    FLAGS_use_apksigner=${FLAGS_TRUE}
+  fi
+
   snapshot_file_properties "${system_mnt}" > "${working_dir}/properties.orig"
 
   if ! sign_framework_apks "${system_mnt}" "${key_dir}" "${working_dir}"; then
@@ -462,7 +591,14 @@ sign_android_internal() {
     return 1
   fi
 
-  replace_ota_cert "${system_mnt}" "${key_dir}/releasekey.x509.pem"
+  local ota_cert_kn="releasekey"
+
+  if [[ -n ${KEYCFG_ANDROID_CLOUD_KEY_PREFIX} ]]; then
+    replace_ota_cert_cloud "${system_mnt}" "${key_dir}" "${ota_cert_kn}"
+  else
+    replace_ota_cert "${system_mnt}" "${key_dir}/${ota_cert_kn}.x509.pem"
+  fi
+
   if ! image_content_integrity_check "${system_mnt}" "${working_dir}" \
                                      "replace_ota_cert"; then
     return 1
@@ -492,7 +628,7 @@ sign_android_internal() {
         sudo "${fsck_erofs}" "--extract=${vendor_mnt}/vendor" "${vendor_img}"
       fi
 
-      if ! arc_generate_packages_cache "${system_mnt}" "${vendor_mnt}" \
+      if ! sudo arc_generate_packages_cache "${system_mnt}" "${vendor_mnt}" \
           "${working_dir}/packages_cache.xml" \
           "${working_dir}/file_hash_cache"; then
         die "Failed to generate packages cache."
@@ -529,10 +665,12 @@ sign_android_internal() {
   sudo rm -rf "${system_img}"
 
   if [[ "${image_type}" == "squashfs" ]]; then
-    info "Repacking squashfs image with ${squashfs_comp_flags[*]}"
-    sudo "${mksquashfs}" "${system_mnt}" "${system_img}" \
-      -context-file "${file_contexts}" -mount-point "/" \
-      -no-progress "${squashfs_comp_flags[@]}"
+    info "Repacking squashfs image with ${compression_flags[*]}"
+    if ! sudo "${mksquashfs}" "${system_mnt}" "${system_img}" \
+         -context-file "${file_contexts}" -mount-point "/" \
+         -no-progress "${compression_flags[@]}"; then
+      die "mksquashfs failed"
+    fi
 
     list_files_in_squashfs_image "${unsquashfs}" "${system_img}" > \
         "${working_dir}/image_file_list.new"
@@ -541,11 +679,12 @@ sign_android_internal() {
     # List all files inside the image.
     sudo find "${system_mnt}" > "${working_dir}/image_file_list.new"
 
-    info "Repacking erofs image"
-    # TODO(b/286165395): Detect the original EROFS compression type and use it
-    #                    instead of always using lz4hc.
-    sudo "${mkfs_erofs}" -z lz4hc -C262144 --file-contexts "${file_contexts}" \
-      "${system_img}" "${system_mnt}"
+    info "Repacking erofs image with ${compression_flags[*]}"
+    if ! sudo "${mkfs_erofs}" "${compression_flags[@]}" \
+         --file-contexts "${file_contexts}" \
+         "${system_img}" "${system_mnt}"; then
+      die "mkfs.erofs failed"
+    fi
   fi
 
   local new_size
