@@ -17,6 +17,7 @@
 
 # Load common constants and variables.
 . "$(dirname "$0")/common.sh"
+. "$(dirname "$0")/lib/keycfg.sh"
 
 # Abort on errors.
 set -e
@@ -41,11 +42,13 @@ where <type> is one of:
              accessory_usbpd (sign USB-PD accessory firmware)
              accessory_rwsig (sign accessory RW firmware)
              gsc_firmware (sign a GSC firmware image)
+             uefi_kernel (sign a UEFI kernel image)
 
 output_image: File name of the signed output image
 version_file: File name of where to read the kernel and firmware versions.
 --cloud-signing: Instead of relying on a local key directory, retrieve keys
   from Cloud KMS.
+--debug: Show more information for debugging purpose.
 
 If you are signing an image, you must specify an [output_image] and
 optionally, a [version_file].
@@ -79,6 +82,15 @@ check_argc() {
   esac
 }
 
+# Run futility as root with some preserved environment variables.
+sudo_futility() {
+  sudo "KMS_PKCS11_CONFIG=${KMS_PKCS11_CONFIG}" "${FUTILITY}" ${FUTILITY_EXTRA_FLAGS} "$@"
+}
+
+do_futility() {
+  "${FUTILITY}" ${FUTILITY_EXTRA_FLAGS} "$@"
+}
+
 # TODO(gauravsh): These are duplicated from chromeos-setimage. We need
 # to move all signing and rootfs code to one single place where it can be
 # reused. crosbug.com/19543
@@ -214,7 +226,7 @@ update_rootfs_hash() {
 
   # If we can't find dm parameters in the kernel config, bail out now.
   local kernel_config
-  kernel_config=$(sudo "${FUTILITY}" dump_kernel_config "${loop_kern}")
+  kernel_config=$(sudo_futility dump_kernel_config "${loop_kern}")
   local dm_config
   dm_config=$(get_dmparams_from_config "${kernel_config}")
   if [ -z "${dm_config}" ]; then
@@ -270,7 +282,7 @@ update_rootfs_hash() {
   for kernelpart in 2 4 6; do
     loop_kern="${loopdev}p${kernelpart}"
     if ! new_kernel_config="$(
-         sudo "${FUTILITY}" dump_kernel_config "${loop_kern}" 2>/dev/null)" &&
+         sudo_futility dump_kernel_config "${loop_kern}" 2>/dev/null)" &&
        [[ "${kernelpart}" == 4 ]]; then
       # Legacy images don't have partition 4.
       info "Skipping empty kernel partition 4 (legacy images)."
@@ -300,7 +312,7 @@ update_rootfs_hash() {
       keyblock="${kern_c_keyblock}"
       priv_key="${kern_c_privkey}"
     fi
-    sudo "${FUTILITY}" vbutil_kernel --repack "${loop_kern}" \
+    sudo_futility vbutil_kernel --repack "${loop_kern}" \
       --keyblock "${keyblock}" \
       --signprivate "${priv_key}" \
       --version "${KERNEL_VERSION}" \
@@ -320,16 +332,16 @@ update_stateful_partition_vblock() {
   temp_out_vb="$(make_temp_file)"
 
   local loop_kern="${loopdev}p4"
-  if [[ -z "$(sudo "${FUTILITY}" dump_kernel_config "${loop_kern}" \
+  if [[ -z "$(sudo_futility dump_kernel_config "${loop_kern}" \
         2>/dev/null)" ]]; then
     info "Building vmlinuz_hd.vblock from legacy image partition 2."
     loop_kern="${loopdev}p2"
   fi
 
   # vblock should always use kernel keyblock.
-  sudo "${FUTILITY}" vbutil_kernel --repack "${temp_out_vb}" \
-    --keyblock "${KEY_DIR}/kernel.keyblock" \
-    --signprivate "${KEY_DIR}/kernel_data_key.vbprivk" \
+  sudo_futility vbutil_kernel --repack "${temp_out_vb}" \
+    --keyblock "${KEYCFG_KERNEL_KEYBLOCK}" \
+    --signprivate "${KEYCFG_KERNEL_VBPRIVK}" \
     --oldblob "${loop_kern}" \
     --vblockonly
 
@@ -365,13 +377,14 @@ repack_firmware_bundle() {
   if [ ! -s "${target}" ]; then
     return 1
   elif grep -q '^##CUTHERE##' "${target}"; then
-    # Bundle supports repacking.
+    # Bundle supports repacking (--repack, --sb_repack)
     # Workaround issue crosbug.com/p/33719
     sed -i \
       's/shar -Q -q -x -m -w/shar -Q -q -x -m --no-character-count/' \
       "${target}"
-    "${target}" --sb_repack "${input_dir}" ||
-      die "Updating firmware autoupdate (--sb_repack) failed."
+    "${target}" --repack "${input_dir}" ||
+      "${target}" --sb_repack "${input_dir}" ||
+        die "Updating firmware autoupdate (--repack) failed."
   else
     # Legacy bundle using uuencode + tar.gz.
     # Replace MD5 checksum in the firmware update payload.
@@ -411,9 +424,29 @@ sign_firmware() {
 # Args: INPUT_IMAGE KEY_DIR OUTPUT_IMAGE
 sign_update_payload() {
   local image=$1
-  local key_dir=$2
+  local key_info=$2
   local output=$3
-  local key_output key_size key_file="${key_dir}/update_key.pem"
+  local key_output key_size
+
+  if [[ "${key_info}" == "remote:"* ]]; then
+    # get label from key_info with format "remote:<libkmsp11.so>:<slot>:<label>"
+    IFS=":" read -r -a parsed_info <<< "${key_info}"
+    if [[ "${#parsed_info[@]}" -ne 4 ]]; then
+      die "Failed to parse key info '${key_info}'"
+    fi
+    local p11_module="${parsed_info[1]}"
+    # We omit slot because the pkeyutl command will look in every keyring in the
+    # config file for the named key.
+    local key_label="${parsed_info[3]}"
+    # Hashing algorithm is always SHA-256.
+    PKCS11_MODULE_PATH="${p11_module}" openssl pkeyutl -pkeyopt \
+      rsa_padding_mode:pkcs1 -pkeyopt digest:sha256 -engine pkcs11 \
+      -keyform engine -sign --inkey "pkcs11:object=${key_label}" \
+      -in "${image}" -out "${output}"
+    return
+  fi
+  # Strip the prefix "local:"
+  key_info="${key_info#local:}"
   # Maps key size to verified boot's algorithm id (for pad_digest_utility).
   # Hashing algorithm is always SHA-256.
   local algo algos=(
@@ -423,7 +456,7 @@ sign_update_payload() {
     [8192]=10
   )
 
-  key_output=$(${FUTILITY} show "${key_file}")
+  key_output=$(do_futility show "${key_info}")
   key_size=$(echo "${key_output}" | sed -n '/Key length/s/[^0-9]*//p')
   algo=${algos[${key_size}]}
   if [[ -z ${algo} ]]; then
@@ -431,7 +464,7 @@ sign_update_payload() {
   fi
 
   pad_digest_utility "${algo}" "${image}" | \
-    openssl rsautl -sign -pkcs -inkey "${key_file}" -out "${output}"
+    openssl rsautl -sign -pkcs -inkey "${key_info}" -out "${output}"
 }
 
 # Re-sign the firmware AU payload inside the image rootfs with a new keys.
@@ -448,14 +481,29 @@ resign_firmware_payload() {
   local rootfs_dir
   rootfs_dir=$(make_temp_dir)
   mount_loop_image_partition "${loopdev}" 3 "${rootfs_dir}"
-  local firmware_bundle="${rootfs_dir}/usr/sbin/chromeos-firmwareupdate"
+
+  local ret=0
+  resign_firmware_shellball "${rootfs_dir}/usr/sbin/chromeos-firmwareupdate" || ret=$?
+  sudo umount "${rootfs_dir}"
+  if [[ "${ret}" == 0 ]]; then
+    info "Re-signed firmware AU payload in ${loopdev}"
+  else
+    error "Couldn't sign firmware AU payload in ${loopdev}"
+  fi
+  return "${ret}"
+}
+
+# Re-sign the firmware AU payload provided with a new key.
+# Args: firmware_bundle
+resign_firmware_shellball() {
+  local firmware_bundle=$1
+
   local shellball_dir
   shellball_dir=$(make_temp_dir)
 
   # extract_firmware_bundle can fail if the image has no firmware update.
   if ! extract_firmware_bundle "${firmware_bundle}" "${shellball_dir}"; then
     # Unmount now to prevent changes.
-    sudo umount "${rootfs_dir}"
     info "Didn't find a firmware update. Not signing firmware."
     return
   fi
@@ -489,12 +537,12 @@ resign_firmware_payload() {
       read # Burn the first line (header line)
       while IFS="," read -r output_name bios_image key_id ec_image brand_code
       do
-        local key_suffix=''
         local extra_args=()
         local full_command=()
-        local board_name
 
-        rootkey="${KEY_DIR}/root_key.vbpubk"
+        rootkey="$(get_root_key_vbpubk)"
+
+        info "Signing firmware image ${bios_image} for ${output_name}"
 
         # If there are OEM specific keys available, we're going to use them.
         # Otherwise, we're going to ignore key_id from the config file and
@@ -504,36 +552,41 @@ resign_firmware_payload() {
         # whether dynamic signature blocks are available or not.
         # This is what updater4.sh currently uses to make the decision.
         if [[ -e "${KEY_DIR}/loem.ini" ]]; then
-          # loem.ini has the format KEY_ID_VALUE = KEY_INDEX
           local match
-          match="$(grep -E "[0-9]+ *= *${key_id}$" "${KEY_DIR}/loem.ini")"
           local key_index
+
+          # loem.ini has the format KEY_ID_VALUE = KEY_INDEX
+          if ! match="$(grep -E "^[0-9]+ *= *${key_id}$" "${KEY_DIR}/loem.ini")"; then
+            die "The loem key_id ${key_id} not found in loem.ini!"
+          fi
+
           # shellcheck disable=SC2001
           key_index="$(echo "${match}" | sed 's/ *= *.*$//g')"
           info "Detected key index from loem.ini as ${key_index} for ${key_id}"
           if [[ -z "${key_index}" ]]; then
-            die "Failed to find key_id ${key_id} in loem.ini file for " \
+            die "Failed to extract key_index ${key_id} in loem.ini file for" \
               "${output_name}"
           fi
-          key_suffix=".loem${key_index}"
+
           shellball_keyset_dir="${shellball_dir}/keyset"
           mkdir -p "${shellball_keyset_dir}"
           extra_args+=(
             --loemdir "${shellball_keyset_dir}"
             --loemid "${output_name}"
           )
-          rootkey="${KEY_DIR}/root_key${key_suffix}.vbpubk"
+          rootkey="$(get_root_key_vbpubk "${key_index}")"
           cp "${rootkey}" "${shellball_keyset_dir}/rootkey.${output_name}"
         fi
 
-        info "Signing firmware image ${bios_image} for ${output_name} " \
-          "with key suffix ${key_suffix}"
+        info "Using root key: ${rootkey##*/}"
 
         local temp_fw
         temp_fw=$(make_temp_file)
 
-        local signprivate="${KEY_DIR}/firmware_data_key${key_suffix}.vbprivk"
-        local keyblock="${KEY_DIR}/firmware${key_suffix}.keyblock"
+        local signprivate
+        local keyblock
+        signprivate="$(get_firmware_vbprivk "${key_index}")"
+        keyblock="$(get_firmware_keyblock "${key_index}")"
 
         # Path to bios.bin.
         local bios_path="${shellball_dir}/${bios_image}"
@@ -552,9 +605,9 @@ resign_firmware_payload() {
             # futility writes byproduct files to CWD, so we cd to temp dir.
             pushd "$(make_temp_dir)" > /dev/null
             full_command=(
-              "${FUTILITY}" sign
+              do_futility sign
               --type rwsig
-              --prikey "${KEY_DIR}/key_ec_efs.vbprik2"
+              --prikey "${KEYCFG_KEY_EC_EFS_VBPRIK2}"
               --ecrw_out "${rw_bin}"
               "${ec_path}"
             )
@@ -577,10 +630,10 @@ resign_firmware_payload() {
 
         # Resign bios.bin.
         full_command=(
-          "${FUTILITY}" sign
+          do_futility sign
           --signprivate "${signprivate}"
           --keyblock "${keyblock}"
-          --kernelkey "${KEY_DIR}/kernel_subkey.vbpubk"
+          --kernelkey "${KEYCFG_KERNEL_SUBKEY_VBPUBK}"
           --version "${FIRMWARE_VERSION}"
           "${extra_args[@]}"
           "${bios_path}"
@@ -595,9 +648,9 @@ resign_firmware_payload() {
         # For development phases, when the GBB can be updated still, set the
         # recovery and root keys in the image.
         full_command=(
-          "${FUTILITY}" gbb
+          do_futility gbb
           -s
-          --recoverykey="${KEY_DIR}/recovery_key.vbpubk"
+          --recoverykey="${KEYCFG_RECOVERY_KEY_VBPUBK}"
           --rootkey="${rootkey}" "${temp_fw}"
           "${bios_path}"
         )
@@ -607,9 +660,7 @@ resign_firmware_payload() {
         echo "After setting GBB on ${bios_path}: md5 =" \
           "$(md5sum "${bios_path}" | awk '{print $1}')"
 
-        board_name="$(get_boardvar_from_lsb_release "${rootfs_dir}")"
-        echo "Board name from lsb-release: ${board_name}"
-        if [[ ${board_name} == *guybrush* ]]; then
+        if [[ -e "${shellball_dir}/models/guybrush" ]]; then
           echo "Not looking for RO_GSCVD on guybrush, b/263378945"
         elif futility dump_fmap -p "${bios_path}" | grep -q RO_GSCVD; then
           # Attempt AP RO verification signing only in case the FMAP includes
@@ -620,16 +671,16 @@ resign_firmware_payload() {
             die "No brand code for ${bios_path} in signer_config.csv"
           fi
 
-          arv_root="${KEY_DIR}/arv_root.vbpubk"
+          arv_root="${KEYCFG_ARV_ROOT_VBPUBK}"
           if [[ ! -f ${arv_root} ]]; then
             die "No AP RO verification keys, could not create RO_GSCVD"
           fi
 
           # Resign the RO_GSCVD FMAP area.
           full_command=(
-            "${FUTILITY}" gscvd
-            --keyblock "${KEY_DIR}/arv_platform.keyblock"
-            --platform_priv "${KEY_DIR}/arv_platform.vbprivk"
+            do_futility gscvd
+            --keyblock "${KEYCFG_ARV_PLATFORM_KEYBLOCK}"
+            --platform_priv "${KEYCFG_ARV_PLATFORM_VBPRIVK}"
             --board_id "${brand_code}"
             --root_pub_key "${arv_root}"
             "${bios_path}"
@@ -672,8 +723,8 @@ resign_firmware_payload() {
   echo "" >"${signer_notes}"
   echo "Signed with keyset in $(readlink -f "${KEY_DIR}") ." >>"${signer_notes}"
   # record recovery_key
-  key="${KEY_DIR}/recovery_key.vbpubk"
-  sha1=$(${FUTILITY} vbutil_key --unpack "${key}" \
+  key="${KEYCFG_RECOVERY_KEY_VBPUBK}"
+  sha1=$(do_futility vbutil_key --unpack "${key}" \
     | grep sha1sum | cut -d" " -f9)
   echo "recovery: ${sha1}" >>"${signer_notes}"
   # record root_key(s)
@@ -681,27 +732,25 @@ resign_firmware_payload() {
     echo "List sha1sum of all loem/model's signatures:" >>"${signer_notes}"
     for key in "${shellball_keyset_dir}"/rootkey.*; do
       model="${key##*.}"
-      sha1=$(${FUTILITY} vbutil_key --unpack "${key}" \
+      sha1=$(do_futility vbutil_key --unpack "${key}" \
         | grep sha1sum | cut -d" " -f9)
       echo "  ${model}: ${sha1}" >>"${signer_notes}"
     done
   else
     echo "List sha1sum of single key's signature:" >>"${signer_notes}"
-    key="${KEY_DIR}/root_key.vbpubk"
-    sha1=$(${FUTILITY} vbutil_key --unpack "${key}" \
+    key="$(get_root_key_vbpubk)"
+    sha1=$(do_futility vbutil_key --unpack "${key}" \
       | grep sha1sum | cut -d" " -f9)
     echo "  root: ${sha1}" >>"${signer_notes}"
   fi
 
+  local new_shellball
   new_shellball=$(make_temp_file)
   cp -f "${firmware_bundle}" "${new_shellball}"
   chmod a+rx "${new_shellball}"
   repack_firmware_bundle "${shellball_dir}" "${new_shellball}"
   sudo cp -f "${new_shellball}" "${firmware_bundle}"
   sudo chmod a+rx "${firmware_bundle}"
-  # Unmount now to flush changes.
-  sudo umount "${rootfs_dir}"
-  info "Re-signed firmware AU payload in ${loopdev}"
 }
 
 # Remove old container key if it exists.
@@ -739,19 +788,15 @@ resign_android_image_if_exists() {
   fi
 
   info "Found ARC image version '${arc_version}', re-signing APKs."
-  # TODO(crbug.com/1141907): remove set -x and set +x below.
-  set -x
   "${SCRIPT_DIR}/sign_android_image.sh" "${rootfs_dir}" "${KEY_DIR}/android"
 
   if ! sudo umount "${rootfs_dir}"; then
     error "umount ${rootfs_dir} failed"
     sudo lsof -n "${rootfs_dir}"
     ps auxf
-    set +x
     return 1
   fi
   info "Re-signed Android image"
-  set +x
 }
 
 # Check whether the image's board is reven or not.
@@ -804,14 +849,24 @@ sign_uefi_binaries() {
     # change the glob so that they don't get resigned.
     efi_glob="grub*.efi"
   fi
-  "${SCRIPT_DIR}/sign_uefi.py" "${esp_dir}" "${KEY_DIR}/uefi" "${efi_glob}"
+
+  local sign_uefi_cmd=(
+      "${SCRIPT_DIR}/sign_uefi.py"
+      --private-key "${KEYCFG_UEFI_PRIVATE_KEY}"
+      --sign-cert "${KEYCFG_UEFI_SIGN_CERT}"
+      --verify-cert "${KEYCFG_UEFI_VERIFY_CERT}"
+      --kernel-subkey-vbpubk "${KEYCFG_KERNEL_SUBKEY_VBPUBK}"
+      --crdyshim-private-key "${KEYCFG_UEFI_CRDYSHIM_PRIVATE_KEY}"
+      --efi-glob "${efi_glob}"
+  )
+
+  "${sign_uefi_cmd[@]}" --target-dir "${esp_dir}"
   sudo umount "${esp_dir}"
 
   local rootfs_dir
   rootfs_dir="$(make_temp_dir)"
   mount_loop_image_partition "${loopdev}" 3 "${rootfs_dir}"
-  "${SCRIPT_DIR}/sign_uefi.py" "${rootfs_dir}/boot" "${KEY_DIR}/uefi" \
-                               "${efi_glob}"
+  "${sign_uefi_cmd[@]}" --target-dir "${rootfs_dir}/boot"
   sudo umount "${rootfs_dir}"
 
   info "Signed UEFI binaries"
@@ -879,7 +934,7 @@ verify_image() {
   local partnum
   for partnum in 2 4; do
     info "Considering Kernel partition ${partnum}"
-    kernel_config=$(sudo "${FUTILITY}" dump_kernel_config \
+    kernel_config=$(sudo_futility dump_kernel_config \
       "${loopdev}p${partnum}")
     local hash_image
     hash_image=$(make_temp_file)
@@ -914,7 +969,7 @@ EOF
 
   # Now try and verify kernel partition signature.
   set +e
-  local try_key=${KEY_DIR}/recovery_key.vbpubk
+  local try_key="${KEYCFG_RECOVERY_KEY_VBPUBK}"
   info "Testing key verification..."
   # The recovery key is only used in the recovery mode.
   echo -n "With Recovery Key (Recovery Mode ON, Dev Mode OFF): " && \
@@ -924,7 +979,7 @@ EOF
   { load_kernel_test "${INPUT_IMAGE}" "${try_key}" -b 3 >/dev/null 2>&1 && \
     echo "YES"; } || echo "NO"
 
-  try_key=${KEY_DIR}/kernel_subkey.vbpubk
+  try_key="${KEYCFG_KERNEL_SUBKEY_VBPUBK}"
   # The SSD key is only used in non-recovery mode.
   echo -n "With SSD Key (Recovery Mode OFF, Dev Mode OFF): " && \
   { load_kernel_test "${INPUT_IMAGE}" "${try_key}" -b 0 >/dev/null 2>&1  && \
@@ -954,7 +1009,7 @@ update_recovery_kernel_hash() {
 
   # Update the kernel B hash in the recovery kernel command line.
   local old_kernel_config
-  old_kernel_config="$(sudo "${FUTILITY}" \
+  old_kernel_config="$(sudo_futility \
     dump_kernel_config "${loop_recovery_kernel}")"
   local old_kernb_hash
   old_kernb_hash="$(echo "${old_kernel_config}" |
@@ -975,7 +1030,7 @@ update_recovery_kernel_hash() {
   cat "${new_kernel_config}"
 
   # Re-calculate kernel partition signature and command line.
-  sudo "${FUTILITY}" vbutil_kernel --repack "${loop_recovery_kernel}" \
+  sudo_futility vbutil_kernel --repack "${loop_recovery_kernel}" \
     --keyblock "${keyblock}" \
     --signprivate "${privkey}" \
     --version "${KERNEL_VERSION}" \
@@ -1014,7 +1069,7 @@ resign_minios_kernels() {
     # Skip miniOS partitions which are empty. This happens when miniOS
     # kernels aren't written to the partitions because the feature is not
     # enabled.
-    if ! sudo "${FUTILITY}" dump_kernel_config "${loop_minios}"; then
+    if ! sudo_futility dump_kernel_config "${loop_minios}"; then
       info "Skipping empty miniOS partition ${loop_minios}."
       continue
     fi
@@ -1033,7 +1088,7 @@ resign_minios_kernels() {
 
     # Assume this is a miniOS kernel.
     local minios_kernel_version=$((KERNEL_VERSION >> 24))
-    if sudo "${FUTILITY}" vbutil_kernel --repack "${loop_minios}" \
+    if sudo_futility vbutil_kernel --repack "${loop_minios}" \
         --keyblock "${keyblock}" \
         --signprivate "${priv_key}" \
         --version "${minios_kernel_version}" \
@@ -1063,7 +1118,7 @@ update_legacy_bootloader() {
 
   # If we can't find the dm parameter in the kernel config, bail out now.
   local kernel_config
-  kernel_config=$(sudo "${FUTILITY}" dump_kernel_config "${loop_kern}")
+  kernel_config=$(sudo_futility dump_kernel_config "${loop_kern}")
   local root_hexdigest
   root_hexdigest="$(get_hash_from_config "${kernel_config}")"
   if [[ -z "${root_hexdigest}" ]]; then
@@ -1185,7 +1240,7 @@ sign_image_file() {
   # config.
   local loop_kerna="${loopdev}p2"
   local kerna_config
-  kerna_config="$(sudo "${FUTILITY}" dump_kernel_config "${loop_kerna}")"
+  kerna_config="$(sudo_futility dump_kernel_config "${loop_kerna}")"
   if [[ "${image_type}" != "factory_install" &&
         " ${kerna_config} " != *" cros_legacy "* &&
         " ${kerna_config} " != *" cros_efi "* ]]; then
@@ -1229,6 +1284,24 @@ sign_image_file() {
   info "Signed ${image_type} image output to ${output}"
 }
 
+# Sign a UEFI kernel kernel image with proper keys.
+# Args: INPUT OUTPUT
+sign_uefi_kernel() {
+  local input="$1"
+  local output="$2"
+
+  info "Preparing UEFI kernel image..."
+  cp --sparse=always "${input}" "${output}"
+
+  "${SCRIPT_DIR}/sign_uefi.py" \
+    --target-file "${output}" \
+    --private-key "${KEYCFG_UEFI_PRIVATE_KEY}" \
+    --sign-cert "${KEYCFG_UEFI_SIGN_CERT}" \
+    --verify-cert "${KEYCFG_UEFI_VERIFY_CERT}" \
+    --kernel-subkey-vbpubk "${KEYCFG_KERNEL_SUBKEY_VBPUBK}" \
+    --crdyshim-private-key "${KEYCFG_UEFI_CRDYSHIM_PRIVATE_KEY}"
+}
+
 main() {
   # Add to the path since some tools reside here and may not be in the non-root
   # system path.
@@ -1243,11 +1316,11 @@ main() {
 
   # Parse arguments with positional and optional options.
   local script_args=()
-  CLOUD_SIGNING=false
+  FUTILITY_EXTRA_FLAGS=""
   while [[ "$#" -gt 0 ]]; do
     case $1 in
-      --cloud-signing)
-        CLOUD_SIGNING=true
+      --debug)
+        FUTILITY_EXTRA_FLAGS+="--debug "
         ;;
       -h|--help)
         usage
@@ -1266,9 +1339,6 @@ main() {
     shift
   done
 
-  if [[ "${CLOUD_SIGNING}" == true ]]; then
-    info "signing with cloud keys"
-  fi
   set -- "${script_args[@]}"
 
   TYPE=$1
@@ -1277,6 +1347,8 @@ main() {
   OUTPUT_IMAGE=$4
   VERSION_FILE=$5
 
+  setup_keycfg "${KEY_DIR}"
+
   # Verification
   case ${TYPE} in
   dump_config)
@@ -1284,7 +1356,7 @@ main() {
     loopdev=$(loopback_partscan "${INPUT_IMAGE}")
     for partnum in 2 4; do
       info "kernel config in partition number ${partnum}:"
-      sudo "${FUTILITY}" dump_kernel_config "${loopdev}p${partnum}"
+      sudo_futility dump_kernel_config "${loopdev}p${partnum}"
       echo
     done
     exit 0
@@ -1317,32 +1389,32 @@ main() {
   # Make all modifications on output copy.
   if [[ "${TYPE}" == "base" ]]; then
     sign_image_file "base" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 2 \
-      "${KEY_DIR}/kernel.keyblock" \
-      "${KEY_DIR}/kernel_data_key.vbprivk" \
-      "${KEY_DIR}/kernel.keyblock" \
-      "${KEY_DIR}/kernel_data_key.vbprivk" \
+      "${KEYCFG_KERNEL_KEYBLOCK}" \
+      "${KEYCFG_KERNEL_VBPRIVK}" \
+      "${KEYCFG_KERNEL_KEYBLOCK}" \
+      "${KEYCFG_KERNEL_VBPRIVK}" \
       "" \
       "" \
-      "${KEY_DIR}/minios_kernel.keyblock" \
+      "${KEYCFG_MINIOS_KERNEL_KEYBLOCK}" \
       "" \
-      "${KEY_DIR}/minios_kernel_data_key.vbprivk"
+      "${KEYCFG_MINIOS_KERNEL_VBPRIVK}"
   elif [[ "${TYPE}" == "recovery" ]]; then
     sign_image_file "recovery" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 4 \
-      "${KEY_DIR}/recovery_kernel.keyblock" \
-      "${KEY_DIR}/recovery_kernel_data_key.vbprivk" \
-      "${KEY_DIR}/kernel.keyblock" \
-      "${KEY_DIR}/kernel_data_key.vbprivk" \
-      "${KEY_DIR}/recovery_kernel.v1.keyblock" \
-      "${KEY_DIR}/recovery_kernel_data_key.vbprivk" \
-      "${KEY_DIR}/minios_kernel.keyblock" \
-      "${KEY_DIR}/minios_kernel.v1.keyblock" \
-      "${KEY_DIR}/minios_kernel_data_key.vbprivk"
+      "${KEYCFG_RECOVERY_KERNEL_KEYBLOCK}" \
+      "${KEYCFG_RECOVERY_KERNEL_VBPRIVK}" \
+      "${KEYCFG_KERNEL_KEYBLOCK}" \
+      "${KEYCFG_KERNEL_VBPRIVK}" \
+      "${KEYCFG_RECOVERY_KERNEL_V1_KEYBLOCK}" \
+      "${KEYCFG_RECOVERY_KERNEL_VBPRIVK}" \
+      "${KEYCFG_MINIOS_KERNEL_KEYBLOCK}" \
+      "${KEYCFG_MINIOS_KERNEL_V1_KEYBLOCK}" \
+      "${KEYCFG_MINIOS_KERNEL_VBPRIVK}"
   elif [[ "${TYPE}" == "factory" ]]; then
     sign_image_file "factory_install" "${INPUT_IMAGE}" "${OUTPUT_IMAGE}" 2 \
-      "${KEY_DIR}/installer_kernel.keyblock" \
-      "${KEY_DIR}/installer_kernel_data_key.vbprivk" \
-      "${KEY_DIR}/installer_kernel.v1.keyblock" \
-      "${KEY_DIR}/installer_kernel_data_key.vbprivk" \
+      "${KEYCFG_INSTALLER_KERNEL_KEYBLOCK}" \
+      "${KEYCFG_INSTALLER_KERNEL_VBPRIVK}" \
+      "${KEYCFG_INSTALLER_KERNEL_V1_KEYBLOCK}" \
+      "${KEYCFG_INSTALLER_KERNEL_VBPRIVK}" \
       "" \
       "" \
       "" \
@@ -1354,34 +1426,45 @@ main() {
     fi
     cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
     sign_firmware "${OUTPUT_IMAGE}" "${KEY_DIR}" "${FIRMWARE_VERSION}"
+  elif [[ "${TYPE}" == "shellball" ]]; then
+    cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
+    resign_firmware_shellball "${OUTPUT_IMAGE}"
+    info "Signed firmware shellball ${OUTPUT_IMAGE}"
   elif [[ "${TYPE}" == "update_payload" ]]; then
-    sign_update_payload "${INPUT_IMAGE}" "${KEY_DIR}" "${OUTPUT_IMAGE}"
+    sign_update_payload "${INPUT_IMAGE}" "${KEYCFG_UPDATE_KEY_PEM}" "${OUTPUT_IMAGE}"
   elif [[ "${TYPE}" == "accessory_usbpd" ]]; then
     KEY_NAME="${KEY_DIR}/key_$(basename "$(dirname "${INPUT_IMAGE}")")"
     if [[ ! -e "${KEY_NAME}.pem" ]]; then
       KEY_NAME="${KEY_DIR}/key"
     fi
     cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-    ${FUTILITY} sign --type usbpd1 --pem "${KEY_NAME}.pem" "${OUTPUT_IMAGE}"
+    do_futility sign --type usbpd1 --pem "${KEY_NAME}.pem" "${OUTPUT_IMAGE}"
   elif [[ "${TYPE}" == "accessory_rwsig" ]]; then
+    # If KEYCFG_ACCESSORY_RWSIG_VBPRIK2 is non-empty, use it.
+    if [[ -n "${KEYCFG_ACCESSORY_RWSIG_VBPRIK2}" ]]; then
+      PRIV_KEY="${KEYCFG_ACCESSORY_RWSIG_VBPRIK2}"
     # If one key is present in this container, assume it's the right one.
     # See crbug.com/863464
-    if [[ ! -e "${KEY_NAME}.vbprik2" ]]; then
+    else
+      shopt -s nullglob
       KEYS=( "${KEY_DIR}"/*.vbprik2 )
+      shopt -u nullglob
       if [[ ${#KEYS[@]} -eq 1 ]]; then
-        KEY_NAME="${KEYS[0]}"
+        PRIV_KEY="${KEYS[0]}"
       else
         die "Expected exactly one key present in keyset for accessory_rwsig"
       fi
     fi
     cp "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
-    ${FUTILITY} sign --type rwsig --prikey "${KEY_NAME}" \
+    do_futility sign --type rwsig --prikey "${PRIV_KEY}" \
              --version "${FIRMWARE_VERSION}" "${OUTPUT_IMAGE}"
   elif [[ "${TYPE}" == "gsc_firmware" ]]; then
     sign_gsc_firmware "${INPUT_IMAGE}" "${KEY_DIR}" "${OUTPUT_IMAGE}"
   elif [[ "${TYPE}" == "hps_firmware" ]]; then
     hps-sign-rom --input "${INPUT_IMAGE}" --output "${OUTPUT_IMAGE}" \
       --private-key "${KEY_DIR}/key_hps.priv.pem"
+  elif [[ "${TYPE}" == "uefi_kernel" ]]; then
+      sign_uefi_kernel "${INPUT_IMAGE}" "${OUTPUT_IMAGE}"
   else
     die "Invalid type ${TYPE}"
   fi
