@@ -87,6 +87,8 @@ enum {
 	LB_TAG_TYPE_C_INFO		= 0x0042,
 	LB_TAG_ACPI_RSDP		= 0x0043,
 	LB_TAG_PCIE			= 0x0044,
+	LB_TAG_EFI_FW_INFO		= 0x0045,
+	LB_TAG_CAPSULE			= 0x0046,
 	/* The following options are CMOS-related */
 	LB_TAG_CMOS_OPTION_TABLE	= 0x00c8,
 	LB_TAG_OPTION			= 0x00c9,
@@ -275,6 +277,11 @@ enum lb_fb_orientation {
 	LB_FB_ORIENTATION_RIGHT_UP = 3,
 };
 
+struct lb_framebuffer_flags {
+	uint8_t has_external_display : 1;
+	uint8_t reserved : 7;
+};
+
 struct lb_framebuffer {
 	uint32_t tag;
 	uint32_t size;
@@ -293,7 +300,8 @@ struct lb_framebuffer {
 	uint8_t reserved_mask_pos;
 	uint8_t reserved_mask_size;
 	uint8_t orientation;
-	uint8_t pad[2];
+	struct lb_framebuffer_flags flags;
+	uint8_t pad;
 };
 
 struct lb_gpio {
@@ -528,7 +536,7 @@ struct	cmos_checksum {
 struct lb_smmstorev2 {
 	uint32_t tag;
 	uint32_t size;
-	uint32_t num_blocks;		/* Number of writeable blocks in SMM */
+	uint32_t num_blocks;		/* Number of writable blocks in SMM */
 	uint32_t block_size;		/* Size of a block in byte. Default: 64 KiB */
 	uint32_t mmap_addr;		/* MMIO address of the store for read only access */
 	uint32_t com_buffer;		/* Physical address of the communication buffer */
@@ -572,4 +580,23 @@ struct lb_acpi_rsdp {
 	lb_uint64_t rsdp_pointer; /* Address of the ACPI RSDP */
 };
 
+/*
+ * Machine-friendly version of a system firmware component.  A component is
+ * identified by a GUID.  coreboot is an obvious main component but there could
+ * be others (like EC) which should get their own instances of the tag.
+ *
+ * The main consumer of this information is UEFI firmware but something else
+ * could reuse it too.
+ *
+ * Larger number in a version field corresponds to a more recent version.
+ */
+struct lb_efi_fw_info {
+	uint32_t tag;
+	uint32_t size;
+	uint8_t guid[16];			/* Called "firmware class" in UEFI */
+	uint32_t version;			/* Current version */
+	uint32_t lowest_supported_version;	/* Lowest allowed version for downgrades */
+	uint32_t fw_size;			/* Size of firmware in bytes */
+} __packed;
+
 #endif
