@@ -97,7 +97,7 @@ u16 spd_ddr3_calc_unique_crc(u8 *spd, int len)
  *		SPD_STATUS_INVALID_FIELD -- A field with an invalid value was
  *					    detected.
  */
-int spd_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd)
+int spd_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_ddr3_raw_data spd)
 {
 	int ret;
 	u16 crc, spd_crc;
@@ -122,7 +122,7 @@ int spd_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd)
 	dimm->dram_type = SPD_MEMORY_TYPE_SDRAM_DDR3;
 	dimm->dimm_type = spd[3] & 0xf;
 
-	crc = spd_ddr3_calc_crc(spd, sizeof(spd_raw_data));
+	crc = spd_ddr3_calc_crc(spd, sizeof(spd_ddr3_raw_data));
 	/* Compare with the CRC in the SPD */
 	spd_crc = (spd[127] << 8) + spd[126];
 	/* Verify the CRC is correct */
@@ -365,7 +365,7 @@ int spd_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd)
 	memcpy(dimm->part_number, &spd[128], 16);
 	printram("  Part number        : %s\n", dimm->part_number);
 
-	memcpy(dimm->serial, &spd[SPD_DIMM_SERIAL_NUM], SPD_DIMM_SERIAL_LEN);
+	memcpy(dimm->serial, &spd[SPD_DDR3_SERIAL_NUM], SPD_DDR3_SERIAL_LEN);
 
 	return ret;
 }
@@ -390,7 +390,7 @@ int spd_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd)
  *		SPD_STATUS_INVALID_FIELD -- A field with an invalid value was
  *					    detected.
  */
-int spd_xmp_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd,
+int spd_xmp_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_ddr3_raw_data spd,
 			enum ddr3_xmp_profile profile)
 {
 	int ret;
@@ -422,6 +422,8 @@ int spd_xmp_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd,
 		/* Medium Timebase =
 		 *   Medium Timebase (MTB) Dividend /
 		 *   Medium Timebase (MTB) Divisor */
+		if (spd[181] == 0) // Avoid dividing by zero.
+			return SPD_STATUS_INVALID;
 		mtb = (((u32)spd[180]) << 8) / spd[181];
 
 		dimm->dimms_per_channel = ((spd[178] >> 2) & 0x3) + 1;
@@ -437,6 +439,8 @@ int spd_xmp_decode_ddr3(struct dimm_attr_ddr3_st *dimm, spd_raw_data spd,
 		/* Medium Timebase =
 		 *   Medium Timebase (MTB) Dividend /
 		 *   Medium Timebase (MTB) Divisor */
+		if (spd[183] == 0) // Avoid dividing by zero.
+			return SPD_STATUS_INVALID;
 		mtb = (((u32)spd[182]) << 8) / spd[183];
 
 		dimm->dimms_per_channel = ((spd[178] >> 4) & 0x3) + 1;
@@ -533,29 +537,7 @@ enum cb_err spd_add_smbios17(const u8 channel, const u8 slot, const u16 selected
 		dimm->dimm_num = slot;
 		memcpy(dimm->module_part_number, info->part_number, 16);
 		dimm->mod_id = info->manufacturer_id;
-
-		switch (info->dimm_type) {
-		case SPD_DDR3_DIMM_TYPE_SO_DIMM:
-			dimm->mod_type = DDR3_SPD_SODIMM;
-			break;
-		case SPD_DDR3_DIMM_TYPE_72B_SO_CDIMM:
-			dimm->mod_type = DDR3_SPD_72B_SO_CDIMM;
-			break;
-		case SPD_DDR3_DIMM_TYPE_72B_SO_RDIMM:
-			dimm->mod_type = DDR3_SPD_72B_SO_RDIMM;
-			break;
-		case SPD_DDR3_DIMM_TYPE_UDIMM:
-			dimm->mod_type = DDR3_SPD_UDIMM;
-			break;
-		case SPD_DDR3_DIMM_TYPE_RDIMM:
-			dimm->mod_type = DDR3_SPD_RDIMM;
-			break;
-		case SPD_DDR3_DIMM_TYPE_UNDEFINED:
-		default:
-			dimm->mod_type = SPD_UNDEFINED;
-			break;
-		}
-
+		dimm->mod_type = info->dimm_type;
 		dimm->bus_width = MEMORY_BUS_WIDTH_64; // non-ECC only
 		memcpy(dimm->serial, info->serial,
 		       MIN(sizeof(dimm->serial), sizeof(info->serial)));
