@@ -7,6 +7,7 @@
 
 #include <acpi/acpi.h>
 #include <assert.h>
+#include <cbmem.h>
 #include <device/pci_ops.h>
 #include <bootmode.h>
 #include <console/console.h>
@@ -108,7 +109,7 @@ struct resource *pci_get_resource(struct device *dev, unsigned long index)
 	     PCI_BASE_ADDRESS_MEM_LIMIT_64)) {
 		/* Find the high bits that move. */
 		moving |=
-		    ((resource_t) pci_moving_config32(dev, index + 4)) << 32;
+		    ((resource_t)pci_moving_config32(dev, index + 4)) << 32;
 	}
 
 	/* Find the resource constraints.
@@ -408,7 +409,7 @@ static void configure_adjustable_base(const struct device *dev,
 
 	int max_requested_bits = __fls64(size_mask);
 	if (max_requested_bits > CONFIG_PCIEXP_DEFAULT_MAX_RESIZABLE_BAR_BITS) {
-		printk(BIOS_WARNING, "WARNING: Device %s requests a BAR with"
+		printk(BIOS_WARNING, "Device %s requests a BAR with"
 		       " %u bits of address space, which coreboot is not"
 		       " configured to hand out, truncating to %u bits\n",
 		       dev_path(dev), max_requested_bits,
@@ -417,7 +418,7 @@ static void configure_adjustable_base(const struct device *dev,
 	}
 
 	if (!(res->flags & IORESOURCE_PCI64) && max_requested_bits > 32) {
-		printk(BIOS_ERR, "ERROR: Resizable BAR requested"
+		printk(BIOS_ERR, "Resizable BAR requested"
 		       " above 32 bits, but PCI function reported a"
 		       " 32-bit BAR.");
 		return;
@@ -496,13 +497,13 @@ static void pci_bridge_read_bases(struct device *dev)
 	resource_t moving_base, moving_limit, moving;
 
 	/* See if the bridge I/O resources are implemented. */
-	moving_base = ((u32) pci_moving_config8(dev, PCI_IO_BASE)) << 8;
+	moving_base = ((u32)pci_moving_config8(dev, PCI_IO_BASE)) << 8;
 	moving_base |=
-	  ((u32) pci_moving_config16(dev, PCI_IO_BASE_UPPER16)) << 16;
+	  ((u32)pci_moving_config16(dev, PCI_IO_BASE_UPPER16)) << 16;
 
-	moving_limit = ((u32) pci_moving_config8(dev, PCI_IO_LIMIT)) << 8;
+	moving_limit = ((u32)pci_moving_config8(dev, PCI_IO_LIMIT)) << 8;
 	moving_limit |=
-	  ((u32) pci_moving_config16(dev, PCI_IO_LIMIT_UPPER16)) << 16;
+	  ((u32)pci_moving_config16(dev, PCI_IO_LIMIT_UPPER16)) << 16;
 
 	moving = moving_base & moving_limit;
 
@@ -511,14 +512,14 @@ static void pci_bridge_read_bases(struct device *dev)
 
 	/* See if the bridge prefmem resources are implemented. */
 	moving_base =
-	  ((resource_t) pci_moving_config16(dev, PCI_PREF_MEMORY_BASE)) << 16;
+	  ((resource_t)pci_moving_config16(dev, PCI_PREF_MEMORY_BASE)) << 16;
 	moving_base |=
-	  ((resource_t) pci_moving_config32(dev, PCI_PREF_BASE_UPPER32)) << 32;
+	  ((resource_t)pci_moving_config32(dev, PCI_PREF_BASE_UPPER32)) << 32;
 
 	moving_limit =
-	  ((resource_t) pci_moving_config16(dev, PCI_PREF_MEMORY_LIMIT)) << 16;
+	  ((resource_t)pci_moving_config16(dev, PCI_PREF_MEMORY_LIMIT)) << 16;
 	moving_limit |=
-	  ((resource_t) pci_moving_config32(dev, PCI_PREF_LIMIT_UPPER32)) << 32;
+	  ((resource_t)pci_moving_config32(dev, PCI_PREF_LIMIT_UPPER32)) << 32;
 
 	moving = moving_base & moving_limit;
 	/* Initialize the prefetchable memory constraints on the current bus. */
@@ -526,8 +527,8 @@ static void pci_bridge_read_bases(struct device *dev)
 				   IORESOURCE_MEM | IORESOURCE_PREFETCH);
 
 	/* See if the bridge mem resources are implemented. */
-	moving_base = ((u32) pci_moving_config16(dev, PCI_MEMORY_BASE)) << 16;
-	moving_limit = ((u32) pci_moving_config16(dev, PCI_MEMORY_LIMIT)) << 16;
+	moving_base = ((u32)pci_moving_config16(dev, PCI_MEMORY_BASE)) << 16;
+	moving_limit = ((u32)pci_moving_config16(dev, PCI_MEMORY_LIMIT)) << 16;
 
 	moving = moving_base & moving_limit;
 
@@ -561,16 +562,30 @@ void pci_domain_read_resources(struct device *dev)
 	res->flags = IORESOURCE_IO | IORESOURCE_SUBTRACTIVE |
 		     IORESOURCE_ASSIGNED;
 
-	/* Initialize the system-wide memory resources constraints. */
+	/*
+	 * Initialize 32-bit memory resource constraints.
+	 *
+	 * There are often undeclared chipset resources in lower memory
+	 * and memory right below the 4G barrier. Hence, only allow
+	 * one big range from cbmem_top to the configured limit.
+	 */
 	res = new_resource(dev, IOINDEX_SUBTRACTIVE(1, 0));
-	res->limit = (1ULL << cpu_phys_address_size()) - 1;
+	res->base  = cbmem_top();
+	res->limit = CONFIG_DOMAIN_RESOURCE_32BIT_LIMIT - 1;
+	res->flags = IORESOURCE_MEM | IORESOURCE_SUBTRACTIVE |
+		     IORESOURCE_ASSIGNED;
+
+	/* Initialize 64-bit memory resource constraints above 4G. */
+	res = new_resource(dev, IOINDEX_SUBTRACTIVE(2, 0));
+	res->base  = 4ULL * GiB;
+	res->limit = (1ULL << soc_phys_address_size()) - 1;
 	res->flags = IORESOURCE_MEM | IORESOURCE_SUBTRACTIVE |
 		     IORESOURCE_ASSIGNED;
 }
 
 void pci_domain_set_resources(struct device *dev)
 {
-	assign_resources(dev->link_list);
+	assign_resources(dev->downstream);
 }
 
 static void pci_store_resource(const struct device *const dev,
@@ -698,16 +713,13 @@ static void pci_set_resource(struct device *dev, struct resource *resource)
 void pci_dev_set_resources(struct device *dev)
 {
 	struct resource *res;
-	struct bus *bus;
 	u8 line;
 
 	for (res = dev->resource_list; res; res = res->next)
 		pci_set_resource(dev, res);
 
-	for (bus = dev->link_list; bus; bus = bus->next) {
-		if (bus->children)
-			assign_resources(bus);
-	}
+	if (dev->downstream && dev->downstream->children)
+		assign_resources(dev->downstream);
 
 	/* Set a default latency timer. */
 	pci_write_config8(dev, PCI_LATENCY_TIMER, 0x40);
@@ -770,10 +782,10 @@ void pci_bus_enable_resources(struct device *dev)
 	 * Enable I/O in command register if there is VGA card
 	 * connected with (even it does not claim I/O resource).
 	 */
-	if (dev->link_list->bridge_ctrl & PCI_BRIDGE_CTL_VGA)
+	if (dev->downstream->bridge_ctrl & PCI_BRIDGE_CTL_VGA)
 		dev->command |= PCI_COMMAND_IO;
 	ctrl = pci_read_config16(dev, PCI_BRIDGE_CONTROL);
-	ctrl |= dev->link_list->bridge_ctrl;
+	ctrl |= dev->downstream->bridge_ctrl;
 	ctrl |= (PCI_BRIDGE_CTL_PARITY | PCI_BRIDGE_CTL_SERR); /* Error check. */
 	printk(BIOS_DEBUG, "%s bridge ctrl <- %04x\n", dev_path(dev), ctrl);
 	pci_write_config16(dev, PCI_BRIDGE_CONTROL, ctrl);
@@ -831,6 +843,12 @@ static int should_run_oprom(struct device *dev, struct rom_header *rom)
 {
 	static int should_run = -1;
 
+	if (dev->upstream->segment_group) {
+		printk(BIOS_ERR, "Only option ROMs of devices in first PCI segment group can "
+				 "be run.\n");
+		return 0;
+	}
+
 	if (CONFIG(VENDORCODE_ELTAN_VBOOT))
 		if (rom != NULL)
 			if (!verified_boot_should_run_oprom(rom))
@@ -972,7 +990,7 @@ static void pci_bridge_vga_compat(struct bus *const bus)
 	pci_write_config16(bus->dev, PCI_BRIDGE_CONTROL, bridge_ctrl);
 
 	/* If the upstream bridge doesn't support VGA16, we don't have to check */
-	bus->no_vga16 |= bus->dev->bus->no_vga16;
+	bus->no_vga16 |= bus->dev->upstream->no_vga16;
 	if (bus->no_vga16)
 		return;
 
@@ -1197,7 +1215,7 @@ struct device *pci_probe_dev(struct device *dev, struct bus *bus,
 	if (!dev) {
 		struct device dummy;
 
-		dummy.bus = bus;
+		dummy.upstream = bus;
 		dummy.path.type = DEVICE_PATH_PCI;
 		dummy.path.pci.devfn = devfn;
 
@@ -1299,8 +1317,9 @@ struct device *pci_probe_dev(struct device *dev, struct bus *bus,
  */
 unsigned int pci_match_simple_dev(struct device *dev, pci_devfn_t sdev)
 {
-	return dev->bus->secondary == PCI_DEV2SEGBUS(sdev) &&
-			dev->path.pci.devfn == PCI_DEV2DEVFN(sdev);
+	return dev->upstream->secondary == PCI_DEV2BUS(sdev) &&
+		dev->upstream->segment_group == PCI_DEV2SEG(sdev) &&
+		dev->path.pci.devfn == PCI_DEV2DEVFN(sdev);
 }
 
 /**
@@ -1314,14 +1333,14 @@ unsigned int pci_match_simple_dev(struct device *dev, pci_devfn_t sdev)
  */
 uint16_t pci_find_cap_recursive(const struct device *dev, uint16_t cap)
 {
-	assert(dev->bus);
+	assert(dev->upstream);
 	uint16_t pos = pci_find_capability(dev, cap);
-	const struct device *bridge = dev->bus->dev;
+	const struct device *bridge = dev->upstream->dev;
 	while (bridge && (bridge->path.type == DEVICE_PATH_PCI)) {
-		assert(bridge->bus);
+		assert(bridge->upstream);
 		if (!pci_find_capability(bridge, cap))
 			return 0;
-		bridge = bridge->bus->dev;
+		bridge = bridge->upstream->dev;
 	}
 	return pos;
 }
@@ -1413,7 +1432,8 @@ void pci_scan_bus(struct bus *bus, unsigned int min_devfn,
 	struct device *dev, **prev;
 	int once = 0;
 
-	printk(BIOS_DEBUG, "PCI: %s for bus %02x\n", __func__, bus->secondary);
+	printk(BIOS_DEBUG, "PCI: %s for segment group %02x bus %02x\n", __func__,
+	       bus->segment_group, bus->secondary);
 
 	/* Maximum sane devfn is 0xFF. */
 	if (max_devfn > 0xff) {
@@ -1423,7 +1443,7 @@ void pci_scan_bus(struct bus *bus, unsigned int min_devfn,
 		max_devfn=0xff;
 	}
 
-	post_code(0x24);
+	post_code(POSTCODE_ENTER_PCI_SCAN_BUS);
 
 	if (pci_bus_only_one_child(bus))
 		max_devfn = MIN(max_devfn, 0x07);
@@ -1464,8 +1484,6 @@ void pci_scan_bus(struct bus *bus, unsigned int min_devfn,
 		}
 	}
 
-	post_code(0x25);
-
 	/*
 	 * Warn if any leftover static devices are found.
 	 * There's probably a problem in devicetree.cb.
@@ -1473,7 +1491,6 @@ void pci_scan_bus(struct bus *bus, unsigned int min_devfn,
 
 	prev = &bus->children;
 	for (dev = bus->children; dev; dev = dev->sibling) {
-
 		/*
 		 * If static device is not PCI then enable it here and don't
 		 * treat it as a leftover device.
@@ -1516,7 +1533,7 @@ void pci_scan_bus(struct bus *bus, unsigned int min_devfn,
 	 * side of any bridges that may be on this bus plus any devices.
 	 * Return how far we've got finding sub-buses.
 	 */
-	post_code(0x55);
+	post_code(POSTCODE_EXIT_PCI_SCAN_BUS);
 }
 
 typedef enum {
@@ -1528,12 +1545,25 @@ typedef enum {
 static void pci_bridge_route(struct bus *link, scan_state state)
 {
 	struct device *dev = link->dev;
-	struct bus *parent = dev->bus;
+	struct bus *parent = dev->upstream;
 	uint8_t primary, secondary, subordinate;
 
 	if (state == PCI_ROUTE_SCAN) {
 		link->secondary = parent->subordinate + 1;
 		link->subordinate = link->secondary + dev->hotplug_buses;
+		link->max_subordinate = parent->max_subordinate
+						? parent->max_subordinate
+						: (PCI_BUSES_PER_SEGMENT_GROUP - 1);
+		link->segment_group = parent->segment_group;
+	}
+
+	if (link->secondary > link->max_subordinate)
+		die("%s: No more busses available!\n", __func__);
+
+	/* This ought to only happen with hotplug buses.  */
+	if (link->subordinate > link->max_subordinate) {
+		printk(BIOS_WARNING, "%s: Limiting subordinate busses\n", __func__);
+		link->subordinate = link->max_subordinate;
 	}
 
 	if (state == PCI_ROUTE_CLOSE) {
@@ -1543,7 +1573,7 @@ static void pci_bridge_route(struct bus *link, scan_state state)
 	} else if (state == PCI_ROUTE_SCAN) {
 		primary = parent->secondary;
 		secondary = link->secondary;
-		subordinate = 0xff; /* MAX PCI_BUS number here */
+		subordinate = link->max_subordinate;
 	} else if (state == PCI_ROUTE_FINAL) {
 		primary = parent->secondary;
 		secondary = link->secondary;
@@ -1594,17 +1624,17 @@ void do_pci_scan_bridge(struct device *dev,
 
 	printk(BIOS_SPEW, "%s for %s\n", __func__, dev_path(dev));
 
-	if (dev->link_list == NULL) {
+	if (dev->downstream == NULL) {
 		struct bus *link;
 		link = malloc(sizeof(*link));
 		if (link == NULL)
 			die("Couldn't allocate a link!\n");
 		memset(link, 0, sizeof(*link));
 		link->dev = dev;
-		dev->link_list = link;
+		dev->downstream = link;
 	}
 
-	bus = dev->link_list;
+	bus = dev->downstream;
 
 	pci_bridge_vga_compat(bus);
 
@@ -1637,9 +1667,9 @@ void pci_scan_bridge(struct device *dev)
  *
  * @param dev Pointer to the domain.
  */
-void pci_domain_scan_bus(struct device *dev)
+void pci_host_bridge_scan_bus(struct device *dev)
 {
-	struct bus *link = dev->link_list;
+	struct bus *link = dev->downstream;
 	pci_scan_bus(link, PCI_DEVFN(0, 0), 0xff);
 }
 
@@ -1702,8 +1732,8 @@ static int swizzle_irq_pins(struct device *dev, struct device **parent_bridge)
 
 	/* While our current device has parent devices */
 	child = dev;
-	for (parent = child->bus->dev; parent; parent = parent->bus->dev) {
-		parent_bus = parent->bus->secondary;
+	for (parent = child->upstream->dev; parent; parent = parent->upstream->dev) {
+		parent_bus = parent->upstream->secondary;
 		parent_devfn = parent->path.pci.devfn;
 		child_devfn = child->path.pci.devfn;
 
@@ -1767,7 +1797,7 @@ int get_pci_irq_pins(struct device *dev, struct device **parent_bdg)
 	if (!(dev->enabled && (dev->path.type == DEVICE_PATH_PCI)))
 		return -1;
 
-	bus = dev->bus->secondary;
+	bus = dev->upstream->secondary;
 	devfn = dev->path.pci.devfn;
 
 	/* Get and validate the interrupt pin used. Only 1-4 are allowed */
@@ -1820,7 +1850,6 @@ void pci_assign_irqs(struct device *dev, const unsigned char pIntAtoD[4])
 	slot = dev->path.pci.devfn >> 3;
 
 	for (; dev ; dev = dev->sibling) {
-
 		if (dev->path.pci.devfn >> 3 != slot)
 			break;
 
