@@ -24,7 +24,9 @@
 #include <device/pnp.h>
 #include <drivers/tpm/tpm_ppi.h>
 #include <timer.h>
+
 #include "chip.h"
+#include "tpm.h"
 
 #define PREFIX "lpc_tpm: "
 
@@ -35,9 +37,9 @@
 		printk(BIOS_DEBUG, fmt, ##args);	\
 	}
 #define TPM_DEBUG_IO_READ(reg_, val_) \
-	TPM_DEBUG("Read reg 0x%x returns 0x%x\n", (reg_), (val_))
+	TPM_DEBUG("Read reg %#x returns %#x\n", (reg_), (val_))
 #define TPM_DEBUG_IO_WRITE(reg_, val_) \
-	TPM_DEBUG("Write reg 0x%x with 0x%x\n", (reg_), (val_))
+	TPM_DEBUG("Write reg %#x with %#x\n", (reg_), (val_))
 #define printf(x...) printk(BIOS_ERR, x)
 
 /* coreboot wrapper for TPM driver (end) */
@@ -55,6 +57,7 @@
 #define TIS_REG_STS                    0x18
 #define TIS_REG_BURST_COUNT            0x19
 #define TIS_REG_DATA_FIFO              0x24
+#define TIS_REG_INTF_ID                0x30
 #define TIS_REG_DID_VID                0xf00
 #define TIS_REG_RID                    0xf04
 
@@ -74,27 +77,16 @@
 #define TIS_ACCESS_REQUEST_USE         (1 << 1) /* 0x02 */
 #define TIS_ACCESS_TPM_ESTABLISHMENT   (1 << 0) /* 0x01 */
 
-/*
- * Error value returned if a tpm register does not enter the expected state
- * after continuous polling. No actual TPM register reading ever returns ~0,
- * so this value is a safe error indication to be mixed with possible status
- * register values.
- */
-#define TPM_TIMEOUT_ERR			(~0)
-
-/* Error value returned on various TPM driver errors */
-#define TPM_DRIVER_ERR		(~0)
-
  /* 1 second is plenty for anything TPM does.*/
 #define MAX_DELAY_US	USECS_PER_SEC
 
 /*
  * Structures defined below allow creating descriptions of TPM vendor/device
- * ID information for run time discovery. The only device the system knows
- * about at this time is Infineon slb9635
+ * ID information for run time discovery.
  */
 struct device_name {
 	u16 dev_id;
+	enum tpm_family family;
 	const char *const dev_name;
 };
 
@@ -105,37 +97,33 @@ struct vendor_name {
 };
 
 static const struct device_name atmel_devices[] = {
-	{0x3204, "AT97SC3204"},
+	{0x3204, TPM_1, "AT97SC3204"},
 	{0xffff}
 };
 
 static const struct device_name infineon_devices[] = {
-	{0x000b, "SLB9635 TT 1.2"},
-#if CONFIG(TPM2)
-	{0x001a, "SLB9665 TT 2.0"},
-	{0x001b, "SLB9670 TT 2.0"},
-	{0x001d, "SLB9672 TT 2.0"},
-#else
-	{0x001a, "SLB9660 TT 1.2"},
-	{0x001b, "SLB9670 TT 1.2"},
-#endif
+	{0x000b, TPM_1, "SLB9635 TT 1.2"},
+	{0x001a, TPM_1, "SLB9660 TT 1.2"},
+	{0x001b, TPM_1, "SLB9670 TT 1.2"},
+	{0x001a, TPM_2, "SLB9665 TT 2.0"},
+	{0x001b, TPM_2, "SLB9670 TT 2.0"},
+	{0x001d, TPM_2, "SLB9672 TT 2.0"},
 	{0xffff}
 };
 
 static const struct device_name nuvoton_devices[] = {
-	{0x00fe, "NPCT420AA V2"},
+	{0x00fe, TPM_1, "NPCT420AA V2"},
 	{0xffff}
 };
 
 static const struct device_name stmicro_devices[] = {
-	{0x0000, "ST33ZP24" },
+	{0x0000, TPM_1, "ST33ZP24" },
 	{0xffff}
 };
 
 static const struct device_name swtpm_devices[] = {
-#if CONFIG(TPM2)
-	{0x0001, "SwTPM 2.0" },
-#endif
+	{0x0001, TPM_1, "SwTPM 1.2" },
+	{0x0001, TPM_2, "SwTPM 2.0" },
 	{0xffff}
 };
 
@@ -201,6 +189,20 @@ static inline void tpm_write_access(u8 data, int locality)
 	write8(TIS_REG(locality, TIS_REG_ACCESS), data);
 }
 
+static inline u32 tpm_read_intf_cap(int locality)
+{
+	u32 value = read32(TIS_REG(locality, TIS_REG_INTF_CAPABILITY));
+	TPM_DEBUG_IO_READ(TIS_REG_INTF_CAPABILITY, value);
+	return value;
+}
+
+static inline u32 tpm_read_intf_id(int locality)
+{
+	u32 value = read32(TIS_REG(locality, TIS_REG_INTF_ID));
+	TPM_DEBUG_IO_READ(TIS_REG_INTF_ID, value);
+	return value;
+}
+
 static inline u32 tpm_read_did_vid(int locality)
 {
 	u32 value = read32(TIS_REG(locality, TIS_REG_DID_VID));
@@ -248,9 +250,9 @@ static inline u32 tpm_read_int_polarity(int locality)
  * @mask - bitmask for the bitfield(s) to watch
  * @expected - value the field(s) are supposed to be set to
  *
- * Returns 0 on success or TPM_TIMEOUT_ERR on timeout.
+ * Returns TPM_SUCCESS on success or TPM_CB_TIMEOUT on timeout.
  */
-static int tis_wait_sts(int locality, u8 mask, u8 expected)
+static tpm_result_t tis_wait_sts(int locality, u8 mask, u8 expected)
 {
 	struct stopwatch sw;
 
@@ -258,24 +260,24 @@ static int tis_wait_sts(int locality, u8 mask, u8 expected)
 	do {
 		u8 value = tpm_read_status(locality);
 		if ((value & mask) == expected)
-			return 0;
+			return TPM_SUCCESS;
 		udelay(1);
 	} while (!stopwatch_expired(&sw));
-	return TPM_TIMEOUT_ERR;
+	return TPM_CB_TIMEOUT;
 }
 
-static inline int tis_wait_ready(int locality)
+static inline tpm_result_t tis_wait_ready(int locality)
 {
 	return tis_wait_sts(locality, TIS_STS_COMMAND_READY,
 	                    TIS_STS_COMMAND_READY);
 }
 
-static inline int tis_wait_valid(int locality)
+static inline tpm_result_t tis_wait_valid(int locality)
 {
 	return tis_wait_sts(locality, TIS_STS_VALID, TIS_STS_VALID);
 }
 
-static inline int tis_wait_valid_data(int locality)
+static inline tpm_result_t tis_wait_valid_data(int locality)
 {
 	const u8 has_data = TIS_STS_DATA_AVAILABLE | TIS_STS_VALID;
 	return tis_wait_sts(locality, has_data, has_data);
@@ -302,9 +304,9 @@ static inline int tis_expect_data(int locality)
  * @mask - bitmask for the bitfield(s) to watch
  * @expected - value the field(s) are supposed to be set to
  *
- * Returns 0 on success or TPM_TIMEOUT_ERR on timeout.
+ * Returns TPM_SUCCESS on success or TPM_CB_TIMEOUT on timeout.
  */
-static int tis_wait_access(int locality, u8 mask, u8 expected)
+static tpm_result_t tis_wait_access(int locality, u8 mask, u8 expected)
 {
 	struct stopwatch sw;
 
@@ -312,13 +314,13 @@ static int tis_wait_access(int locality, u8 mask, u8 expected)
 	do {
 		u8 value = tpm_read_access(locality);
 		if ((value & mask) == expected)
-			return 0;
+			return TPM_SUCCESS;
 		udelay(1);
 	} while (!stopwatch_expired(&sw));
-	return TPM_TIMEOUT_ERR;
+	return TPM_CB_TIMEOUT;
 }
 
-static inline int tis_wait_received_access(int locality)
+static inline tpm_result_t tis_wait_received_access(int locality)
 {
 	return tis_wait_access(locality, TIS_ACCESS_ACTIVE_LOCALITY,
 	                       TIS_ACCESS_ACTIVE_LOCALITY);
@@ -345,10 +347,8 @@ static inline void tis_request_access(int locality)
  * In practice not all TPMs behave the same so it is necessary to be
  * flexible when trying to set command ready.
  *
- * Returns 0 on success if the TPM is ready for transactions.
- * Returns TPM_TIMEOUT_ERR if the command ready bit does not get set.
  */
-static int tis_command_ready(u8 locality)
+static tpm_result_t tis_command_ready(u8 locality)
 {
 	u32 status;
 
@@ -360,7 +360,7 @@ static int tis_command_ready(u8 locality)
 
 	/* Check if command ready is set yet */
 	if (status & TIS_STS_COMMAND_READY)
-		return 0;
+		return TPM_SUCCESS;
 
 	/* 2nd attempt to set command ready */
 	tpm_write_status(TIS_STS_COMMAND_READY, locality);
@@ -369,27 +369,61 @@ static int tis_command_ready(u8 locality)
 }
 
 /*
+ * pc80_tis_probe()
+ *
  * Probe the TPM device and try determining its manufacturer/device name.
  *
- * Returns 0 on success (the device is found or was found during an earlier
- * invocation) or TPM_DRIVER_ERR if the device is not found.
+ * Returns TPM_SUCCESS on success (the device is found or was found during
+ * an earlier invocation) or TPM_CB_FAIL if the device is not found.
  */
-static u32 tis_probe(void)
+static tpm_result_t pc80_tpm_probe(enum tpm_family *family)
 {
-	const char *device_name = "unknown";
-	const char *vendor_name = device_name;
-	const struct device_name *dev;
-	u32 didvid;
-	u16 vid, did;
-	int i;
+	static enum tpm_family tpm_family;
 
-	if (vendor_dev_id)
-		return 0;  /* Already probed. */
+	const char *device_name = NULL;
+	const char *vendor_name = NULL;
+	const struct device_name *dev;
+	u32 didvid, intf_id;
+	u16 vid, did;
+	u8 locality = 0, intf_type;
+	int i;
+	const char *family_str;
+
+	if (vendor_dev_id) {
+		if (family != NULL)
+			*family = tpm_family;
+		return TPM_SUCCESS;  /* Already probed. */
+	}
 
 	didvid = tpm_read_did_vid(0);
 	if (!didvid || (didvid == 0xffffffff)) {
 		printf("%s: No TPM device found\n", __func__);
-		return TPM_DRIVER_ERR;
+		return TPM_CB_FAIL;
+	}
+
+	intf_id = tpm_read_intf_id(locality);
+	intf_type = (intf_id & 0xf);
+	if (intf_type == 0xf) {
+		u32 intf_cap = tpm_read_intf_cap(locality);
+		u8 intf_version = (intf_cap >> 28) & 0x7;
+		switch (intf_version) {
+		case 0:
+		case 2:
+			tpm_family = TPM_1;
+			break;
+		case 3:
+			tpm_family = TPM_2;
+			break;
+		default:
+			printf("%s: Unexpected TPM interface version: %d\n", __func__,
+			       intf_version);
+			return TPM_CB_PROBE_FAILURE;
+		}
+	} else if (intf_type == 0) {
+		tpm_family = TPM_2;
+	} else {
+		printf("%s: Unexpected TPM interface type: %d\n", __func__, intf_type);
+		return TPM_CB_PROBE_FAILURE;
 	}
 
 	vendor_dev_id = didvid;
@@ -398,15 +432,14 @@ static u32 tis_probe(void)
 	did = (didvid >> 16) & 0xffff;
 	for (i = 0; i < ARRAY_SIZE(vendor_names); i++) {
 		int j = 0;
-		u16 known_did;
 		if (vid == vendor_names[i].vendor_id) {
 			vendor_name = vendor_names[i].vendor_name;
 		} else {
 			continue;
 		}
 		dev = &vendor_names[i].dev_names[j];
-		while ((known_did = dev->dev_id) != 0xffff) {
-			if (known_did == did) {
+		while (dev->dev_id != 0xffff) {
+			if (dev->dev_id == did && dev->family == tpm_family) {
 				device_name = dev->dev_name;
 				break;
 			}
@@ -415,9 +448,21 @@ static u32 tis_probe(void)
 		}
 		break;
 	}
-	/* this will have to be converted into debug printout */
-	printk(BIOS_INFO, "Found TPM %s by %s\n", device_name, vendor_name);
-	return 0;
+
+	family_str = (tpm_family == TPM_1 ? "TPM 1.2" : "TPM 2.0");
+	if (vendor_name == NULL) {
+		printk(BIOS_INFO, "Found %s 0x%04x by 0x%04x\n", family_str, did, vid);
+	} else if (device_name == NULL) {
+		printk(BIOS_INFO, "Found %s 0x%04x by %s (0x%04x)\n", family_str, did,
+		       vendor_name, vid);
+	} else {
+		printk(BIOS_INFO, "Found %s %s (0x%04x) by %s (0x%04x)\n", family_str,
+		       device_name, did, vendor_name, vid);
+	}
+
+	if (family != NULL)
+		*family = tpm_family;
+	return TPM_SUCCESS;
 }
 
 /*
@@ -428,19 +473,21 @@ static u32 tis_probe(void)
  * @data - address of the data to send, byte by byte
  * @len - length of the data to send
  *
- * Returns 0 on success, TPM_DRIVER_ERR on error (in case the device does
+ * Returns TPM_SUCCESS on success, TPM_CB_FAIL on error (in case the device does
  * not accept the entire command).
  */
-static u32 tis_senddata(const u8 *const data, u32 len)
+static tpm_result_t tis_senddata(const u8 *const data, u32 len)
 {
 	u32 offset = 0;
 	u16 burst = 0;
 	u8 locality = 0;
+	tpm_result_t rc = TPM_SUCCESS;
 
-	if (tis_wait_ready(locality)) {
-		printf("%s:%d - failed to get 'command_ready' status\n",
-		       __FILE__, __LINE__);
-		return TPM_DRIVER_ERR;
+	rc = tis_wait_ready(locality);
+	if (rc) {
+		printf("%s:%d - failed to get 'command_ready' status with error %#x\n",
+		       __FILE__, __LINE__, rc);
+		return rc;
 	}
 	burst = tpm_read_burst_count(locality);
 
@@ -454,7 +501,7 @@ static u32 tis_senddata(const u8 *const data, u32 len)
 			if (stopwatch_expired(&sw)) {
 				printf("%s:%d failed to feed %u bytes of %u\n",
 				       __FILE__, __LINE__, len - offset, len);
-				return TPM_DRIVER_ERR;
+				return TPM_CB_TIMEOUT;
 			}
 			udelay(1);
 			burst = tpm_read_burst_count(locality);
@@ -473,10 +520,11 @@ static u32 tis_senddata(const u8 *const data, u32 len)
 		while (count--)
 			tpm_write_data(data[offset++], locality);
 
-		if (tis_wait_valid(locality) || !tis_expect_data(locality)) {
-			printf("%s:%d TPM command feed overflow\n",
-			       __FILE__, __LINE__);
-			return TPM_DRIVER_ERR;
+		rc = tis_wait_valid(locality);
+		if (rc || !tis_expect_data(locality)) {
+			printf("%s:%d TPM command feed overflow with error %#x\n",
+			       __FILE__, __LINE__, rc);
+			return rc ? rc : TPM_CB_FAIL;
 		}
 
 		burst = tpm_read_burst_count(locality);
@@ -496,16 +544,17 @@ static u32 tis_senddata(const u8 *const data, u32 len)
 	 * Verify that TPM does not expect any more data as part of this
 	 * command.
 	 */
-	if (tis_wait_valid(locality) || tis_expect_data(locality)) {
-		printf("%s:%d unexpected TPM status 0x%x\n",
-		       __FILE__, __LINE__, tpm_read_status(locality));
-		return TPM_DRIVER_ERR;
+	rc = tis_wait_valid(locality);
+	if (rc || tis_expect_data(locality)) {
+		printf("%s:%d unexpected TPM error %#x with status %#x\n",
+		       __FILE__, __LINE__, rc, tpm_read_status(locality));
+		return rc ? rc : TPM_CB_FAIL;
 	}
 
 	/* OK, sitting pretty, let's start the command execution. */
 	tpm_write_status(TIS_STS_TPM_GO, locality);
 
-	return 0;
+	return TPM_SUCCESS;
 }
 
 /*
@@ -516,22 +565,25 @@ static u32 tis_senddata(const u8 *const data, u32 len)
  * @buffer - address where to read the response, byte by byte.
  * @len - pointer to the size of buffer
  *
- * On success stores the number of received bytes to len and returns 0. On
- * errors (misformatted TPM data or synchronization problems) returns
- * TPM_DRIVER_ERR.
+ * On success stores the number of received bytes to len and returns
+ * TPM_SUCCESS. On errors (misformatted TPM data or synchronization
+ * problems) returns TPM_CB_FAIL.
  */
-static u32 tis_readresponse(u8 *buffer, size_t *len)
+static tpm_result_t tis_readresponse(u8 *buffer, size_t *len)
 {
 	u16 burst_count;
 	u32 offset = 0;
 	u8 locality = 0;
 	u32 expected_count = *len;
 	int max_cycles = 0;
+	tpm_result_t rc = TPM_SUCCESS;
 
 	/* Wait for the TPM to process the command */
-	if (tis_wait_valid_data(locality)) {
-		printf("%s:%d failed processing command\n", __FILE__, __LINE__);
-		return TPM_DRIVER_ERR;
+	rc = tis_wait_valid_data(locality);
+	if (rc) {
+		printf("%s:%d failed processing command with error %#x\n",
+			   __FILE__, __LINE__, rc);
+		return rc;
 	}
 
 	do {
@@ -539,7 +591,7 @@ static u32 tis_readresponse(u8 *buffer, size_t *len)
 			if (max_cycles++ == MAX_DELAY_US) {
 				printf("%s:%d TPM stuck on read\n",
 				       __FILE__, __LINE__);
-				return TPM_DRIVER_ERR;
+				return TPM_CB_FAIL;
 			}
 			udelay(1);
 		}
@@ -567,16 +619,17 @@ static u32 tis_readresponse(u8 *buffer, size_t *len)
 					printf("%s:%d bad response size %u\n",
 					       __FILE__, __LINE__,
 					       expected_count);
-					return TPM_DRIVER_ERR;
+					return TPM_CB_FAIL;
 				}
 			}
 		}
 
 		/* Wait for the next portion */
-		if (tis_wait_valid(locality)) {
-			printf("%s:%d failed to read response\n",
-			       __FILE__, __LINE__);
-			return TPM_DRIVER_ERR;
+		rc = tis_wait_valid(locality);
+		if (rc) {
+			printf("%s:%d failed to read response with error %#x\n",
+			       __FILE__, __LINE__, rc);
+			return rc;
 		}
 
 		if (offset == expected_count)
@@ -594,63 +647,50 @@ static u32 tis_readresponse(u8 *buffer, size_t *len)
 
 	/* * Make sure we indeed read all there was. */
 	if (tis_has_valid_data(locality)) {
-		printf("%s:%d wrong receive status: %x %u bytes left\n",
+		printf("%s:%d wrong receive status: %#x %u bytes left\n",
 		       __FILE__, __LINE__, tpm_read_status(locality),
 	               tpm_read_burst_count(locality));
-		return TPM_DRIVER_ERR;
+		return TPM_CB_FAIL;
 	}
 
 	/* Tell the TPM that we are done. */
-	if (tis_command_ready(locality) == TPM_TIMEOUT_ERR)
-		return TPM_DRIVER_ERR;
+	rc = tis_command_ready(locality);
+	if (rc)
+		return rc;
 
 	*len = offset;
-	return 0;
+	return TPM_SUCCESS;
 }
 
 /*
- * tis_init()
- *
- * Initialize the TPM device. Returns 0 on success or TPM_DRIVER_ERR on
- * failure (in case device probing did not succeed).
- */
-int tis_init(void)
-{
-	if (tis_probe())
-		return TPM_DRIVER_ERR;
-	return 0;
-}
-
-/*
- * tis_open()
+ * pc80_tis_open()
  *
  * Requests access to locality 0 for the caller.
  *
- * Returns 0 on success, TPM_DRIVER_ERR on failure.
+ * Returns TPM_SUCCESS on success, TSS Error on failure.
  */
-int tis_open(void)
+static tpm_result_t pc80_tis_open(void)
 {
 	u8 locality = 0; /* we use locality zero for everything */
+	tpm_result_t rc = TPM_SUCCESS;
 
 	if (!tis_has_access(locality)) {
 		/* request access to locality */
 		tis_request_access(locality);
 
 		/* did we get a lock? */
-		if (tis_wait_received_access(locality)) {
-			printf("%s:%d - failed to lock locality %u\n",
-			__FILE__, __LINE__, locality);
-			return TPM_DRIVER_ERR;
+		rc = tis_wait_received_access(locality);
+		if (rc) {
+			printf("%s:%d - failed to lock locality %u with error %#x\n",
+			__FILE__, __LINE__, locality, rc);
+			return rc;
 		}
 
 		/* Certain TPMs seem to need some delay here or they hang... */
 		udelay(10);
 	}
 
-	if (tis_command_ready(locality) == TPM_TIMEOUT_ERR)
-		return TPM_DRIVER_ERR;
-
-	return 0;
+	return tis_command_ready(locality);
 }
 
 /*
@@ -663,21 +703,42 @@ int tis_open(void)
  * @recvbuf - memory to save the response to
  * @recv_len - pointer to the size of the response buffer
  *
- * Returns 0 on success (and places the number of response bytes at recv_len)
- * or TPM_DRIVER_ERR on failure.
+ * Returns TPM_SUCCESS on success (and places the number of response bytes
+ * at recv_len) or TPM_CB_FAIL on failure.
  */
-int tis_sendrecv(const uint8_t *sendbuf, size_t send_size,
-		 uint8_t *recvbuf, size_t *recv_len)
+static tpm_result_t pc80_tpm_sendrecv(const uint8_t *sendbuf, size_t send_size,
+				      uint8_t *recvbuf, size_t *recv_len)
 {
-	if (tis_senddata(sendbuf, send_size)) {
-		printf("%s:%d failed sending data to TPM\n",
-		       __FILE__, __LINE__);
-		return TPM_DRIVER_ERR;
+	tpm_result_t rc = tis_senddata(sendbuf, send_size);
+	if (rc) {
+		printf("%s:%d failed sending data to TPM with error %#x\n",
+		       __FILE__, __LINE__, rc);
+		return rc;
 	}
 
 	return tis_readresponse(recvbuf, recv_len);
 }
 
+/*
+ * pc80_tis_probe()
+ *
+ * Probe for the TPM device and set it up for use within locality 0.
+ *
+ * @tpm_family - pointer to tpm_family which is set to TPM family of the device.
+ *
+ * Returns pointer to send-receive function on success or NULL on failure.
+ */
+tis_sendrecv_fn pc80_tis_probe(enum tpm_family *family)
+{
+	if (pc80_tpm_probe(family))
+		return NULL;
+
+	if (pc80_tis_open())
+		return NULL;
+
+	return &pc80_tpm_sendrecv;
+}
+
 /*
  * tis_setup_interrupt()
  *
@@ -691,14 +752,15 @@ int tis_sendrecv(const uint8_t *sendbuf, size_t send_size,
  * @vector - TPM interrupt vector
  * @polarity - TPM interrupt polarity
  *
- * Returns 0 on success, TPM_DRIVER_ERR on failure.
+ * Returns TPM_SUCCESS on success, TPM_CB_FAIL on failure.
  */
-static int tis_setup_interrupt(int vector, int polarity)
+static tpm_result_t tis_setup_interrupt(int vector, int polarity)
 {
 	u8 locality = 0;
+	tpm_result_t rc = tlcl_lib_init();
 
-	if (tlcl_lib_init())
-		return TPM_DRIVER_ERR;
+	if (rc)
+		return rc;
 
 	/* Set TPM interrupt vector */
 	tpm_write_int_vector(vector, locality);
@@ -706,7 +768,7 @@ static int tis_setup_interrupt(int vector, int polarity)
 	/* Set TPM interrupt polarity and disable interrupts */
 	tpm_write_int_polarity(polarity, locality);
 
-	return 0;
+	return TPM_SUCCESS;
 }
 
 static void lpc_tpm_read_resources(struct device *dev)
@@ -717,9 +779,11 @@ static void lpc_tpm_read_resources(struct device *dev)
 
 static void lpc_tpm_set_resources(struct device *dev)
 {
-	tpm_config_t *config = (tpm_config_t *)dev->chip_info;
+	struct drivers_pc80_tpm_config *config;
 	DEVTREE_CONST struct resource *res;
 
+	config = (struct drivers_pc80_tpm_config *)dev->chip_info;
+
 	for (res = dev->resource_list; res; res = res->next) {
 		if (!(res->flags & IORESOURCE_ASSIGNED))
 			continue;
@@ -749,7 +813,7 @@ static void lpc_tpm_fill_ssdt(const struct device *dev)
 	acpigen_write_scope(path);
 	acpigen_write_device(acpi_device_name(dev));
 
-	if (CONFIG(TPM2)) {
+	if (tlcl_get_family() == TPM_2) {
 		acpigen_write_name_string("_HID", "MSFT0101");
 		acpigen_write_name_string("_CID", "MSFT0101");
 	} else {
@@ -832,18 +896,30 @@ static struct device_operations lpc_tpm_ops = {
 #endif
 };
 
+static struct device_operations noop_tpm_ops = {
+	.read_resources = noop_read_resources,
+	.set_resources = noop_set_resources,
+};
+
 static struct pnp_info pnp_dev_info[] = {
 	{ .flags = PNP_IRQ0 }
 };
 
 static void enable_dev(struct device *dev)
 {
-	if (CONFIG(TPM))
-		pnp_enable_devices(dev, &lpc_tpm_ops,
-			ARRAY_SIZE(pnp_dev_info), pnp_dev_info);
+	if (CONFIG(TPM)) {
+		if (pc80_tis_probe(NULL) == NULL) {
+			dev->enabled = 0;
+			return;
+		}
+
+		pnp_enable_devices(dev, &lpc_tpm_ops, ARRAY_SIZE(pnp_dev_info), pnp_dev_info);
+	} else {
+		pnp_enable_devices(dev, &noop_tpm_ops, ARRAY_SIZE(pnp_dev_info), pnp_dev_info);
+	}
 }
 
 struct chip_operations drivers_pc80_tpm_ops = {
-	CHIP_NAME("LPC TPM")
+	.name = "LPC TPM",
 	.enable_dev = enable_dev
 };
