@@ -12,12 +12,28 @@
 
 #include "spi_flash_internal.h"
 
+#if CONFIG(SPI_FLASH_FORCE_4_BYTE_ADDR_MODE)
+#define ADDR_MOD 1
+#else
+#define ADDR_MOD 0
+#endif
+
+#define SPI_FLASH_EXIT_4BYTE_STAGE	\
+	(ENV_INITIAL_STAGE || CONFIG(BOOT_DEVICE_MEMORY_MAPPED))
+
 static void spi_flash_addr(u32 addr, u8 *cmd)
 {
 	/* cmd[0] is actual command */
-	cmd[1] = addr >> 16;
-	cmd[2] = addr >> 8;
-	cmd[3] = addr >> 0;
+	if (CONFIG(SPI_FLASH_FORCE_4_BYTE_ADDR_MODE)) {
+		cmd[1] = addr >> 24;
+		cmd[2] = addr >> 16;
+		cmd[3] = addr >> 8;
+		cmd[4] = addr >> 0;
+	} else {
+		cmd[1] = addr >> 16;
+		cmd[2] = addr >> 8;
+		cmd[3] = addr >> 0;
+	}
 }
 
 static int do_spi_flash_cmd(const struct spi_slave *spi, const u8 *dout,
@@ -142,29 +158,29 @@ int spi_flash_cmd_write(const struct spi_slave *spi, const u8 *cmd,
 int spi_flash_cmd_read(const struct spi_flash *flash, u32 offset,
 				  size_t len, void *buf)
 {
-	u8 cmd[5];
+	u8 cmd[5 + ADDR_MOD];
 	int ret, cmd_len;
 	int (*do_cmd)(const struct spi_slave *spi, const u8 *din,
 		      size_t in_bytes, void *out, size_t out_bytes);
 
 	if (CONFIG(SPI_FLASH_NO_FAST_READ)) {
-		cmd_len = 4;
+		cmd_len = 4 + ADDR_MOD;
 		cmd[0] = CMD_READ_ARRAY_SLOW;
 		do_cmd = do_spi_flash_cmd;
 	} else if (flash->flags.dual_io && flash->spi.ctrlr->xfer_dual) {
-		cmd_len = 5;
+		cmd_len = 5 + ADDR_MOD;
 		cmd[0] = CMD_READ_FAST_DUAL_IO;
-		cmd[4] = 0;
+		cmd[4 + ADDR_MOD] = 0;
 		do_cmd = do_dual_io_cmd;
 	} else if (flash->flags.dual_output && flash->spi.ctrlr->xfer_dual) {
-		cmd_len = 5;
+		cmd_len = 5 + ADDR_MOD;
 		cmd[0] = CMD_READ_FAST_DUAL_OUTPUT;
-		cmd[4] = 0;
+		cmd[4 + ADDR_MOD] = 0;
 		do_cmd = do_dual_output_cmd;
 	} else {
-		cmd_len = 5;
+		cmd_len = 5 + ADDR_MOD;
 		cmd[0] = CMD_READ_ARRAY_FAST;
-		cmd[4] = 0;
+		cmd[4 + ADDR_MOD] = 0;
 		do_cmd = do_spi_flash_cmd;
 	}
 
@@ -229,7 +245,7 @@ int spi_flash_cmd_erase(const struct spi_flash *flash, u32 offset, size_t len)
 {
 	u32 start, end, erase_size;
 	int ret = -1;
-	u8 cmd[4];
+	u8 cmd[4 + ADDR_MOD];
 
 	erase_size = flash->sector_size;
 	if (offset % erase_size || len % erase_size) {
@@ -249,10 +265,15 @@ int spi_flash_cmd_erase(const struct spi_flash *flash, u32 offset, size_t len)
 		spi_flash_addr(offset, cmd);
 		offset += erase_size;
 
-#if CONFIG(DEBUG_SPI_FLASH)
-		printk(BIOS_SPEW, "SF: erase %2x %2x %2x %2x (%x)\n", cmd[0], cmd[1],
-		      cmd[2], cmd[3], offset);
-#endif
+		if (CONFIG(DEBUG_SPI_FLASH)) {
+			if (ADDR_MOD)
+				printk(BIOS_SPEW, "SF: erase %2x %2x %2x %2x %2x (%x)\n",
+					cmd[0], cmd[1], cmd[2], cmd[3], cmd[4], offset);
+			else
+				printk(BIOS_SPEW, "SF: erase %2x %2x %2x %2x (%x)\n",
+					cmd[0], cmd[1], cmd[2], cmd[3], offset);
+		}
+
 		ret = spi_flash_cmd(&flash->spi, CMD_WRITE_ENABLE, NULL, 0);
 		if (ret)
 			goto out;
@@ -286,7 +307,7 @@ int spi_flash_cmd_write_page_program(const struct spi_flash *flash, u32 offset,
 	size_t chunk_len;
 	size_t actual;
 	int ret = 0;
-	u8 cmd[4];
+	u8 cmd[4 + ADDR_MOD];
 
 	page_size = flash->page_size;
 	cmd[0] = flash->pp_cmd;
@@ -298,9 +319,16 @@ int spi_flash_cmd_write_page_program(const struct spi_flash *flash, u32 offset,
 
 		spi_flash_addr(offset, cmd);
 		if (CONFIG(DEBUG_SPI_FLASH)) {
-			printk(BIOS_SPEW, "PP: %p => cmd = { 0x%02x 0x%02x%02x%02x } chunk_len = %zu\n",
-				buf + actual, cmd[0], cmd[1], cmd[2], cmd[3],
-				chunk_len);
+			if (ADDR_MOD)
+				printk(BIOS_SPEW,
+					"PP: %p => cmd = { 0x%02x 0x%02x%02x%02x%02x } chunk_len = %zu\n",
+					buf + actual, cmd[0], cmd[1], cmd[2], cmd[3], cmd[4],
+					chunk_len);
+			else
+				printk(BIOS_SPEW,
+					"PP: %p => cmd = { 0x%02x 0x%02x%02x%02x } chunk_len = %zu\n",
+					buf + actual, cmd[0], cmd[1], cmd[2], cmd[3],
+					chunk_len);
 		}
 
 		ret = spi_flash_cmd(&flash->spi, flash->wren_cmd, NULL, 0);
@@ -369,6 +397,9 @@ static const struct spi_flash_vendor_info *spi_flash_vendors[] = {
 #if CONFIG(SPI_FLASH_WINBOND)
 	&spi_flash_winbond_vi,
 #endif
+#if CONFIG(SPI_FLASH_ISSI)
+	&spi_flash_issi_vi,
+#endif
 };
 #define IDCODE_LEN 5
 
@@ -442,6 +473,7 @@ static int find_match(const struct spi_slave *spi, struct spi_flash *flash,
 		return fill_spi_flash(spi, flash, vi, part);
 	}
 
+	printk(BIOS_WARNING, "SF: no match for ID %04x %04x\n", id[0], id[1]);
 	return -1;
 }
 
@@ -522,8 +554,10 @@ int spi_flash_probe(unsigned int bus, unsigned int cs, struct spi_flash *flash)
 			CONFIG_ROM_SIZE);
 	}
 
-	if (CONFIG(SPI_FLASH_EXIT_4_BYTE_ADDR_MODE) && ENV_INITIAL_STAGE)
+	if (CONFIG(SPI_FLASH_EXIT_4_BYTE_ADDR_MODE) && SPI_FLASH_EXIT_4BYTE_STAGE) {
+		printk(BIOS_DEBUG, "SF: Exiting 4-byte addressing mode\n");
 		spi_flash_cmd(&flash->spi, CMD_EXIT_4BYTE_ADDR_MODE, NULL, 0);
+	}
 
 	return 0;
 }
@@ -576,12 +610,12 @@ int spi_flash_status(const struct spi_flash *flash, u8 *reg)
 int spi_flash_is_write_protected(const struct spi_flash *flash,
 				 const struct region *region)
 {
-	struct region flash_region = { 0 };
+	struct region flash_region;
 
 	if (!flash || !region)
 		return -1;
 
-	flash_region.size = flash->size;
+	flash_region = region_create(0, flash->size);
 
 	if (!region_is_subregion(&flash_region, region))
 		return -1;
@@ -599,13 +633,13 @@ int spi_flash_set_write_protected(const struct spi_flash *flash,
 				  const struct region *region,
 				  const enum spi_flash_status_reg_lockdown mode)
 {
-	struct region flash_region = { 0 };
+	struct region flash_region;
 	int ret;
 
 	if (!flash)
 		return -1;
 
-	flash_region.size = flash->size;
+	flash_region = region_create(0, flash->size);
 
 	if (!region_is_subregion(&flash_region, region))
 		return -1;
@@ -721,12 +755,12 @@ int spi_flash_ctrlr_protect_region(const struct spi_flash *flash,
 				   const enum ctrlr_prot_type type)
 {
 	const struct spi_ctrlr *ctrlr;
-	struct region flash_region = { 0 };
+	struct region flash_region;
 
 	if (!flash)
 		return -1;
 
-	flash_region.size = flash->size;
+	flash_region = region_create(0, flash->size);
 
 	if (!region_is_subregion(&flash_region, region))
 		return -1;
