@@ -723,6 +723,19 @@ int google_chromeec_cbi_get_ssfc(uint32_t *ssfc)
 	return cbi_get_uint32(ssfc, CBI_TAG_SSFC);
 }
 
+bool google_chromeec_get_ucsi_enabled(void)
+{
+	int rv;
+
+	rv = google_chromeec_check_feature(EC_FEATURE_UCSI_PPM);
+	if (rv < 0) {
+		printk(BIOS_INFO, "Cannot check if EC_FEATURE_UCSI_PPM is available: status = %d\n", rv);
+		return false;
+	}
+
+	return rv != 0;
+}
+
 static int cbi_get_string(char *buf, size_t bufsize, uint32_t tag)
 {
 	struct ec_params_get_cbi params = {
@@ -1181,6 +1194,11 @@ int google_chromeec_get_pd_port_caps(int port,
 void google_chromeec_init(void)
 {
 	google_chromeec_log_uptimeinfo();
+
+	/* Enable automatic fan control */
+	if (CONFIG(EC_GOOGLE_CHROMEEC_AUTO_FAN_CTRL)) {
+		ec_cmd_thermal_auto_fan_ctrl(PLAT_EC);
+	}
 }
 
 int google_ec_running_ro(void)
@@ -1522,3 +1540,29 @@ int google_chromeec_regulator_get_voltage(uint32_t index, uint32_t *voltage_mv)
 	*voltage_mv = resp.voltage_mv;
 	return 0;
 }
+
+void google_chromeec_clear_ec_ap_idle(void)
+{
+	/* Send EC command to clear AP_IDLE flag */
+	if (!google_chromeec_reboot(EC_REBOOT_NO_OP, EC_REBOOT_FLAG_CLEAR_AP_IDLE |
+				    EC_REBOOT_FLAG_ON_AP_SHUTDOWN))
+		printk(BIOS_INFO, "Successfully clear AP_IDLE flag\n");
+	else
+		printk(BIOS_ERR, "Failed to clear EC AP_IDLE flag\n");
+}
+
+bool google_chromeec_is_battery_present_and_above_critical_threshold(void)
+{
+	struct ec_params_battery_dynamic_info params = {
+		.index = 0,
+	};
+	struct ec_response_battery_dynamic_info resp;
+
+	if (ec_cmd_battery_get_dynamic(PLAT_EC, &params, &resp) == 0) {
+		/* Check if battery is present and LEVEL_CRITICAL is not set */
+		if (resp.flags && !(resp.flags & EC_BATT_FLAG_LEVEL_CRITICAL))
+			return true;
+	}
+
+	return false;
+}
