@@ -6,6 +6,14 @@
 #include <types.h>
 #include <memory_info.h>
 
+#define update_max(len, max_len, stmt)		\
+	do {					\
+		int tmp = stmt;			\
+						\
+		max_len = MAX(max_len, tmp);	\
+		len += tmp;			\
+	} while (0)
+
 unsigned long smbios_write_tables(unsigned long start);
 int smbios_add_string(u8 *start, const char *str);
 int smbios_string_table_len(u8 *start);
@@ -24,7 +32,8 @@ int smbios_write_type9(unsigned long *current, int *handle,
 			const enum slot_data_bus_bandwidth bandwidth,
 			const enum misc_slot_usage usage,
 			const enum misc_slot_length length,
-			const u16 id, u8 slot_char1, u8 slot_char2, u8 bus, u8 dev_func);
+			const u16 id, u8 slot_char1, u8 slot_char2,
+			u8 segment_group, u8 bus, u8 dev_func);
 enum smbios_bmc_interface_type;
 int smbios_write_type38(unsigned long *current, int *handle,
 			const enum smbios_bmc_interface_type interface_type,
@@ -32,7 +41,7 @@ int smbios_write_type38(unsigned long *current, int *handle,
 			const u64 base_addr, const u8 base_modifier,
 			const u8 irq);
 int smbios_write_type41(unsigned long *current, int *handle,
-			const char *name, u8 instance, u16 segment,
+			const char *name, u8 instance, u16 segment_group,
 			u8 bus, u8 device, u8 function, u8 device_type);
 enum smbios_temp_location;
 enum smbios_temp_status;
@@ -61,9 +70,12 @@ const char *smbios_system_version(void);
 void smbios_system_set_uuid(u8 *uuid);
 const char *smbios_system_sku(void);
 
+void smbios_cpu_get_core_counts(u16 *core_count, u16 *thread_count);
 unsigned int smbios_cpu_get_max_speed_mhz(void);
 unsigned int smbios_cpu_get_current_speed_mhz(void);
 unsigned int smbios_cpu_get_voltage(void);
+unsigned int smbios_get_max_sockets(void);
+unsigned int smbios_soc_get_max_sockets(void);
 
 const char *smbios_mainboard_manufacturer(void);
 const char *smbios_mainboard_product_name(void);
@@ -210,6 +222,7 @@ typedef enum {
 	MEMORY_TYPE_HBM2 = 0x21,
 	MEMORY_TYPE_DDR5 = 0x22,
 	MEMORY_TYPE_LPDDR5 = 0x23,
+	MEMORY_TYPE_HBM3 = 0x24,
 } smbios_memory_type;
 
 typedef enum {
@@ -269,6 +282,7 @@ typedef enum {
 	SMBIOS_TEMPERATURE_PROBE = 28,
 	SMBIOS_SYSTEM_BOOT_INFORMATION = 32,
 	SMBIOS_IPMI_DEVICE_INFORMATION = 38,
+	SMBIOS_SYSTEM_POWER_SUPPLY = 39,
 	SMBIOS_ONBOARD_DEVICES_EXTENDED_INFORMATION = 41,
 	SMBIOS_TPM_DEVICE = 43,
 	SMBIOS_END_OF_TABLE = 127,
@@ -484,6 +498,107 @@ struct smbios_type4 {
 #define SMBIOS_PROCESSOR_STATUS_POPULATED		(1 << 6)
 #define SMBIOS_PROCESSOR_STATUS_CPU_ENABLED		(1 << 0)
 
+enum smbios_processor_type {
+	SMBIOS_PROCESSOR_TYPE_OTHER = 0x01,
+	SMBIOS_PROCESSOR_TYPE_UNKNOWN = 0x02,
+	SMBIOS_PROCESSOR_TYPE_CENTRAL = 0x03,
+	SMBIOS_PROCESSOR_TYPE_MATH = 0x04,
+	SMBIOS_PROCESSOR_TYPE_DSP = 0x05,
+	SMBIOS_PROCESSOR_TYPE_VIDEO = 0x06,
+};
+
+/* enum for socket type */
+enum smbios_processor_upgrade_field {
+	PROCESSOR_UPGRADE_OTHER = 0x01,
+	PROCESSOR_UPGRADE_UNKNOWN = 0x02,
+	PROCESSOR_UPGRADE_DAUGHTER_BOARD = 0x03,
+	PROCESSOR_UPGRADE_ZIF_SOCKET = 0x04,
+	PROCESSOR_UPGRADE_REPLACEABLE_PIGGY_BACK = 0x05,
+	PROCESSOR_UPGRADE_NONE = 0x06,
+	PROCESSOR_UPGRADE_LIF_SOCKET = 0x07,
+	PROCESSOR_UPGRADE_SLOT_1 = 0x08,
+	PROCESSOR_UPGRADE_SLOT_2 = 0x09,
+	PROCESSOR_UPGRADE_370_PIN_SOCKET = 0x0a,
+	PROCESSOR_UPGRADE_SLOT_A = 0x0b,
+	PROCESSOR_UPGRADE_SLOT_M = 0x0c,
+	PROCESSOR_UPGRADE_SOCKET_423 = 0x0d,
+	PROCESSOR_UPGRADE_SOCKET_A = 0x0e,
+	PROCESSOR_UPGRADE_SOCKET_478 = 0x0f,
+	PROCESSOR_UPGRADE_SOCKET_754 = 0x10,
+	PROCESSOR_UPGRADE_SOCKET_940 = 0x11,
+	PROCESSOR_UPGRADE_SOCKET_939 = 0x12,
+	PROCESSOR_UPGRADE_SOCKET_MPGA604 = 0x13,
+	PROCESSOR_UPGRADE_SOCKET_LGA771 = 0x14,
+	PROCESSOR_UPGRADE_SOCKET_LGA775 = 0x15,
+	PROCESSOR_UPGRADE_SOCKET_S1 = 0x16,
+	PROCESSOR_UPGRADE_SOCKET_AM2 = 0x17,
+	PROCESSOR_UPGRADE_SOCKET_F = 0x18,
+	PROCESSOR_UPGRADE_SOCKET_LGA1366 = 0x19,
+	PROCESSOR_UPGRADE_SOCKET_G34 = 0x1a,
+	PROCESSOR_UPGRADE_SOCKET_AM3 = 0x1b,
+	PROCESSOR_UPGRADE_SOCKET_C32 = 0x1c,
+	PROCESSOR_UPGRADE_SOCKET_LGA1156 = 0x1d,
+	PROCESSOR_UPGRADE_SOCKET_LGA1567 = 0x1e,
+	PROCESSOR_UPGRADE_SOCKET_PGA988A = 0x1f,
+	PROCESSOR_UPGRADE_SOCKET_BGA1288 = 0x20,
+	PROCESSOR_UPGRADE_SOCKET_RPGA988B = 0x21,
+	PROCESSOR_UPGRADE_SOCKET_BGA1023 = 0x22,
+	PROCESSOR_UPGRADE_SOCKET_BGA1224 = 0x23,
+	PROCESSOR_UPGRADE_SOCKET_LGA1155 = 0x24,
+	PROCESSOR_UPGRADE_SOCKET_LGA1356 = 0x25,
+	PROCESSOR_UPGRADE_SOCKET_LGA2011 = 0x26,
+	PROCESSOR_UPGRADE_SOCKET_FS1 = 0x27,
+	PROCESSOR_UPGRADE_SOCKET_FS2 = 0x28,
+	PROCESSOR_UPGRADE_SOCKET_FM1 = 0x29,
+	PROCESSOR_UPGRADE_SOCKET_FM2 = 0x2a,
+	PROCESSOR_UPGRADE_SOCKET_LGA2011_3 = 0x2b,
+	PROCESSOR_UPGRADE_SOCKET_LGA1356_3 = 0x2c,
+	PROCESSOR_UPGRADE_SOCKET_LGA1150 = 0x2d,
+	PROCESSOR_UPGRADE_SOCKET_BGA1168 = 0x2e,
+	PROCESSOR_UPGRADE_SOCKET_BGA1234 = 0x2f,
+	PROCESSOR_UPGRADE_SOCKET_BGA1364 = 0x30,
+	PROCESSOR_UPGRADE_SOCKET_AM4 = 0x31,
+	PROCESSOR_UPGRADE_SOCKET_LGA1151 = 0x32,
+	PROCESSOR_UPGRADE_SOCKET_BGA1356 = 0x33,
+	PROCESSOR_UPGRADE_SOCKET_BGA1440 = 0x34,
+	PROCESSOR_UPGRADE_SOCKET_BGA1515 = 0x35,
+	PROCESSOR_UPGRADE_SOCKET_LGA3647_1 = 0x36,
+	PROCESSOR_UPGRADE_SOCKET_SP3 = 0x37,
+	PROCESSOR_UPGRADE_SOCKET_SP3R2 = 0x38,
+	PROCESSOR_UPGRADE_SOCKET_LGA2066 = 0x39,
+	PROCESSOR_UPGRADE_SOCKET_BGA1392 = 0x3a,
+	PROCESSOR_UPGRADE_SOCKET_BGA1510 = 0x3b,
+	PROCESSOR_UPGRADE_SOCKET_BGA1528 = 0x3c,
+	PROCESSOR_UPGRADE_SOCKET_LGA4189 = 0x3d,
+	PROCESSOR_UPGRADE_SOCKET_LGA1200 = 0x3e,
+	PROCESSOR_UPGRADE_SOCKET_LGA4677 = 0x3f,
+	PROCESSOR_UPGRADE_SOCKET_LGA1700 = 0x40,
+	PROCESSOR_UPGRADE_SOCKET_BGA1744 = 0x41,
+	PROCESSOR_UPGRADE_SOCKET_BGA1781 = 0x42,
+	PROCESSOR_UPGRADE_SOCKET_BGA1211 = 0x43,
+	PROCESSOR_UPGRADE_SOCKET_BGA2422 = 0x44,
+	PROCESSOR_UPGRADE_SOCKET_LGA1211 = 0x45,
+	PROCESSOR_UPGRADE_SOCKET_LGA2422 = 0x46,
+	PROCESSOR_UPGRADE_SOCKET_LGA5773 = 0x47,
+	PROCESSOR_UPGRADE_SOCKET_BGA5773 = 0x48,
+};
+
+/* defines for processor family */
+#define SMBIOS_PROCESSOR_FAMILY_OTHER			0x01
+#define SMBIOS_PROCESSOR_FAMILY_UNKNOWN			0x02
+#define SMBIOS_PROCESSOR_FAMILY_INTEL486		0x06
+#define SMBIOS_PROCESSOR_FAMILY_PENTIUM_PRO		0x0c
+#define SMBIOS_PROCESSOR_FAMILY_XEON			0xb3
+#define SMBIOS_PROCESSOR_FAMILY_FROM_FAMILY2		0xfe
+
+/* defines for processor family 2 */
+#define SMBIOS_PROCESSOR_FAMILY2_ARMV8			0x101
+
+/* defines for processor characteristics */
+#define PROCESSOR_64BIT_CAPABLE				(1 << 2)
+#define PROCESSOR_MULTI_CORE				(1 << 3)
+#define PROCESSOR_POWER_PERFORMANCE_CONTROL		(1 << 7)
+
 /* defines for supported_sram_type/current_sram_type */
 
 #define SMBIOS_CACHE_SRAM_TYPE_OTHER			(1 << 0)
@@ -720,6 +835,7 @@ enum misc_slot_type {
 	SlotTypePciExpressMini52pinWithBSKO = 0x21,
 	SlotTypePciExpressMini52pinWithoutBSKO = 0x22,
 	SlotTypePciExpressMini76pin = 0x23,
+	SlotTypePciExpressOCPNIC30SFF = 0x26,
 	SlotTypePC98C20 = 0xA0,
 	SlotTypePC98C24 = 0xA1,
 	SlotTypePC98E = 0xA2,
@@ -1000,6 +1116,69 @@ enum smbios_bmc_interface_type {
 	SMBIOS_BMC_INTERFACE_SMBUS,
 };
 
+typedef enum {
+	PowerSupplyTypeOther = 1,
+	PowerSupplyTypeUnknown = 2,
+	PowerSupplyTypeLinear = 3,
+	PowerSupplyTypeSwitching = 4,
+	PowerSupplyTypeBattery = 5,
+	PowerSupplyTypeUps = 6,
+	PowerSupplyTypeConverter = 7,
+	PowerSupplyTypeRegulator = 8
+} power_supply_type;
+
+typedef enum {
+	PowerSupplyStatusOther = 1,
+	PowerSupplyStatusUnknown = 2,
+	PowerSupplyStatusOk = 3,
+	PowerSupplyStatusNonCritical = 4,
+	PowerSupplyStatusCritical = 5
+} power_supply_status;
+
+typedef enum {
+	PowerSupplyInputVoltageRangeSwitchingOther = 1,
+	PowerSupplyInputVoltageRangeSwitchingUnknown = 2,
+	PowerSupplyInputVoltageRangeSwitchingManual = 3,
+	PowerSupplyInputVoltageRangeSwitchingAutoSwitch = 4,
+	PowerSupplyInputVoltageRangeSwitchingWideRange = 5,
+	PowerSupplyInputVoltageRangeSwitchingNotApplicable = 6
+} power_supply_input_voltage_range_switching;
+
+struct power_supply_ch {
+	u16 reserved				:2;
+	u16 power_supply_type			:4;
+	u16 power_supply_status			:3;
+	u16 input_voltage_range_switch		:4;
+	u16 power_supply_unplugged		:1;
+	u16 power_supply_present		:1;
+	u16 power_supply_hot_replaceble		:1;
+};
+
+struct smbios_type39 {
+	struct smbios_header header;
+	u8 power_unit_group;
+	u8 location;
+	u8 device_name;
+	u8 manufacturer;
+	u8 serial_number;
+	u8 asset_tag_number;
+	u8 model_part_number;
+	u8 revision_level;
+	u16 max_power_capacity;
+	u16 power_supply_characteristics;
+	u16 input_voltage_probe_handle;
+	u16 cooling_device_handle;
+	u16 input_current_probe_handle;
+	u8 eos[2];
+} __packed;
+
+int smbios_write_type39(unsigned long *current, int *handle,
+			u8 unit_group, const char *loc, const char *dev_name,
+			const char *man, const char *serial_num,
+			const char *tag_num, const char *part_num,
+			const char *rev_lvl, u16 max_pow_cap,
+			const struct power_supply_ch *ps_ch);
+
 typedef enum {
 	SMBIOS_DEVICE_TYPE_OTHER = 0x01,
 	SMBIOS_DEVICE_TYPE_UNKNOWN,
@@ -1058,6 +1237,24 @@ struct smbios_type127 {
 	u8 eos[2];
 } __packed;
 
+/* Provided to help architecture code */
+int smbios_write_type7(unsigned long *current,
+		       const int handle,
+		       const u8 level,
+		       const u8 sram_type,
+		       const enum smbios_cache_associativity associativity,
+		       const enum smbios_cache_type type,
+		       const size_t max_cache_size,
+		       const size_t cache_size);
+enum smbios_cache_associativity smbios_cache_associativity(const u8 num);
+
+/* Must be defined by architecture code */
+int smbios_write_type4(unsigned long *current, int handle);
+int smbios_write_type7_cache_parameters(unsigned long *current,
+					int *handle,
+					int *max_struct_size,
+					struct smbios_type4 *type4);
+
 void smbios_fill_dimm_manufacturer_from_id(uint16_t mod_id,
 	struct smbios_type17 *t);
 void smbios_fill_dimm_asset_tag(const struct dimm_info *dimm,
