@@ -10,9 +10,9 @@ extern u8 _dram[];
 #define REGION_SIZE(name) ((size_t)_##name##_size)
 
 #define DECLARE_REGION(name)	\
-	extern u8 _##name[];	\
-	extern u8 _e##name[];	\
-	extern u8 _##name##_size[];
+	__maybe_unused extern u8 _##name[];	\
+	__maybe_unused extern u8 _e##name[];	\
+	__maybe_unused extern u8 _##name##_size[];
 
 /*
  * Regions can be declared optional if not all configurations provide them in
@@ -23,9 +23,9 @@ extern u8 _dram[];
  * should then check (REGION_SIZE(name) != 0) before accessing it.
  */
 #define DECLARE_OPTIONAL_REGION(name)	\
-	__weak extern u8 _##name[];	\
-	__weak extern u8 _e##name[];	\
-	__weak extern u8 _##name##_size[];
+	__maybe_unused __weak extern u8 _##name[];	\
+	__maybe_unused __weak extern u8 _e##name[];	\
+	__maybe_unused __weak extern u8 _##name##_size[];
 
 DECLARE_REGION(sram)
 DECLARE_OPTIONAL_REGION(timestamp)
@@ -39,7 +39,7 @@ DECLARE_REGION(cbfs_mcache)
 DECLARE_REGION(fmap_cache)
 DECLARE_REGION(tpm_log)
 
-#if ENV_ROMSTAGE && CONFIG(ASAN_IN_ROMSTAGE)
+#if ENV_SEPARATE_ROMSTAGE && CONFIG(ASAN_IN_ROMSTAGE)
 DECLARE_REGION(bss)
 DECLARE_REGION(asan_shadow)
 #endif
@@ -50,6 +50,11 @@ DECLARE_REGION(heap)
 DECLARE_REGION(asan_shadow)
 #endif
 
+#if ENV_SEPARATE_DATA_AND_BSS
+DECLARE_REGION(data)
+DECLARE_REGION(data_load)
+#endif
+
 /* Regions for execution units. */
 
 DECLARE_REGION(payload)
