@@ -1,5 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
+#include <amdblocks/acpi.h>
 #include <console/console.h>
 #include <device/device.h>
 #include <amdblocks/amd_pci_util.h>
@@ -14,30 +15,14 @@
 
 /* TODO: recheck IRQ tables */
 
-/*
- * These arrays set up the FCH PCI_INTR registers 0xC00/0xC01.
- * This table is responsible for physically routing the PIC and
- * IOAPIC IRQs to the different PCI devices on the system.  It
- * is read and written via registers 0xC00/0xC01 as an
- * Index/Data pair.  These values are chipset and mainboard
- * dependent and should be updated accordingly.
- */
-static uint8_t fch_pic_routing[0x80];
-static uint8_t fch_apic_routing[0x80];
-
-_Static_assert(sizeof(fch_pic_routing) == sizeof(fch_apic_routing),
-	"PIC and APIC FCH interrupt tables must be the same size");
-
-static const struct fch_irq_routing {
-	uint8_t intr_index;
-	uint8_t pic_irq_num;
-	uint8_t apic_irq_num;
-} mandolin_fch[] = {
+/* The IRQ mapping in fch_irq_map ends up getting written to the indirect address space that is
+   accessed via I/O ports 0xc00/0xc01. */
+static const struct fch_irq_routing fch_irq_map[] = {
 	{ PIRQ_A,	8,		16 },
 	{ PIRQ_B,	10,		17 },
 	{ PIRQ_C,	11,		18 },
 	{ PIRQ_D,	12,		19 },
-	{ PIRQ_SCI,	9,		 9 },
+	{ PIRQ_SCI,	ACPI_SCI_IRQ,	ACPI_SCI_IRQ },
 	{ PIRQ_SD,	PIRQ_NC,	16 },
 	{ PIRQ_SDIO,	PIRQ_NC,	16 },
 	{ PIRQ_SATA,	PIRQ_NC,	19 },
@@ -57,25 +42,10 @@ static const struct fch_irq_routing {
 	{ PIRQ_MISC2,	0x00,		0x00 },
 };
 
-static void init_tables(void)
+const struct fch_irq_routing *mb_get_fch_irq_mapping(size_t *length)
 {
-	const struct fch_irq_routing *entry;
-	int i;
-
-	memset(fch_pic_routing, PIRQ_NC, sizeof(fch_pic_routing));
-	memset(fch_apic_routing, PIRQ_NC, sizeof(fch_apic_routing));
-
-	for (i = 0; i < ARRAY_SIZE(mandolin_fch); i++) {
-		entry = mandolin_fch + i;
-		fch_pic_routing[entry->intr_index] = entry->pic_irq_num;
-		fch_apic_routing[entry->intr_index] = entry->apic_irq_num;
-	}
-}
-
-static void pirq_setup(void)
-{
-	intr_data_ptr = fch_apic_routing;
-	picr_data_ptr = fch_pic_routing;
+	*length = ARRAY_SIZE(fch_irq_map);
+	return fch_irq_map;
 }
 
 static void mainboard_init(void *chip_info)
@@ -94,14 +64,6 @@ static void mainboard_init(void *chip_info)
 		mainboard_program_emmc_gpios();
 }
 
-static void mainboard_enable(struct device *dev)
-{
-	init_tables();
-	/* Initialize the PIRQ data structures for consumption */
-	pirq_setup();
-}
-
 struct chip_operations mainboard_ops = {
 	.init = mainboard_init,
-	.enable_dev = mainboard_enable,
 };
