@@ -14,19 +14,19 @@ const u32 cim_verb_data[] = {
 
 	/* Pin Widget Verb Table */
 	AZALIA_PIN_CFG(2, 0x11, 0x99430130),
-	AZALIA_PIN_CFG(2, 0x12, 0x411111f0),
+	AZALIA_PIN_CFG(2, 0x12, AZALIA_PIN_CFG_NC(0)),
 	AZALIA_PIN_CFG(2, 0x14, 0x01014410),
-	AZALIA_PIN_CFG(2, 0x15, 0x411111f0),
-	AZALIA_PIN_CFG(2, 0x16, 0x411111f0),
-	AZALIA_PIN_CFG(2, 0x17, 0x411111f0),
+	AZALIA_PIN_CFG(2, 0x15, AZALIA_PIN_CFG_NC(0)),
+	AZALIA_PIN_CFG(2, 0x16, AZALIA_PIN_CFG_NC(0)),
+	AZALIA_PIN_CFG(2, 0x17, AZALIA_PIN_CFG_NC(0)),
 	AZALIA_PIN_CFG(2, 0x18, 0x01a19c50),
 	AZALIA_PIN_CFG(2, 0x19, 0x02a19c60),
 	AZALIA_PIN_CFG(2, 0x1a, 0x0181345f),
 	AZALIA_PIN_CFG(2, 0x1b, 0x02214c20),
-	AZALIA_PIN_CFG(2, 0x1c, 0x411111f0),
+	AZALIA_PIN_CFG(2, 0x1c, AZALIA_PIN_CFG_NC(0)),
 	AZALIA_PIN_CFG(2, 0x1d, 0x4004c601),
-	AZALIA_PIN_CFG(2, 0x1e, 0x411111f0),
-	AZALIA_PIN_CFG(2, 0x1f, 0x411111f0),
+	AZALIA_PIN_CFG(2, 0x1e, AZALIA_PIN_CFG_NC(0)),
+	AZALIA_PIN_CFG(2, 0x1f, AZALIA_PIN_CFG_NC(0)),
 };
 
 #endif
