@@ -3,7 +3,6 @@
 #include <baseboard/gpio.h>
 #include <baseboard/variants.h>
 #include <gpio.h>
-#include <soc/gpio.h>
 #include <soc/ramstage.h>
 
 void variant_configure_pads(void)
@@ -17,7 +16,8 @@ void variant_configure_pads(void)
 	base_pads = variant_gpio_table(&base_num);
 	gpio_padbased_override(padbased_table, base_pads, base_num);
 	override_pads = variant_gpio_override_table(&override_num);
-	gpio_padbased_override(padbased_table, override_pads, override_num);
+	if (override_pads != NULL)
+		gpio_padbased_override(padbased_table, override_pads, override_num);
 	fw_config_gpio_padbased_override(padbased_table);
 	gpio_configure_pads_with_padbased(padbased_table);
 	free(padbased_table);
