@@ -15,10 +15,6 @@ static const struct pad_config wfc_disable_pads[] = {
 	PAD_NC_LOCK(GPP_D16, NONE, LOCK_CONFIG),
 	/* H22 : WCAM_MCLK_R */
 	PAD_NC(GPP_H22, NONE),
-	/* R6 : DMIC_WCAM_CLK_R */
-	PAD_NC(GPP_R6, NONE),
-	/* R7 : DMIC_WCAM_DATA */
-	PAD_NC(GPP_R7, NONE),
 };
 
 void fw_config_gpio_padbased_override(struct pad_config *padbased_table)
@@ -28,5 +24,4 @@ void fw_config_gpio_padbased_override(struct pad_config *padbased_table)
 		gpio_padbased_override(padbased_table, wfc_disable_pads,
 						ARRAY_SIZE(wfc_disable_pads));
 	}
-
 }
