@@ -19,7 +19,7 @@ static void *smp_write_config_table(void *v)
 
 	firewire = dev_find_device(0x104c, 0x8023, 0);
 	if (firewire) {
-		firewire_bus = firewire->bus->secondary;
+		firewire_bus = firewire->upstream->secondary;
 		printk(BIOS_SPEW, "Firewire device is on bus %x\n",
 				firewire_bus);
 	}
@@ -30,14 +30,14 @@ static void *smp_write_config_table(void *v)
 	if (!riser)
 		riser = dev_find_device(0x3388, 0x0022, 0);
 	if (riser) {
-		riser_bus = riser->link_list->secondary;
+		riser_bus = riser->downstream->secondary;
 		printk(BIOS_SPEW, "Riser bus is %x\n", riser_bus);
 	}
 
 	mptable_write_buses(mc, NULL, &isa_bus);
 
 	/* I/O APICs:	APIC ID	Version	State		Address */
-	u8 ioapic_id = smp_write_ioapic_from_hw(mc, VIO_APIC_VADDR);
+	u8 ioapic_id = smp_write_ioapic_from_hw(mc, IO_APIC_ADDR);
 
 	/* Legacy Interrupts */
 
