@@ -18,12 +18,14 @@
 #define MBOX_BIOS_CMD_SX_INFO			0x03
 #define   MBOX_BIOS_CMD_SX_INFO_SLEEP_TYPE_MAX	0x07
 #define MBOX_BIOS_CMD_RSM_INFO			0x04
-#define MBOX_BIOS_CMD_PSP_QUERY			0x05
+#define MBOX_BIOS_CMD_PSP_FTPM_QUERY		0x05
 #define MBOX_BIOS_CMD_BOOT_DONE			0x06
 #define MBOX_BIOS_CMD_CLEAR_S3_STS		0x07
 #define MBOX_BIOS_CMD_S3_DATA_INFO		0x08
 #define MBOX_BIOS_CMD_NOP			0x09
+#define MBOX_BIOS_CMD_HSTI_QUERY		0x14
 #define MBOX_BIOS_CMD_PSB_AUTO_FUSING		0x21
+#define MBOX_BIOS_CMD_PSP_CAPS_QUERY		0x27
 #define MBOX_BIOS_CMD_SET_SPL_FUSE		0x2d
 #define MBOX_BIOS_CMD_QUERY_SPL_FUSE		0x47
 #define MBOX_BIOS_CMD_I2C_TPM_ARBITRATION	0x64
@@ -43,22 +45,21 @@
  * AMD reference code aligns and pads all buffers to 32 bytes.
  */
 struct mbox_buffer_header {
-	u32 size;	/* total size of buffer */
-	u32 status;	/* command status, filled by PSP if applicable */
+	uint32_t size;		/* total size of buffer */
+	uint32_t status;	/* command status, filled by PSP if applicable */
 } __packed;
 
 /*
- * command-specific buffer definitions:  see NDA document #54267
- * The following commands need a buffer definition if they are to be used.
- * All other commands will work with the default buffer.
- * MBOX_BIOS_CMD_SMM_INFO		MBOX_BIOS_CMD_PSP_QUERY
- * MBOX_BIOS_CMD_SX_INFO		MBOX_BIOS_CMD_S3_DATA_INFO
- * MBOX_BIOS_CMD_RSM_INFO
+ * x86 to PSP mailbox commands that don't take any parameter or return any data, use the
+ * mbox_default_buffer, while x86 to PSP commands that either pass data to the PSP or get data
+ * returned from the PSP use command-specific buffer definitions. For details on the specific
+ * buffer definitions for the various commands, see NDA document #54267 for the generations
+ * before family 17h and NDA document #55758 for the generations from family 17h on.
  */
 
 struct mbox_default_buffer {	/* command-response buffer unused by command */
 	struct mbox_buffer_header header;
-} __attribute__((packed, aligned(32)));
+} __packed __aligned(32);
 
 struct smm_req_buffer {
 	uint64_t smm_base;		/* TSEG base */
@@ -73,20 +74,35 @@ struct smm_req_buffer {
 	uint64_t psp_mbox_smm_flag_address;
 } __packed;
 
+/* MBOX_BIOS_CMD_SMM_INFO */
 struct mbox_cmd_smm_info_buffer {
 	struct mbox_buffer_header header;
 	struct smm_req_buffer req;
-} __attribute__((packed, aligned(32)));
+} __packed __aligned(32);
 
+/* MBOX_BIOS_CMD_SX_INFO */
 struct mbox_cmd_sx_info_buffer {
 	struct mbox_buffer_header header;
-	u8 sleep_type;
-} __attribute__((packed, aligned(32)));
+	uint8_t sleep_type;
+} __packed __aligned(32);
 
+/* MBOX_BIOS_CMD_PSP_FTPM_QUERY, MBOX_BIOS_CMD_PSP_CAPS_QUERY */
+struct mbox_cmd_capability_query_buffer {
+	struct mbox_buffer_header header;
+	uint32_t capabilities;
+} __packed __aligned(32);
+
+/* MBOX_BIOS_CMD_HSTI_QUERY */
+struct mbox_cmd_hsti_query_buffer {
+	struct mbox_buffer_header header;
+	uint32_t state;
+} __packed __aligned(32);
+
+/* MBOX_BIOS_CMD_SET_SPL_FUSE */
 struct mbox_cmd_late_spl_buffer {
 	struct mbox_buffer_header header;
 	uint32_t	spl_value;
-} __attribute__((packed, aligned(32)));
+} __packed __aligned(32);
 
 struct dtpm_config {
 	uint32_t gpio;
@@ -99,6 +115,7 @@ enum dtpm_request_type {
 	DTPM_REQUEST_MAX,
 };
 
+/* MBOX_BIOS_CMD_I2C_TPM_ARBITRATION */
 struct mbox_cmd_dtpm_config_buffer {
 	struct mbox_buffer_header header;
 	uint32_t request_type;
@@ -108,11 +125,32 @@ struct mbox_cmd_dtpm_config_buffer {
 #define PSP_INIT_TIMEOUT 10000 /* 10 seconds */
 #define PSP_CMD_TIMEOUT 1000 /* 1 second */
 
+#define C2P_BUFFER_MAXSIZE 0xc00 /* Core-to-PSP buffer */
+#define P2C_BUFFER_MAXSIZE 0xc00 /* PSP-to-core buffer */
+
+/* PSP to x86 status */
+enum mbox_p2c_status {
+	MBOX_PSP_SUCCESS		= 0x00,
+	MBOX_PSP_INVALID_PARAMETER	= 0x01,
+	MBOX_PSP_CRC_ERROR		= 0x02,
+	MBOX_PSP_COMMAND_PROCESS_ERROR	= 0x04,
+	MBOX_PSP_UNSUPPORTED		= 0x08,
+	MBOX_PSP_SPI_BUSY_ASYNC		= 0x0a,
+	MBOX_PSP_SPI_BUSY		= 0x0b,
+};
+
+uintptr_t get_psp_mmio_base(void);
+
 void psp_print_cmd_status(int cmd_status, struct mbox_buffer_header *header);
 
 /* This command needs to be implemented by the generation specific code. */
-int send_psp_command(u32 command, void *buffer);
+int send_psp_command(uint32_t command, void *buffer);
 
-uint32_t soc_read_c2p38(void);
+enum cb_err psp_get_ftpm_capabilties(uint32_t *capabilities);
+enum cb_err psp_get_psp_capabilities(uint32_t *capabilities);
+enum cb_err psp_get_hsti_state(uint32_t *state);
+enum cb_err soc_read_c2p38(uint32_t *msg_38_value);
+
+void enable_psp_smi(void);
 
 #endif /* __AMD_PSP_DEF_H__ */
