@@ -3,6 +3,7 @@
 #include <assert.h>
 #include <console/console.h>
 #include <cpu/x86/msr.h>
+#include <cpu/intel/common/common.h>
 #include <cpu/intel/cpu_ids.h>
 #include <device/device.h>
 #include <drivers/wifi/generic/wifi.h>
@@ -12,6 +13,7 @@
 #include <gpio.h>
 #include <intelbasecode/debug_feature.h>
 #include <intelblocks/cpulib.h>
+#include <intelblocks/cse.h>
 #include <intelblocks/pcie_rp.h>
 #include <option.h>
 #include <soc/iomap.h>
@@ -70,10 +72,12 @@ static void pcie_rp_init(FSP_M_CONFIG *m_cfg, uint32_t en_mask, enum pcie_rp_typ
 			printk(BIOS_WARNING, "Missing root port clock structure definition\n");
 			continue;
 		}
-		if (clk_req_mapping & (1 << cfg[i].clk_req))
-			printk(BIOS_WARNING, "Found overlapped clkreq assignment on clk req %d\n"
-				, cfg[i].clk_req);
+
 		if (!(cfg[i].flags & PCIE_RP_CLK_REQ_UNUSED)) {
+			if (clk_req_mapping & (1 << cfg[i].clk_req))
+				printk(BIOS_WARNING,
+				       "Found overlapped clkreq assignment on clk req %d\n",
+				       cfg[i].clk_req);
 			m_cfg->PcieClkSrcClkReq[cfg[i].clk_src] = cfg[i].clk_req;
 			clk_req_mapping |= 1 << cfg[i].clk_req;
 		}
@@ -270,6 +274,11 @@ static void fill_fspm_tcss_params(FSP_M_CONFIG *m_cfg,
 	m_cfg->TcssDma0En = is_devfn_enabled(SA_DEVFN_TCSS_DMA0);
 	m_cfg->TcssDma1En = is_devfn_enabled(SA_DEVFN_TCSS_DMA1);
 
+	m_cfg->UsbTcPortEnPreMem = 0;
+	for (int i = 0; i < MAX_TYPE_C_PORTS; i++)
+		if (config->tcss_ports[i].enable)
+			m_cfg->UsbTcPortEnPreMem |= BIT(i);
+
 #if (CONFIG(SOC_INTEL_RAPTORLAKE) && !CONFIG(FSP_USE_REPO)) || \
 	(!CONFIG(SOC_INTEL_ALDERLAKE_PCH_N) && CONFIG(FSP_USE_REPO))
 	m_cfg->DisableDynamicTccoldHandshake =
@@ -346,7 +355,7 @@ static void fill_fspm_trace_params(FSP_M_CONFIG *m_cfg,
 		const struct soc_intel_alderlake_config *config)
 {
 	/* Set debug probe type */
-	m_cfg->PlatformDebugConsent = CONFIG_SOC_INTEL_ALDERLAKE_DEBUG_CONSENT;
+	m_cfg->PlatformDebugConsent = CONFIG_SOC_INTEL_COMMON_DEBUG_CONSENT;
 
 	/* CrashLog config */
 	m_cfg->CpuCrashLogDevice = CONFIG(SOC_INTEL_CRASHLOG) && is_devfn_enabled(SA_DEVFN_TMT);
@@ -403,6 +412,47 @@ static void debug_override_memory_init_params(FSP_M_CONFIG *mupd)
 	debug_get_pch_cpu_tracehub_modes(&mupd->CpuTraceHubMode, &mupd->PchTraceHubMode);
 }
 
+static void fill_fspm_sign_of_life(FSP_M_CONFIG *m_cfg,
+				   FSPM_ARCH_UPD *arch_upd)
+{
+	const char *name;
+	bool esol_required = false;
+
+	/*
+	 * Memory training
+	 *
+	 * If valid MRC cache data is not found, FSP should perform a memory
+	 * training. Memory training can take a while so let's inform the end
+	 * user with an on-screen text message.
+	 */
+	if (!arch_upd->NvsBufferPtr) {
+		esol_required = true;
+		name = "memory training";
+		elog_add_event_byte(ELOG_TYPE_FW_EARLY_SOL, ELOG_FW_EARLY_SOL_MRC);
+	}
+
+	/*
+	 * CSE Sync
+	 *
+	 * If currently running CSE RW firmware version is different than CSE version
+	 * packed as part of the CBFS then CSE sync will be triggered. CSE sync can take
+	 * < 1-minute hence, let's inform the end user with an on-screen text message.
+	 */
+	if (CONFIG(SOC_INTEL_CSE_LITE_SKU) && is_cse_fw_update_required()) {
+		if (esol_required) {
+			name = "memory training and CSE update";
+		} else {
+			name = "CSE update";
+			esol_required =  true;
+		}
+
+		elog_add_event_byte(ELOG_TYPE_FW_EARLY_SOL, ELOG_FW_EARLY_SOL_CSE_SYNC);
+	}
+
+	if (esol_required)
+		ux_inform_user_of_update_operation(name);
+}
+
 void platform_fsp_memory_init_params_cb(FSPM_UPD *mupd, uint32_t version)
 {
 	const struct soc_intel_alderlake_config *config;
@@ -426,15 +476,9 @@ void platform_fsp_memory_init_params_cb(FSPM_UPD *mupd, uint32_t version)
 		}
 	}
 
-	/*
-	 * If valid MRC cache data is not found, FSP should perform a memory
-	 * training. Memory training can take a while so let's inform the end
-	 * user with an on-screen text message.
-	 */
-	if (!arch_upd->NvsBufferPtr) {
-		if (ux_inform_user_of_update_operation("memory training"))
-			elog_add_event_byte(ELOG_TYPE_FW_EARLY_SOL, ELOG_FW_EARLY_SOL_MRC);
-	}
+	if (CONFIG(CHROMEOS_ENABLE_ESOL))
+		fill_fspm_sign_of_life(m_cfg, arch_upd);
+
 	config = config_of_soc();
 
 	soc_memory_init_params(m_cfg, config);
