@@ -11,7 +11,6 @@
 #include <cpu/x86/msr.h>
 #include <device/device.h>
 #include <device/pci.h>
-#include <delay.h>
 #include <intelblocks/cpulib.h>
 #include <intelblocks/msr.h>
 #include <intelblocks/power_limit.h>
@@ -20,7 +19,7 @@
 #include <soc/soc_chip.h>
 #include <soc/systemagent.h>
 #include <spi_flash.h>
-#include "stddef.h"
+#include <stddef.h>
 
 /*
  * SoC implementation
@@ -84,9 +83,9 @@ int soc_get_uncore_prmmr_base_and_mask(uint64_t *prmrr_base,
 {
 	msr_t msr;
 	msr = rdmsr(MSR_PRMRR_BASE_0);
-	*prmrr_base = (uint64_t) msr.hi << 32 | msr.lo;
+	*prmrr_base = (uint64_t)msr.hi << 32 | msr.lo;
 	msr = rdmsr(MSR_PRMRR_PHYS_MASK);
-	*prmrr_mask = (uint64_t) msr.hi << 32 | msr.lo;
+	*prmrr_mask = (uint64_t)msr.hi << 32 | msr.lo;
 	return 0;
 }
 
@@ -103,15 +102,15 @@ void soc_add_configurable_mmio_resources(struct device *dev, int *resource_cnt)
 	struct sa_mmio_descriptor cfg_rsrc[6]; /* Increase size when adding more resources */
 
 	/* MMCONF */
-	size = get_mmcfg_size(dev);
+	size = sa_get_mmcfg_size();
 	if (size > 0)
 		set_mmio_resource(&(cfg_rsrc[count++]), CONFIG_ECAM_MMCONF_BASE_ADDRESS,
 			size, "MMCONF");
 
 	/* DSM */
-	size = get_dsm_size(dev);
+	size = sa_get_dsm_size();
 	if (size > 0) {
-		base = pci_read_config32(dev, DSM_BASE_ADDR_REG) & 0xFFF00000;
+		base = pci_read_config32(dev, BDSM) & 0xFFF00000;
 		set_mmio_resource(&(cfg_rsrc[count++]), base, size, "DSM");
 	}
 
@@ -134,14 +133,14 @@ void soc_add_configurable_mmio_resources(struct device *dev, int *resource_cnt)
 	}
 
 	/* GSM */
-	size = get_gsm_size(dev);
+	size = sa_get_gsm_size();
 	if (size > 0) {
 		base = sa_get_gsm_base();
 		set_mmio_resource(&(cfg_rsrc[count++]), base, size, "GSM");
 	}
 
 	/* DPR */
-	size = get_dpr_size(dev);
+	size = sa_get_dpr_size();
 	if (size > 0) {
 		/* DPR just below TSEG: */
 		base = tseg_base - size;
@@ -208,116 +207,3 @@ uint32_t soc_systemagent_max_chan_capacity_mib(u8 capid0_a_ddrsz)
 		return 65536;
 	}
 }
-
-uint64_t get_mmcfg_size(const struct device *dev)
-{
-	uint32_t pciexbar_reg;
-	uint64_t mmcfg_length;
-
-	if (!dev) {
-		printk(BIOS_DEBUG, "%s : device is null\n", __func__);
-		return 0;
-	}
-
-	pciexbar_reg = pci_read_config32(dev, PCIEXBAR);
-
-	if (!(pciexbar_reg & (1 << 0))) {
-		printk(BIOS_DEBUG, "%s : PCIEXBAR disabled\n", __func__);
-		return 0;
-	}
-
-	switch ((pciexbar_reg & MASK_PCIEXBAR_LENGTH) >> PCIEXBAR_LENGTH_LSB) {
-	case PCIEXBAR_LENGTH_4096MB:
-		mmcfg_length = 4 * ((uint64_t)GiB);
-		break;
-	case PCIEXBAR_LENGTH_2048MB:
-		mmcfg_length = 2 * ((uint64_t)GiB);
-		break;
-	case PCIEXBAR_LENGTH_1024MB:
-		mmcfg_length = 1 * GiB;
-		break;
-	case PCIEXBAR_LENGTH_512MB:
-		mmcfg_length = 512 * MiB;
-		break;
-	case PCIEXBAR_LENGTH_256MB:
-		mmcfg_length = 256 * MiB;
-		break;
-	case PCIEXBAR_LENGTH_128MB:
-		mmcfg_length = 128 * MiB;
-		break;
-	case PCIEXBAR_LENGTH_64MB:
-		mmcfg_length = 64 * MiB;
-		break;
-	default:
-		printk(BIOS_DEBUG, "%s : PCIEXBAR - invalid length (0x%x)\n", __func__,
-			pciexbar_reg & MASK_PCIEXBAR_LENGTH);
-		mmcfg_length = 0x0;
-		break;
-	}
-
-	return mmcfg_length;
-}
-
-uint64_t get_dsm_size(const struct device *dev)
-{
-	// - size : B0/D0/F0:R 50h [15:8]
-	uint32_t reg32 = pci_read_config32(dev, GGC);
-	uint64_t size;
-	uint32_t size_field = (reg32 & MASK_DSM_LENGTH) >> MASK_DSM_LENGTH_LSB;
-	if (size_field <= 0x10) { // 0x0 - 0x10
-		size = size_field * 32 * MiB;
-	} else if ((size_field >= 0xF0) && (size_field >= 0xFE)) {
-		size = ((uint64_t)size_field - 0xEF) * 4 * MiB;
-	} else {
-		switch (size_field) {
-		case 0x20:
-			size = 1 * GiB;
-			break;
-		case 0x30:
-			size = 1536 * MiB;
-			break;
-		case 0x40:
-			size = 2 * (uint64_t)GiB;
-			break;
-		default:
-			printk(BIOS_DEBUG, "%s : DSM - invalid length (0x%x)\n",
-				__func__, size_field);
-			size = 0x0;
-			break;
-		}
-	}
-	return size;
-}
-
-uint64_t get_gsm_size(const struct device *dev)
-{
-	const u32 gsm_size = pci_read_config32(dev, GGC);
-	uint64_t size;
-	uint32_t size_field = (gsm_size & MASK_GSM_LENGTH) >> MASK_GSM_LENGTH_LSB;
-	switch (size_field) {
-	case 0x0:
-		size = 0;
-		break;
-	case 0x1:
-		size = 2 * MiB;
-		break;
-	case 0x2:
-		size = 4 * MiB;
-		break;
-	case 0x3:
-		size = 8 * MiB;
-		break;
-	default:
-		size = 0;
-		break;
-	}
-	return size;
-}
-uint64_t get_dpr_size(const struct device *dev)
-{
-	uint64_t size;
-	uint32_t dpr_reg = pci_read_config32(dev, DPR_REG);
-	uint32_t size_field = (dpr_reg & MASK_DPR_LENGTH) >> MASK_DPR_LENGTH_LSB;
-	size = (uint64_t)size_field * MiB;
-	return size;
-}
