@@ -10,6 +10,7 @@
 #include <intelblocks/gspi.h>
 #include <intelblocks/lpc_lib.h>
 #include <intelblocks/power_limit.h>
+#include <intelblocks/xhci.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <soc/pch.h>
@@ -36,7 +37,6 @@ enum chip_pl2_4_cfg {
 };
 
 struct soc_intel_cannonlake_config {
-
 	/* Common struct containing soc config data required by common code */
 	struct soc_intel_common_config common_soc_config;
 
@@ -237,7 +237,7 @@ struct soc_intel_cannonlake_config {
 	/* Enables support for Teton Glacier hybrid storage device */
 	bool TetonGlacierMode;
 
-	/* Enable/Disable EIST. 1b:Enabled, 0b:Disabled */
+	/* Enable/Disable EIST. true:Enabled, false:Disabled */
 	bool eist_enable;
 
 	/* Enable C6 DRAM */
