@@ -7,13 +7,13 @@
 #include <device/mmio.h>
 #include <device/pci.h>
 #include <intelblocks/cpulib.h>
+#include <intelblocks/cse.h>
 #include <intelblocks/lpc_lib.h>
 #include <intelblocks/pcr.h>
 #include <intelblocks/pmclib.h>
 #include <intelblocks/systemagent.h>
 #include <intelblocks/tco.h>
 #include <intelblocks/thermal.h>
-#include <intelpch/lockdown.h>
 #include <soc/p2sb.h>
 #include <soc/pci_devs.h>
 #include <soc/pcr_ids.h>
@@ -79,13 +79,6 @@ static void pch_finalize(void)
 	 */
 	if (!CONFIG(USE_INTEL_FSP_MP_INIT))
 		cpu_lt_lock_memory();
-
-}
-
-static void sa_finalize(void)
-{
-	if (get_lockdown_config() == CHIPSET_LOCKDOWN_COREBOOT)
-		sa_lock_pam();
 }
 
 static void soc_finalize(void *unused)
@@ -94,10 +87,12 @@ static void soc_finalize(void *unused)
 
 	pch_finalize();
 	apm_control(APM_CNT_FINALIZE);
-	sa_finalize();
+	if (CONFIG(DISABLE_HECI1_AT_PRE_BOOT) &&
+			CONFIG(SOC_INTEL_COMMON_BLOCK_HECI1_DISABLE_USING_PMC_IPC))
+		heci1_disable();
 
 	/* Indicate finalize step with post code */
-	post_code(POST_OS_BOOT);
+	post_code(POSTCODE_OS_BOOT);
 }
 
 BOOT_STATE_INIT_ENTRY(BS_OS_RESUME, BS_ON_ENTRY, soc_finalize, NULL);
