@@ -28,8 +28,11 @@ enum pch_pmc_xtal {
  */
 enum pch_pmc_xtal pmc_get_xtal_freq(void);
 
-/* pmc_send_pci_enum_done() - send PMC IPC to inform PCI enumeration is done. */
-void pmc_send_pci_enum_done(void);
+/*
+ * pmc_send_pci_enum_done() - send PMC IPC to inform both BIOS Reset
+ * and PCI enumeration is done.
+ */
+void pmc_send_bios_reset_pci_enum_done(void);
 
 /* Forward declare the power state struct here */
 struct chipset_power_state;
@@ -135,6 +138,8 @@ void pmc_disable_all_gpe(void);
 /* Clear STD and GPIO GPE status registers. */
 void pmc_clear_all_gpe_status(void);
 
+/* Write PMC MMIO register value with the provided bitmask. */
+void pmc_or_mmio32(uint32_t offset, uint32_t ormask);
 /* Clear status bits in Power and Reset Status (PRSTS) register */
 void pmc_clear_prsts(void);
 
@@ -181,6 +186,9 @@ int pmc_fill_power_state(struct chipset_power_state *ps);
  */
 void pmc_gpe_init(void);
 
+/* Clear PMC GEN_PMCON_X register power failure status bits */
+void pmc_clear_pmcon_pwr_failure_sts(void);
+
 /* Clear PMC GEN_PMCON_A register status bits */
 void pmc_clear_pmcon_sts(void);
 
@@ -260,4 +268,10 @@ uint8_t get_pm_pwr_cyc_dur(uint8_t slp_s4_min_assert, uint8_t slp_s3_min_assert,
 /* API to set ACPI mode */
 void pmc_set_acpi_mode(void);
 
+/*
+ * This function reads and prints SoC QDF information using PMC interface
+ * if SOC_QDF_DYNAMIC_READ_PMC config is enabled.
+ */
+void pmc_dump_soc_qdf_info(void);
+
 #endif /* SOC_INTEL_COMMON_BLOCK_PMCLIB_H */
