@@ -87,8 +87,11 @@ static void pch_lpc_loop_resources(struct device *dev)
 {
 	struct resource *res;
 
+	if (!dev->enabled)
+		return;
+
 	for (res = dev->resource_list; res; res = res->next) {
-		if (res->flags & IORESOURCE_IO)
+		if ((res->flags & IORESOURCE_IO) && (res->flags & IORESOURCE_ASSIGNED))
 			lpc_open_pmio_window(res->base, res->size);
 	}
 	pch_lpc_set_child_resources(dev);
@@ -100,13 +103,13 @@ static void pch_lpc_loop_resources(struct device *dev)
  */
 static void pch_lpc_set_child_resources(struct device *dev)
 {
-	struct bus *link;
 	struct device *child;
 
-	for (link = dev->link_list; link; link = link->next) {
-		for (child = link->children; child; child = child->sibling)
-			pch_lpc_loop_resources(child);
-	}
+	if (!dev->downstream)
+		return;
+
+	for (child = dev->downstream->children; child; child = child->sibling)
+		pch_lpc_loop_resources(child);
 }
 
 static void pch_lpc_set_resources(struct device *dev)
@@ -138,6 +141,78 @@ struct device_operations lpc_ops = {
 };
 
 static const unsigned short pci_device_ids[] = {
+	PCI_DID_INTEL_PTL_U_H_ESPI_0,
+	PCI_DID_INTEL_PTL_U_H_ESPI_1,
+	PCI_DID_INTEL_PTL_U_H_ESPI_2,
+	PCI_DID_INTEL_PTL_U_H_ESPI_3,
+	PCI_DID_INTEL_PTL_U_H_ESPI_4,
+	PCI_DID_INTEL_PTL_U_H_ESPI_5,
+	PCI_DID_INTEL_PTL_U_H_ESPI_6,
+	PCI_DID_INTEL_PTL_U_H_ESPI_7,
+	PCI_DID_INTEL_PTL_U_H_ESPI_8,
+	PCI_DID_INTEL_PTL_U_H_ESPI_9,
+	PCI_DID_INTEL_PTL_U_H_ESPI_10,
+	PCI_DID_INTEL_PTL_U_H_ESPI_11,
+	PCI_DID_INTEL_PTL_U_H_ESPI_12,
+	PCI_DID_INTEL_PTL_U_H_ESPI_13,
+	PCI_DID_INTEL_PTL_U_H_ESPI_14,
+	PCI_DID_INTEL_PTL_U_H_ESPI_15,
+	PCI_DID_INTEL_PTL_U_H_ESPI_16,
+	PCI_DID_INTEL_PTL_U_H_ESPI_17,
+	PCI_DID_INTEL_PTL_U_H_ESPI_18,
+	PCI_DID_INTEL_PTL_U_H_ESPI_19,
+	PCI_DID_INTEL_PTL_U_H_ESPI_20,
+	PCI_DID_INTEL_PTL_U_H_ESPI_21,
+	PCI_DID_INTEL_PTL_U_H_ESPI_22,
+	PCI_DID_INTEL_PTL_U_H_ESPI_23,
+	PCI_DID_INTEL_PTL_U_H_ESPI_24,
+	PCI_DID_INTEL_PTL_U_H_ESPI_25,
+	PCI_DID_INTEL_PTL_U_H_ESPI_26,
+	PCI_DID_INTEL_PTL_U_H_ESPI_27,
+	PCI_DID_INTEL_PTL_U_H_ESPI_28,
+	PCI_DID_INTEL_PTL_U_H_ESPI_29,
+	PCI_DID_INTEL_PTL_U_H_ESPI_30,
+	PCI_DID_INTEL_PTL_U_H_ESPI_31,
+	PCI_DID_INTEL_PTL_H_ESPI_0,
+	PCI_DID_INTEL_PTL_H_ESPI_1,
+	PCI_DID_INTEL_PTL_H_ESPI_2,
+	PCI_DID_INTEL_PTL_H_ESPI_3,
+	PCI_DID_INTEL_PTL_H_ESPI_4,
+	PCI_DID_INTEL_PTL_H_ESPI_5,
+	PCI_DID_INTEL_PTL_H_ESPI_6,
+	PCI_DID_INTEL_PTL_H_ESPI_7,
+	PCI_DID_INTEL_PTL_H_ESPI_8,
+	PCI_DID_INTEL_PTL_H_ESPI_9,
+	PCI_DID_INTEL_PTL_H_ESPI_10,
+	PCI_DID_INTEL_PTL_H_ESPI_11,
+	PCI_DID_INTEL_PTL_H_ESPI_12,
+	PCI_DID_INTEL_PTL_H_ESPI_13,
+	PCI_DID_INTEL_PTL_H_ESPI_14,
+	PCI_DID_INTEL_PTL_H_ESPI_15,
+	PCI_DID_INTEL_PTL_H_ESPI_16,
+	PCI_DID_INTEL_PTL_H_ESPI_17,
+	PCI_DID_INTEL_PTL_H_ESPI_18,
+	PCI_DID_INTEL_PTL_H_ESPI_19,
+	PCI_DID_INTEL_PTL_H_ESPI_20,
+	PCI_DID_INTEL_PTL_H_ESPI_21,
+	PCI_DID_INTEL_PTL_H_ESPI_22,
+	PCI_DID_INTEL_PTL_H_ESPI_23,
+	PCI_DID_INTEL_PTL_H_ESPI_24,
+	PCI_DID_INTEL_PTL_H_ESPI_25,
+	PCI_DID_INTEL_PTL_H_ESPI_26,
+	PCI_DID_INTEL_PTL_H_ESPI_27,
+	PCI_DID_INTEL_PTL_H_ESPI_28,
+	PCI_DID_INTEL_PTL_H_ESPI_29,
+	PCI_DID_INTEL_PTL_H_ESPI_30,
+	PCI_DID_INTEL_PTL_H_ESPI_31,
+	PCI_DID_INTEL_LNL_ESPI_0,
+	PCI_DID_INTEL_LNL_ESPI_1,
+	PCI_DID_INTEL_LNL_ESPI_2,
+	PCI_DID_INTEL_LNL_ESPI_3,
+	PCI_DID_INTEL_LNL_ESPI_4,
+	PCI_DID_INTEL_LNL_ESPI_5,
+	PCI_DID_INTEL_LNL_ESPI_6,
+	PCI_DID_INTEL_LNL_ESPI_7,
 	PCI_DID_INTEL_MTL_ESPI_0,
 	PCI_DID_INTEL_MTL_ESPI_1,
 	PCI_DID_INTEL_MTL_ESPI_2,
@@ -182,9 +257,9 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_RPP_S_ESPI_1,
 	PCI_DID_INTEL_RPP_S_ESPI_2,
 	PCI_DID_INTEL_RPP_S_ESPI_3,
-	PCI_DID_INTEL_RPP_S_ESPI_4,
-	PCI_DID_INTEL_RPP_S_ESPI_5,
-	PCI_DID_INTEL_RPP_S_ESPI_6,
+	PCI_DID_INTEL_RPP_S_ESPI_Z790,
+	PCI_DID_INTEL_RPP_S_ESPI_H770,
+	PCI_DID_INTEL_RPP_S_ESPI_B760,
 	PCI_DID_INTEL_RPP_S_ESPI_7,
 	PCI_DID_INTEL_RPP_S_ESPI_8,
 	PCI_DID_INTEL_RPP_S_ESPI_9,
@@ -246,13 +321,6 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_CNP_H_LPC_QM370,
 	PCI_DID_INTEL_CNP_H_LPC_HM370,
 	PCI_DID_INTEL_CNP_H_LPC_CM246,
-	PCI_DID_INTEL_ICL_BASE_U_ESPI,
-	PCI_DID_INTEL_ICL_BASE_Y_ESPI,
-	PCI_DID_INTEL_ICL_U_PREMIUM_ESPI,
-	PCI_DID_INTEL_ICL_U_SUPER_U_ESPI,
-	PCI_DID_INTEL_ICL_U_SUPER_U_ESPI_REV0,
-	PCI_DID_INTEL_ICL_SUPER_Y_ESPI,
-	PCI_DID_INTEL_ICL_Y_PREMIUM_ESPI,
 	PCI_DID_INTEL_CMP_SUPER_U_LPC,
 	PCI_DID_INTEL_CMP_PREMIUM_Y_LPC,
 	PCI_DID_INTEL_CMP_PREMIUM_U_LPC,
@@ -317,7 +385,7 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_JSP_SUPER_ESPI,
 	PCI_DID_INTEL_ADP_S_ESPI_WM690,
 	PCI_DID_INTEL_ADP_S_ESPI_HM670,
-	PCI_DID_INTEL_ADP_S_ESPI_W685,
+	PCI_DID_INTEL_ADP_S_ESPI_W790,
 	PCI_DID_INTEL_ADP_S_ESPI_W680,
 	PCI_DID_INTEL_ADP_S_ESPI_H610,
 	PCI_DID_INTEL_ADP_S_ESPI_B660,
@@ -379,6 +447,7 @@ static const unsigned short pci_device_ids[] = {
 	PCI_DID_INTEL_ADP_M_N_ESPI_30,
 	PCI_DID_INTEL_ADP_M_N_ESPI_31,
 	PCI_DID_INTEL_SPR_ESPI_1,
+	PCI_DID_INTEL_SNR_LPC,
 	0
 };
 
