@@ -78,6 +78,9 @@ static void sa_soc_systemagent_init(struct device *dev)
 	if (m == NULL)
 		return;
 
+	if (!CONFIG(HAVE_CAPID_A_REGISTER))
+		return;
+
 	const uint32_t capid0_a = pci_read_config32(dev, CAPID0_A);
 
 	m->ecc_type = sa_get_ecc_type(capid0_a);
@@ -109,7 +112,7 @@ void sa_add_fixed_mmio_resources(struct device *dev, int *resource_cnt,
 			sa_fixed_resources[i].description, sa_fixed_resources[i].base,
 			sa_fixed_resources[i].size);
 
-		mmio_resource_kb(dev, index++, base / KiB, size / KiB);
+		mmio_range(dev, index++, base, size);
 	}
 
 	*resource_cnt = index;
@@ -126,7 +129,9 @@ void sa_add_fixed_mmio_resources(struct device *dev, int *resource_cnt,
 static const struct sa_mem_map_descriptor sa_memory_map[MAX_MAP_ENTRIES] = {
 	{ TOUUD, true, "TOUUD" },
 	{ TOLUD, false, "TOLUD" },
+#if CONFIG(HAVE_BDSM_BGSM_REGISTER)
 	{ BGSM, false, "BGSM" },
+#endif
 	{ TSEG, false, "TSEG" },
 };
 
@@ -187,30 +192,24 @@ static void sa_get_mem_map(struct device *dev, uint64_t *values)
  */
 static void sa_add_dram_resources(struct device *dev, int *resource_count)
 {
-	uintptr_t base_k;
-	size_t size_k;
 	uint64_t sa_map_values[MAX_MAP_ENTRIES];
-	uintptr_t top_of_ram;
 	int index = *resource_count;
 
-	top_of_ram = (uintptr_t)cbmem_top();
+	const uintptr_t top_of_ram = cbmem_top();
 
 	/* 0 - > 0xa0000 */
-	base_k = 0;
-	size_k = (0xa0000 / KiB) - base_k;
-	ram_resource_kb(dev, index++, base_k, size_k);
+	ram_from_to(dev, index++, 0, 0xa0000);
 
 	/* 0xc0000 -> top_of_ram */
-	base_k = 0xc0000 / KiB;
-	size_k = (top_of_ram / KiB) - base_k;
-	ram_resource_kb(dev, index++, base_k, size_k);
+	ram_from_to(dev, index++, 0xc0000, top_of_ram);
 
 	sa_get_mem_map(dev, &sa_map_values[0]);
 
-	/* top_of_ram -> TOLUD */
-	base_k = top_of_ram;
-	size_k = sa_map_values[SA_TOLUD_REG] - base_k;
-	mmio_resource_kb(dev, index++, base_k / KiB, size_k / KiB);
+	/*
+	 * top_of_ram -> TOLUD: This contains TSEG which needs to be uncacheable
+	 * for proper operation of the smihandler.
+	 */
+	mmio_from_to(dev, index++, top_of_ram, sa_map_values[SA_TOLUD_REG]);
 
 	/* 4GiB -> TOUUD */
 	upper_ram_end(dev, index++, sa_map_values[SA_TOUUD_REG]);
@@ -221,9 +220,8 @@ static void sa_add_dram_resources(struct device *dev, int *resource_count)
 	 * 0xa0000 - 0xbffff: legacy VGA
 	 * 0xc0000 - 0xfffff: RAM
 	 */
-	mmio_resource_kb(dev, index++, 0xa0000 / KiB, (0xc0000 - 0xa0000) / KiB);
-	reserved_ram_resource_kb(dev, index++, 0xc0000 / KiB,
-			(1*MiB - 0xc0000) / KiB);
+	mmio_from_to(dev, index++, 0xa0000, 0xc0000);
+	reserved_ram_from_to(dev, index++, 0xc0000, 1 * MiB);
 
 	*resource_count = index;
 }
@@ -246,7 +244,7 @@ static void imr_resource(struct device *dev, int idx, uint32_t base,
 	 * out of MTRRs. Memory reserved by IMRs is not usable for host
 	 * so mark it reserved.
 	 */
-	reserved_ram_resource_kb(dev, idx, base_k, size_k);
+	reserved_ram_range(dev, idx, base_k * KiB, size_k * KiB);
 }
 
 /*
@@ -275,6 +273,12 @@ static void systemagent_read_resources(struct device *dev)
 {
 	int index = 0;
 
+	/**
+	 * If SoC has multiple PCIe domains, only reading resources from the first one.
+	 */
+	if (!is_dev_on_domain0(dev))
+		return;
+
 	/* Read standard PCI resources. */
 	pci_dev_read_resources(dev);
 
@@ -319,7 +323,7 @@ void ssdt_set_above_4g_pci(const struct device *dev)
 	uint64_t touud;
 	sa_read_map_entry(pcidev_path_on_root(SA_DEVFN_ROOT), &sa_memory_map[SA_TOUUD_REG],
 			  &touud);
-	const uint64_t len = POWER_OF_2(cpu_phys_address_size()) - touud;
+	const uint64_t len = POWER_OF_2(soc_phys_address_size()) - touud;
 
 	const char *scope = acpi_device_path(dev);
 	acpigen_write_scope(scope);
@@ -330,6 +334,84 @@ void ssdt_set_above_4g_pci(const struct device *dev)
 	printk(BIOS_DEBUG, "PCI space above 4GB MMIO is at 0x%llx, len = 0x%llx\n", touud, len);
 }
 
+uint64_t sa_get_mmcfg_size(void)
+{
+	const uint32_t pciexbar_reg = pci_read_config32(__pci_0_00_0, PCIEXBAR);
+
+	if (!(pciexbar_reg & (1 << 0))) {
+		printk(BIOS_ERR, "%s : PCIEXBAR disabled\n", __func__);
+		return 0;
+	}
+
+	switch ((pciexbar_reg & PCIEXBAR_LENGTH_MASK) >> PCIEXBAR_LENGTH_MASK_LSB) {
+	case PCIEXBAR_LENGTH_4096MB:
+		return 4ULL * GiB;
+	case PCIEXBAR_LENGTH_2048MB:
+		return 2ULL * GiB;
+	case PCIEXBAR_LENGTH_1024MB:
+		return 1 * GiB;
+	case PCIEXBAR_LENGTH_512MB:
+		return 512 * MiB;
+	case PCIEXBAR_LENGTH_256MB:
+		return 256 * MiB;
+	case PCIEXBAR_LENGTH_128MB:
+		return 128 * MiB;
+	case PCIEXBAR_LENGTH_64MB:
+		return 64 * MiB;
+	default:
+		printk(BIOS_ERR, "%s : PCIEXBAR - invalid length (0x%x)\n", __func__,
+			(pciexbar_reg & PCIEXBAR_LENGTH_MASK) >> PCIEXBAR_LENGTH_MASK_LSB);
+		return 0x0;
+	}
+}
+
+uint64_t sa_get_dsm_size(void)
+{
+	const uint32_t size_field = (pci_read_config32(__pci_0_00_0, GGC) & DSM_LENGTH_MASK)
+					 >> DSM_LENGTH_MASK_LSB;
+	if (size_field <= 0x10) { /* 0x0 - 0x10 */
+		return size_field * 32 * MiB;
+	} else if ((size_field >= 0xF0) && (size_field >= 0xFE)) {
+		return ((uint64_t)size_field - 0xEF) * 4 * MiB;
+	} else {
+		switch (size_field) {
+		case 0x20:
+			return 1 * GiB;
+		case 0x30:
+			return 1536 * MiB;
+		case 0x40:
+			return 2 * (uint64_t)GiB;
+		default:
+			printk(BIOS_ERR, "%s : DSM - invalid length (0x%x)\n", __func__, size_field);
+			return 0x0;
+		}
+	}
+}
+
+uint64_t sa_get_gsm_size(void)
+{
+	const uint32_t size_field = (pci_read_config32(__pci_0_00_0, GGC) & GSM_LENGTH_MASK)
+					 >> GSM_LENGTH_MASK_LSB;
+	switch (size_field) {
+	case 0x0:
+	default:
+		return 0;
+	case 0x1:
+		return 2 * MiB;
+	case 0x2:
+		return 4 * MiB;
+	case 0x3:
+		return 8 * MiB;
+	}
+}
+
+uint64_t sa_get_dpr_size(void)
+{
+	const uint32_t size_field = (pci_read_config32(__pci_0_00_0, DPR) & DPR_LENGTH_MASK)
+					 >> DPR_LENGTH_MASK_LSB;
+	return (uint64_t)size_field * MiB;
+}
+
 struct device_operations systemagent_ops = {
 	.read_resources   = systemagent_read_resources,
 	.set_resources    = pci_dev_set_resources,
@@ -342,10 +424,17 @@ struct device_operations systemagent_ops = {
 };
 
 static const unsigned short systemagent_ids[] = {
+	PCI_DID_INTEL_PTL_U_ID_1,
+	PCI_DID_INTEL_PTL_H_ID_1,
+	PCI_DID_INTEL_PTL_H_ID_2,
+	PCI_DID_INTEL_LNL_M_ID,
+	PCI_DID_INTEL_LNL_M_ID_1,
 	PCI_DID_INTEL_MTL_M_ID,
 	PCI_DID_INTEL_MTL_P_ID_1,
 	PCI_DID_INTEL_MTL_P_ID_2,
 	PCI_DID_INTEL_MTL_P_ID_3,
+	PCI_DID_INTEL_MTL_P_ID_4,
+	PCI_DID_INTEL_MTL_P_ID_5,
 	PCI_DID_INTEL_GLK_NB,
 	PCI_DID_INTEL_APL_NB,
 	PCI_DID_INTEL_CNL_ID_U,
@@ -367,10 +456,6 @@ static const unsigned short systemagent_ids[] = {
 	PCI_DID_INTEL_CFL_ID_S_S_4,
 	PCI_DID_INTEL_CFL_ID_S_S_6,
 	PCI_DID_INTEL_CFL_ID_S_S_8,
-	PCI_DID_INTEL_ICL_ID_U,
-	PCI_DID_INTEL_ICL_ID_U_2_2,
-	PCI_DID_INTEL_ICL_ID_Y,
-	PCI_DID_INTEL_ICL_ID_Y_2,
 	PCI_DID_INTEL_CML_ULT,
 	PCI_DID_INTEL_CML_ULT_2_2,
 	PCI_DID_INTEL_CML_ULT_6_2,
@@ -414,6 +499,7 @@ static const unsigned short systemagent_ids[] = {
 	PCI_DID_INTEL_JSL_ID_3,
 	PCI_DID_INTEL_JSL_ID_4,
 	PCI_DID_INTEL_JSL_ID_5,
+	PCI_DID_INTEL_JSL_ID_6,
 	PCI_DID_INTEL_ADL_S_ID_1,
 	PCI_DID_INTEL_ADL_S_ID_2,
 	PCI_DID_INTEL_ADL_S_ID_3,
@@ -444,9 +530,29 @@ static const unsigned short systemagent_ids[] = {
 	PCI_DID_INTEL_ADL_N_ID_2,
 	PCI_DID_INTEL_ADL_N_ID_3,
 	PCI_DID_INTEL_ADL_N_ID_4,
+	PCI_DID_INTEL_ADL_N_ID_5,
+	PCI_DID_INTEL_RPL_HX_ID_1,
+	PCI_DID_INTEL_RPL_HX_ID_2,
+	PCI_DID_INTEL_RPL_HX_ID_3,
+	PCI_DID_INTEL_RPL_HX_ID_4,
+	PCI_DID_INTEL_RPL_HX_ID_5,
+	PCI_DID_INTEL_RPL_HX_ID_6,
+	PCI_DID_INTEL_RPL_HX_ID_7,
+	PCI_DID_INTEL_RPL_HX_ID_8,
+	PCI_DID_INTEL_RPL_S_ID_1,
+	PCI_DID_INTEL_RPL_S_ID_2,
+	PCI_DID_INTEL_RPL_S_ID_3,
+	PCI_DID_INTEL_RPL_S_ID_4,
+	PCI_DID_INTEL_RPL_S_ID_5,
 	PCI_DID_INTEL_RPL_P_ID_1,
 	PCI_DID_INTEL_RPL_P_ID_2,
 	PCI_DID_INTEL_RPL_P_ID_3,
+	PCI_DID_INTEL_RPL_P_ID_4,
+	PCI_DID_INTEL_RPL_P_ID_5,
+	PCI_DID_INTEL_RPL_P_ID_6,
+	PCI_DID_INTEL_RPL_P_ID_7,
+	PCI_DID_INTEL_RPL_P_ID_8,
+	PCI_DID_INTEL_SNR_ID,
 	0
 };
 
