@@ -5,17 +5,22 @@
 #include <device/device.h>
 #include <fsp/util.h>
 #include <intelblocks/cpulib.h>
+#include <intelblocks/pcie_rp.h>
 #include <soc/iomap.h>
 #include <soc/pci_devs.h>
+#include <soc/pcie.h>
 #include <soc/romstage.h>
 #include <soc/soc_chip.h>
 
+/* ISA Serial Base selection. */
+enum {
+	ISA_SERIAL_BASE_ADDR_3F8,
+	ISA_SERIAL_BASE_ADDR_2F8,
+} isa_serial_uart_base;
+
 static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 		const struct soc_intel_elkhartlake_config *config)
 {
-	unsigned int i;
-	uint32_t mask = 0;
-
 	/*
 	 * If IGD is enabled, set IGD stolen size to 60MB.
 	 * Otherwise, skip IGD init in FSP.
@@ -27,13 +32,7 @@ static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 	m_cfg->SaGv = config->SaGv;
 	m_cfg->RMT = config->RMT;
 
-	/* PCIe root port configuration */
-	for (i = 0; i < ARRAY_SIZE(config->PcieRpEnable); i++) {
-		if (config->PcieRpEnable[i])
-			mask |= (1 << i);
-	}
-
-	m_cfg->PcieRpEnableMask = mask;
+	m_cfg->PcieRpEnableMask = pcie_rp_enable_mask(pch_rp_groups);
 
 	FSP_ARRAY_LOAD(m_cfg->PcieClkSrcUsage, config->PcieClkSrcUsage);
 	FSP_ARRAY_LOAD(m_cfg->PcieClkSrcClkReq, config->PcieClkSrcClkReq);
@@ -46,29 +45,22 @@ static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 	/* Set CPU Ratio */
 	m_cfg->CpuRatio = 0;
 
-	/* Set debug interface flags */
-	m_cfg->PcdDebugInterfaceFlags = CONFIG(DRIVERS_UART_8250IO) ?
-		DEBUG_INTERFACE_UART_8250IO : DEBUG_INTERFACE_LPSS_SERIAL_IO;
-
-	/* TraceHub configuration */
-	if (is_devfn_enabled(PCH_DEVFN_TRACEHUB) && config->TraceHubMode) {
-		m_cfg->PcdDebugInterfaceFlags |= DEBUG_INTERFACE_TRACEHUB;
-		m_cfg->PchTraceHubMode = config->TraceHubMode;
-		m_cfg->CpuTraceHubMode = config->TraceHubMode;
-	}
-
 	/* Change VmxEnable UPD value according to ENABLE_VMX Kconfig */
 	m_cfg->VmxEnable = CONFIG(ENABLE_VMX);
 
 	/* PCH Master Gating Control */
-	m_cfg->PchMasterClockGating = 1;
-	m_cfg->PchMasterPowerGating = 1;
+	if (config->realtime_tuning_enable) {
+		m_cfg->PchMasterClockGating = 0;
+		m_cfg->PchMasterPowerGating = 0;
+		m_cfg->DisPgCloseIdleTimeout = 0;
+		m_cfg->PowerDownMode = 0;
+	} else {
+		m_cfg->PchMasterClockGating = 1;
+		m_cfg->PchMasterPowerGating = 1;
+	}
 
 	m_cfg->SmbusEnable = is_devfn_enabled(PCH_DEVFN_SMBUS);
 
-	/* Set debug probe type */
-	m_cfg->PlatformDebugConsent = CONFIG_SOC_INTEL_ELKHARTLAKE_DEBUG_CONSENT;
-
 	/* DMAR related config */
 	m_cfg->VtdDisable = 0;
 	m_cfg->X2ApicOptOut = 0x1;
@@ -89,7 +81,6 @@ static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 	m_cfg->WdtDisableAndLock = 0x1;
 
 	m_cfg->HeciCommunication2 = config->Heci2Enable;
-	m_cfg->SerialIoUartDebugControllerNumber = CONFIG_UART_FOR_CONSOLE;
 
 	/* Audio */
 	m_cfg->PchHdaEnable = is_devfn_enabled(PCH_DEVFN_HDA);
@@ -131,12 +122,57 @@ static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 	}
 }
 
+
+static void fill_fsp_debug_params(FSP_M_CONFIG *m_cfg,
+		const struct soc_intel_elkhartlake_config *config)
+{
+	/* Set debug interface flags */
+	m_cfg->PcdDebugInterfaceFlags = CONFIG(DRIVERS_UART_8250IO) ?
+		DEBUG_INTERFACE_UART_8250IO : DEBUG_INTERFACE_LPSS_SERIAL_IO;
+	m_cfg->SerialIoUartDebugControllerNumber = CONFIG_UART_FOR_CONSOLE;
+
+	/* Set debug probe type */
+	m_cfg->PlatformDebugConsent = CONFIG_SOC_INTEL_COMMON_DEBUG_CONSENT;
+
+	/* TraceHub configuration */
+	if (is_devfn_enabled(PCH_DEVFN_TRACEHUB) && config->TraceHubMode) {
+		m_cfg->PcdDebugInterfaceFlags |= DEBUG_INTERFACE_TRACEHUB;
+		m_cfg->PchTraceHubMode = config->TraceHubMode;
+		m_cfg->CpuTraceHubMode = config->TraceHubMode;
+	}
+
+	if (CONFIG(CONSOLE_SERIAL)) {
+		enum fsp_log_level log_level = fsp_map_console_log_level();
+		/* Set Serial debug message level */
+		m_cfg->PcdSerialDebugLevel = log_level;
+		/* Set MRC debug level */
+		m_cfg->MrcTaskDebugEnable = 0xff;
+	} else {
+		/* Disable Serial debug message */
+		m_cfg->PcdSerialDebugLevel = 0;
+		/* Disable MRC debug message */
+		m_cfg->MrcTaskDebugEnable = 0;
+	}
+
+	if (CONFIG(DRIVERS_UART_8250IO)) {
+		if (CONFIG_TTYS0_BASE == 0x3f8)
+			m_cfg->PcdIsaSerialUartBase = ISA_SERIAL_BASE_ADDR_3F8;
+		else if (CONFIG_TTYS0_BASE == 0x2f8)
+			m_cfg->PcdIsaSerialUartBase = ISA_SERIAL_BASE_ADDR_2F8;
+		else {
+			printk(BIOS_ERR, "Unsupported UART I/O base, "
+				"skipping FSP UART base UPD setting\n");
+		}
+	}
+}
+
 void platform_fsp_memory_init_params_cb(FSPM_UPD *mupd, uint32_t version)
 {
 	const struct soc_intel_elkhartlake_config *config = config_of_soc();
 	FSP_M_CONFIG *m_cfg = &mupd->FspmConfig;
 
 	soc_memory_init_params(m_cfg, config);
+	fill_fsp_debug_params(m_cfg, config);
 
 	mainboard_memory_init_params(mupd);
 }
