@@ -1,6 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <assert.h>
+#include <bootmode.h>
 #include <bootsplash.h>
 #include <cbfs.h>
 #include <console/console.h>
@@ -62,7 +63,9 @@ static const struct slot_irq_constraints irq_constraints[] = {
 	{
 		.slot = PCI_DEV_SLOT_DPTF,
 		.fns = {
-			ANY_PIRQ(PCI_DEVFN_DPTF),
+			/* Dynamic Tuning Technology (DTT) device IRQ is not
+			   programmable and is INT_A/PIRQ_A (IRQ 16) */
+			FIXED_INT_PIRQ(PCI_DEVFN_DPTF, PCI_INT_A, PIRQ_A),
 		},
 	},
 	{
@@ -392,7 +395,7 @@ static void fill_fsps_cpu_params(FSP_S_CONFIG *s_cfg,
 	 * This would avoid APs from getting hijacked by FSP while coreboot
 	 * decides to set SkipMpInit UPD.
 	 */
-	s_cfg->CpuMpPpi = (uintptr_t) mp_fill_ppi_services_data();
+	s_cfg->CpuMpPpi = (uintptr_t)mp_fill_ppi_services_data();
 
 	/*
 	 * Fill `2nd microcode loading FSP UPD` if FSP is running CPU feature
@@ -411,7 +414,7 @@ static void fill_fsps_igd_params(FSP_S_CONFIG *s_cfg,
 
 	/* Check if IGD is present and fill Graphics init param accordingly */
 	s_cfg->PeiGraphicsPeimInit = CONFIG(RUN_FSP_GOP) && is_devfn_enabled(PCI_DEVFN_IGD);
-	s_cfg->LidStatus = CONFIG(RUN_FSP_GOP);
+	s_cfg->LidStatus = CONFIG(VBOOT_LID_SWITCH) ? get_lid_switch() : CONFIG(RUN_FSP_GOP);
 	s_cfg->PavpEnable = CONFIG(PAVP);
 }
 
@@ -473,6 +476,8 @@ static void fill_fsps_xhci_params(FSP_S_CONFIG *s_cfg,
 			s_cfg->Usb2OverCurrentPin[i] = config->usb2_ports[i].ocpin;
 		else
 			s_cfg->Usb2OverCurrentPin[i] = OC_SKIP;
+
+		s_cfg->PortResetMessageEnable[i] = config->usb2_ports[i].type_c;
 	}
 
 	max_port = get_max_usb30_port();
@@ -665,6 +670,7 @@ static void fill_fsps_misc_power_params(FSP_S_CONFIG *s_cfg,
 	s_cfg->C1StateUnDemotion = !config->disable_c1_state_auto_demotion;
 	s_cfg->C1StateAutoDemotion = !config->disable_c1_state_auto_demotion;
 	s_cfg->PkgCStateDemotion = !config->disable_package_c_state_demotion;
+	s_cfg->PkgCStateUnDemotion = !config->disable_package_c_state_demotion;
 	s_cfg->PmcV1p05PhyExtFetControlEn = 1;
 
 	/* Enable PCH to CPU energy report feature. */
@@ -699,18 +705,27 @@ static void fill_fsps_irq_params(FSP_S_CONFIG *s_cfg,
 	printk(BIOS_INFO, "IRQ: Using dynamically assigned PCI IO-APIC IRQs\n");
 }
 
-static void arch_silicon_init_params(FSPS_ARCH_UPD *s_arch_cfg)
+static void arch_silicon_init_params(FSPS_ARCHx_UPD *s_arch_cfg)
 {
+
+#if !CONFIG(PLATFORM_USES_FSP2_4)
 	/*
 	 * EnableMultiPhaseSiliconInit for running MultiPhaseSiInit
 	 */
 	s_arch_cfg->EnableMultiPhaseSiliconInit = 1;
+#endif
 
 	/* Assign FspEventHandler arch Upd to use coreboot debug event handler */
 	if (CONFIG(FSP_USES_CB_DEBUG_EVENT_HANDLER) && CONFIG(CONSOLE_SERIAL) &&
 			 CONFIG(FSP_ENABLE_SERIAL_DEBUG))
+
+#if CONFIG(PLATFORM_USES_FSP2_X86_32)
 		s_arch_cfg->FspEventHandler = (FSP_EVENT_HANDLER)
 				fsp_debug_event_handler;
+#else
+		s_arch_cfg->FspEventHandler = (EFI_PHYSICAL_ADDRESS)
+				fsp_debug_event_handler;
+#endif
 }
 
 static void evaluate_ssid(const struct device *dev, uint16_t *svid, uint16_t *ssid)
@@ -756,7 +771,7 @@ static void fill_fsps_pci_ssid_params(FSP_S_CONFIG *s_cfg,
 
 	for (dev = all_devices; dev; dev = dev->next) {
 		if (!(is_dev_enabled(dev) && dev->path.type == DEVICE_PATH_PCI &&
-		    dev->bus->secondary == 0))
+		    dev->upstream->secondary == 0))
 			continue;
 
 		if (dev->path.pci.devfn == PCI_DEVFN_ROOT) {
@@ -818,7 +833,7 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
 {
 	struct soc_intel_meteorlake_config *config;
 	FSP_S_CONFIG *s_cfg = &supd->FspsConfig;
-	FSPS_ARCH_UPD *s_arch_cfg = &supd->FspsArchUpd;
+	FSPS_ARCHx_UPD *s_arch_cfg = &supd->FspsArchUpd;
 
 	config = config_of_soc();
 	arch_silicon_init_params(s_arch_cfg);
@@ -834,7 +849,7 @@ void platform_fsp_silicon_init_params_cb(FSPS_UPD *supd)
  *   1     |  After TCSS initialization completed             |  for TCSS specific init
  *   2     |  Before BIOS Reset CPL is set by FSP-S           |  for CPU specific init
  */
-void platform_fsp_multi_phase_init_cb(uint32_t phase_index)
+void platform_fsp_silicon_multi_phase_init_cb(uint32_t phase_index)
 {
 	switch (phase_index) {
 	case 1:
