@@ -1,17 +1,23 @@
 /* SPDX-License-Identifier: GPL-2.0-only */
 
 #include <assert.h>
+#include <bootmode.h>
 #include <console/console.h>
+#include <cpu/intel/common/common.h>
 #include <cpu/intel/cpu_ids.h>
 #include <cpu/x86/msr.h>
 #include <device/device.h>
+#include <device/pci.h>
 #include <drivers/wifi/generic/wifi.h>
+#include <elog.h>
 #include <fsp/fsp_debug_event.h>
 #include <fsp/util.h>
 #include <intelbasecode/ramtop.h>
 #include <intelblocks/cpulib.h>
+#include <intelblocks/cse.h>
 #include <intelblocks/pcie_rp.h>
 #include <option.h>
+#include <soc/cpu.h>
 #include <soc/gpio_soc_defs.h>
 #include <soc/iomap.h>
 #include <soc/msr.h>
@@ -21,6 +27,7 @@
 #include <soc/soc_chip.h>
 #include <soc/soc_info.h>
 #include <string.h>
+#include <ux_locales.h>
 
 #define FSP_CLK_NOTUSED			0xFF
 #define FSP_CLK_LAN			0x70
@@ -45,10 +52,12 @@ static void pcie_rp_init(FSP_M_CONFIG *m_cfg, uint32_t en_mask,
 			printk(BIOS_WARNING, "Missing root port clock structure definition\n");
 			continue;
 		}
-		if (clk_req_mapping & (1 << cfg[i].clk_req))
-			printk(BIOS_WARNING, "Found overlapped clkreq assignment on clk req %d\n"
-				, cfg[i].clk_req);
+
 		if (!(cfg[i].flags & PCIE_RP_CLK_REQ_UNUSED)) {
+			if (clk_req_mapping & (1 << cfg[i].clk_req))
+				printk(BIOS_WARNING,
+				       "Found overlapped clkreq assignment on clk req %d\n",
+				       cfg[i].clk_req);
 			m_cfg->PcieClkSrcClkReq[cfg[i].clk_src] = cfg[i].clk_req;
 			clk_req_mapping |= 1 << cfg[i].clk_req;
 		}
@@ -106,8 +115,8 @@ static void fill_fspm_igd_params(FSP_M_CONFIG *m_cfg,
 	};
 	m_cfg->InternalGfx = !CONFIG(SOC_INTEL_DISABLE_IGD) && is_devfn_enabled(PCI_DEVFN_IGD);
 	if (m_cfg->InternalGfx) {
-		/* IGD is enabled, set IGD stolen size to 64MB. */
-		m_cfg->IgdDvmt50PreAlloc = IGD_SM_64MB;
+		/* IGD is enabled, set IGD stolen size to 128MB. */
+		m_cfg->IgdDvmt50PreAlloc = IGD_SM_128MB;
 		/* DP port config */
 		m_cfg->DdiPortAConfig = config->ddi_port_A_config;
 		m_cfg->DdiPortBConfig = config->ddi_port_B_config;
@@ -155,8 +164,11 @@ static void fill_fspm_mrc_params(FSP_M_CONFIG *m_cfg,
 	}
 
 	m_cfg->RMT = config->rmt;
+	m_cfg->RMC = 0;
+	m_cfg->MarginLimitCheck = 0;
 	/* Enable MRC Fast Boot */
 	m_cfg->MrcFastBoot = 1;
+	m_cfg->LowerBasicMemTestSize = config->lower_basic_mem_test_size;
 }
 
 static void fill_fspm_cpu_params(FSP_M_CONFIG *m_cfg,
@@ -179,7 +191,8 @@ static void fill_tme_params(FSP_M_CONFIG *m_cfg)
 	m_cfg->TmeEnable = CONFIG(INTEL_TME) && is_tme_supported();
 	if (!m_cfg->TmeEnable)
 		return;
-	m_cfg->GenerateNewTmeKey = CONFIG(TME_KEY_REGENERATION_ON_WARM_BOOT);
+	m_cfg->GenerateNewTmeKey = CONFIG(TME_KEY_REGENERATION_ON_WARM_BOOT) &&
+			 CONFIG(SOC_INTEL_COMMON_BASECODE_RAMTOP);
 	if (m_cfg->GenerateNewTmeKey) {
 		uint32_t ram_top = get_ramtop_addr();
 		if (!ram_top) {
@@ -224,6 +237,27 @@ static void fill_fspm_smbus_params(FSP_M_CONFIG *m_cfg,
 	m_cfg->SmbusEnable = is_devfn_enabled(PCI_DEVFN_SMBUS);
 }
 
+static void fill_fspm_vr_config_params(FSP_M_CONFIG *m_cfg,
+		const struct soc_intel_meteorlake_config *config)
+{
+	/* FastVmode Settings for VR domains */
+	for (size_t domain = 0; domain < NUM_VR_DOMAINS; domain++) {
+		if (config->cep_enable[domain]) {
+			m_cfg->CepEnable[domain] = config->cep_enable[domain];
+			if (config->enable_fast_vmode[domain]) {
+				m_cfg->EnableFastVmode[domain] = config->enable_fast_vmode[domain];
+				m_cfg->IccLimit[domain] = config->fast_vmode_i_trip[domain];
+			}
+		}
+		if (config->ps_cur_1_threshold[domain])
+			m_cfg->Psi1Threshold[domain] = config->ps_cur_1_threshold[domain];
+		if (config->ps_cur_2_threshold[domain])
+			m_cfg->Psi2Threshold[domain] = config->ps_cur_2_threshold[domain];
+		if (config->ps_cur_3_threshold[domain])
+			m_cfg->Psi3Threshold[domain] = config->ps_cur_3_threshold[domain];
+	}
+}
+
 static void fill_fspm_misc_params(FSP_M_CONFIG *m_cfg,
 		const struct soc_intel_meteorlake_config *config)
 {
@@ -240,6 +274,11 @@ static void fill_fspm_misc_params(FSP_M_CONFIG *m_cfg,
 	m_cfg->SkipMbpHob = !CONFIG(FSP_PUBLISH_MBP_HOB);
 
 	m_cfg->SkipExtGfxScan = config->skip_ext_gfx_scan;
+
+	/* Set PsysPmax if it is available in DT.
+	   PsysPmax is in unit of 1/8 Watt */
+	if (config->psys_pmax_watts)
+		m_cfg->PsysPmax = config->psys_pmax_watts * 8;
 }
 
 static void fill_fspm_audio_params(FSP_M_CONFIG *m_cfg,
@@ -251,17 +290,11 @@ static void fill_fspm_audio_params(FSP_M_CONFIG *m_cfg,
 	m_cfg->PchHdaIDispLinkTmode = config->pch_hda_idisp_link_tmode;
 	m_cfg->PchHdaIDispLinkFrequency = config->pch_hda_idisp_link_frequency;
 	m_cfg->PchHdaIDispCodecDisconnect = !config->pch_hda_idisp_codec_enable;
+	m_cfg->PchHdaAudioLinkHdaEnable = config->pch_hda_audio_link_hda_enable;
 
 	for (int i = 0; i < MAX_HD_AUDIO_SDI_LINKS; i++)
 		m_cfg->PchHdaSdiEnable[i] = config->pch_hda_sdi_enable[i];
 
-	/*
-	 * All the PchHdaAudioLink{Hda|Dmic|Ssp|Sndw}Enable UPDs are used by FSP only to
-	 * configure GPIO pads for audio. Mainboard is expected to perform all GPIO
-	 * configuration in coreboot and hence these UPDs are set to 0 to skip FSP GPIO
-	 * configuration for audio pads.
-	 */
-	m_cfg->PchHdaAudioLinkHdaEnable = 0;
 	memset(m_cfg->PchHdaAudioLinkDmicEnable, 0, sizeof(m_cfg->PchHdaAudioLinkDmicEnable));
 	memset(m_cfg->PchHdaAudioLinkSspEnable, 0, sizeof(m_cfg->PchHdaAudioLinkSspEnable));
 	memset(m_cfg->PchHdaAudioLinkSndwEnable, 0, sizeof(m_cfg->PchHdaAudioLinkSndwEnable));
@@ -331,7 +364,7 @@ static void fill_fspm_trace_params(FSP_M_CONFIG *m_cfg,
 		const struct soc_intel_meteorlake_config *config)
 {
 	/* Set debug probe type */
-	m_cfg->PlatformDebugOption = CONFIG_SOC_INTEL_METEORLAKE_DEBUG_CONSENT;
+	m_cfg->PlatformDebugOption = CONFIG_SOC_INTEL_COMMON_DEBUG_CONSENT;
 
 	/* CrashLog config */
 	if (CONFIG(SOC_INTEL_CRASHLOG)) {
@@ -339,6 +372,39 @@ static void fill_fspm_trace_params(FSP_M_CONFIG *m_cfg,
 	}
 }
 
+static void fill_fspm_ibecc_params(FSP_M_CONFIG *m_cfg,
+		const struct soc_intel_meteorlake_config *config)
+{
+	/* In-Band ECC configuration */
+	if (config->ibecc.enable) {
+		m_cfg->Ibecc = config->ibecc.enable;
+		m_cfg->IbeccParity = config->ibecc.parity_en;
+		m_cfg->IbeccOperationMode = config->ibecc.mode;
+		if (m_cfg->IbeccOperationMode == IBECC_MODE_PER_REGION) {
+			FSP_ARRAY_LOAD(m_cfg->IbeccProtectedRegionEnable,
+				       config->ibecc.region_enable);
+			FSP_ARRAY_LOAD(m_cfg->IbeccProtectedRegionBase,
+				       config->ibecc.region_base);
+			FSP_ARRAY_LOAD(m_cfg->IbeccProtectedRegionMask,
+				       config->ibecc.region_mask);
+		}
+	}
+}
+
+static void fill_fsps_acoustic_params(FSP_M_CONFIG *m_cfg,
+		const struct soc_intel_meteorlake_config *config)
+{
+	if (!config->enable_acoustic_noise_mitigation)
+		return;
+
+	m_cfg->AcousticNoiseMitigation = config->enable_acoustic_noise_mitigation;
+
+	for (int i = 0; i < NUM_VR_DOMAINS; i++) {
+		m_cfg->FastPkgCRampDisable[i] = config->disable_fast_pkgc_ramp[i];
+		m_cfg->SlowSlewRate[i] = config->slow_slew_rate_config[i];
+	}
+}
+
 static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 		const struct soc_intel_meteorlake_config *config)
 {
@@ -360,22 +426,77 @@ static void soc_memory_init_params(FSP_M_CONFIG *m_cfg,
 		fill_fspm_usb4_params,
 		fill_fspm_vtd_params,
 		fill_fspm_trace_params,
+		fill_fspm_vr_config_params,
+		fill_fspm_ibecc_params,
+		fill_fsps_acoustic_params,
 	};
 
 	for (size_t i = 0; i < ARRAY_SIZE(fill_fspm_params); i++)
 		fill_fspm_params[i](m_cfg, config);
 }
 
+#define UX_MEMORY_TRAINING_DESC	"memory_training_desc"
+
+#define VGA_INIT_CONTROL_ENABLE		BIT(0)
+/* Tear down legacy VGA mode before exiting FSP-M. */
+#define VGA_INIT_CONTROL_TEAR_DOWN	BIT(1)
+
+static void fill_fspm_sign_of_life(FSP_M_CONFIG *m_cfg,
+				   FSPM_ARCHx_UPD *arch_upd)
+{
+	void *vbt;
+	size_t vbt_size;
+	uint32_t vga_init_control = 0;
+	uint8_t sol_type;
+
+	/* Memory training.  */
+	if (!arch_upd->NvsBufferPtr) {
+		vga_init_control = VGA_INIT_CONTROL_ENABLE |
+			VGA_INIT_CONTROL_TEAR_DOWN;
+		sol_type = ELOG_FW_EARLY_SOL_MRC;
+	}
+
+	if (CONFIG(SOC_INTEL_CSE_LITE_SKU) && is_cse_fw_update_required()) {
+		vga_init_control = VGA_INIT_CONTROL_ENABLE;
+		sol_type = ELOG_FW_EARLY_SOL_CSE_SYNC;
+	}
+
+	if (!vga_init_control)
+		return;
+
+	const char *text = ux_locales_get_text(UX_MEMORY_TRAINING_DESC);
+	/* No localized text found; fallback to built-in English. */
+	if (!text)
+		text = "Your device is finishing an update. "
+		       "This may take 1-2 minutes.\n"
+		       "Please do not turn off your device.";
+
+	vbt = cbfs_map("vbt.bin", &vbt_size);
+	if (!vbt) {
+		printk(BIOS_ERR, "Could not load vbt.bin\n");
+		return;
+	}
+
+	printk(BIOS_INFO, "Enabling FSP-M Sign-of-Life\n");
+	elog_add_event_byte(ELOG_TYPE_FW_EARLY_SOL, sol_type);
+
+	m_cfg->VgaInitControl = vga_init_control;
+	m_cfg->VbtPtr = (efi_uintn_t)vbt;
+	m_cfg->VbtSize = vbt_size;
+	m_cfg->LidStatus = CONFIG(VBOOT_LID_SWITCH) ? get_lid_switch() : CONFIG(RUN_FSP_GOP);
+	m_cfg->VgaMessage = (efi_uintn_t)text;
+}
+
 void platform_fsp_memory_init_params_cb(FSPM_UPD *mupd, uint32_t version)
 {
 	const struct soc_intel_meteorlake_config *config;
 	FSP_M_CONFIG *m_cfg = &mupd->FspmConfig;
-	FSPM_ARCH_UPD *arch_upd = &mupd->FspmArchUpd;
+	FSPM_ARCHx_UPD *arch_upd = &mupd->FspmArchUpd;
 
 	if (CONFIG(FSP_USES_CB_DEBUG_EVENT_HANDLER)) {
 		if (CONFIG(CONSOLE_SERIAL) && CONFIG(FSP_ENABLE_SERIAL_DEBUG)) {
 			enum fsp_log_level log_level = fsp_map_console_log_level();
-			arch_upd->FspEventHandler = (UINT32)((FSP_EVENT_HANDLER *)
+			arch_upd->FspEventHandler = (efi_uintn_t)((FSP_EVENT_HANDLER *)
 					fsp_debug_event_handler);
 			/* Set Serial debug message level */
 			m_cfg->PcdSerialDebugLevel = log_level;
@@ -391,6 +512,10 @@ void platform_fsp_memory_init_params_cb(FSPM_UPD *mupd, uint32_t version)
 	config = config_of_soc();
 
 	soc_memory_init_params(m_cfg, config);
+
+	if (CONFIG(CHROMEOS_ENABLE_ESOL))
+		fill_fspm_sign_of_life(m_cfg, arch_upd);
+
 	mainboard_memory_init_params(mupd);
 }
 
