@@ -34,7 +34,7 @@ bool soc_vtd_enabled(void)
 void soc_add_fixed_mmio_resources(struct device *dev, int *index)
 {
 	static const struct sa_mmio_descriptor soc_fixed_resources[] = {
-		{ PCIEXBAR, CONFIG_MMCONF_BASE_ADDRESS, CONFIG_MMCONF_LENGTH,
+		{ PCIEXBAR, CONFIG_ECAM_MMCONF_BASE_ADDRESS, CONFIG_ECAM_MMCONF_LENGTH,
 				"PCIEXBAR" },
 		{ MCHBAR, MCH_BASE_ADDRESS, MCH_BASE_SIZE, "MCHBAR" },
 		{ DMIBAR, DMI_BASE_ADDRESS, DMI_BASE_SIZE, "DMIBAR" },
@@ -84,9 +84,9 @@ int soc_get_uncore_prmmr_base_and_mask(uint64_t *prmrr_base,
 {
 	msr_t msr;
 	msr = rdmsr(MSR_UNCORE_PRMRR_PHYS_BASE);
-	*prmrr_base = (uint64_t) msr.hi << 32 | msr.lo;
+	*prmrr_base = (uint64_t)msr.hi << 32 | msr.lo;
 	msr = rdmsr(MSR_UNCORE_PRMRR_PHYS_MASK);
-	*prmrr_mask = (uint64_t) msr.hi << 32 | msr.lo;
+	*prmrr_mask = (uint64_t)msr.hi << 32 | msr.lo;
 	return 0;
 }
 
