@@ -4,8 +4,12 @@
 #include <intelblocks/rtc.h>
 #include <console/console.h>
 #include <fsp/util.h>
+#include <smbios.h>
+#include <soc/ddr.h>
+#include <soc/intel/common/smbios.h>
 #include <soc/romstage.h>
 #include <soc/util.h>
+#include <spd.h>
 
 void mainboard_romstage_entry(void)
 {
@@ -15,8 +19,14 @@ void mainboard_romstage_entry(void)
 
 	fsp_memory_init(false);
 	printk(BIOS_DEBUG, "coreboot fsp_memory_init finished...\n");
+	mainboard_ewl_check();
 
-	unlock_pam_regions();
+	if (CONFIG(ENABLE_FSP_ERROR_INFO)) {
+		if (fsp_display_error_info()) {
+			mainboard_fsp_error_handle();
+			die("ERROR: FSP reported an error(s) after running!");
+		}
+	}
 
 	save_dimm_info();
 }
@@ -28,6 +38,9 @@ __weak void mainboard_memory_init_params(FSPM_UPD *mupd)
 
 __weak void mainboard_rtc_failed(void)
 {
-
 }
-__weak void save_dimm_info(void) { }
+
+__weak void mainboard_ewl_check(void) { }
+
+/* mainboard can override this function for their own handling, such as writing a BMC SEL. */
+__weak void mainboard_fsp_error_handle(void) { }
