@@ -1,11 +1,13 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
+#include <arch/vga.h>
 #include <cbmem.h>
 #include <console/console.h>
 #include <cpu/x86/lapic_def.h>
 #include <device/pci.h>
 #include <device/pci_ids.h>
 #include <soc/acpi.h>
+#include <soc/chip_common.h>
 #include <soc/iomap.h>
 #include <soc/pci_devs.h>
 #include <soc/ramstage.h>
@@ -13,8 +15,16 @@
 #include <fsp/util.h>
 #include <security/intel/txt/txt_platform.h>
 #include <security/intel/txt/txt.h>
+#include <soc/config.h>
+#include <soc/numa.h>
+#include <soc/soc_util.h>
 #include <stdint.h>
 
+struct proximity_domains pds = {
+	.num_pds = 0,
+	.pds = NULL,
+};
+
 struct map_entry {
 	uint32_t    reg;
 	int         is_64_bit;
@@ -29,24 +39,48 @@ enum {
 	MMCFG_BASE_REG,
 	MMCFG_LIMIT_REG,
 	TOLM_REG,
+	/* NCMEM and ME ranges are mutually exclusive */
+	NCMEM_BASE_REG,
+	NCMEM_LIMIT_REG,
 	ME_BASE_REG,
 	ME_LIMIT_REG,
 	TSEG_BASE_REG,
 	TSEG_LIMIT_REG,
+	VTDBAR_REG,
 	/* Must be last. */
 	NUM_MAP_ENTRIES
 };
 
+size_t vtd_probe_bar_size(struct device *dev)
+{
+	uint32_t id = pci_read_config32(dev, PCI_VENDOR_ID);
+	assert(id == (PCI_VID_INTEL | (MMAP_VTD_CFG_REG_DEVID << 16)));
+
+	uint32_t val = pci_read_config32(dev, VTD_BAR_CSR);
+	pci_write_config32(dev, VTD_BAR_CSR, (uint32_t)(-4 * KiB));
+	size_t size = (~(pci_read_config32(dev, VTD_BAR_CSR) & ((uint32_t)(-4 * KiB)))) + 1;
+	assert(size != 0);
+	pci_write_config32(dev, VTD_BAR_CSR, val);
+
+	return size;
+}
+
 static struct map_entry memory_map[NUM_MAP_ENTRIES] = {
 		[TOHM_REG] = MAP_ENTRY_LIMIT_64(VTD_TOHM_CSR, 26, "TOHM"),
 		[MMIOL_REG] = MAP_ENTRY_BASE_32(VTD_MMIOL_CSR, "MMIOL"),
 		[MMCFG_BASE_REG] = MAP_ENTRY_BASE_64(VTD_MMCFG_BASE_CSR, "MMCFG_BASE"),
 		[MMCFG_LIMIT_REG] = MAP_ENTRY_LIMIT_64(VTD_MMCFG_LIMIT_CSR, 26, "MMCFG_LIMIT"),
 		[TOLM_REG] = MAP_ENTRY_LIMIT_32(VTD_TOLM_CSR, 26, "TOLM"),
+#if CONFIG(SOC_INTEL_HAS_NCMEM)
+		[NCMEM_BASE_REG] = MAP_ENTRY_BASE_64(VTD_NCMEM_BASE_CSR, "NCMEM_BASE"),
+		[NCMEM_LIMIT_REG] = MAP_ENTRY_LIMIT_64(VTD_NCMEM_LIMIT_CSR, 19, "NCMEM_LIMIT"),
+#else
 		[ME_BASE_REG] = MAP_ENTRY_BASE_64(VTD_ME_BASE_CSR, "ME_BASE"),
 		[ME_LIMIT_REG] = MAP_ENTRY_LIMIT_64(VTD_ME_LIMIT_CSR, 19, "ME_LIMIT"),
+#endif
 		[TSEG_BASE_REG] = MAP_ENTRY_BASE_32(VTD_TSEG_BASE_CSR, "TSEGMB_BASE"),
 		[TSEG_LIMIT_REG] = MAP_ENTRY_LIMIT_32(VTD_TSEG_LIMIT_CSR, 20, "TSEGMB_LIMIT"),
+		[VTDBAR_REG] = MAP_ENTRY_BASE_32(VTD_BAR_CSR, "VTD_BAR"),
 };
 
 static void read_map_entry(struct device *dev, struct map_entry *entry,
@@ -55,7 +89,16 @@ static void read_map_entry(struct device *dev, struct map_entry *entry,
 	uint64_t value;
 	uint64_t mask;
 
-	/* All registers are on a 1MiB granularity. */
+	if (!entry->reg) {
+		*result = 0;
+		return;
+	}
+	if (entry->reg == VTD_BAR_CSR && !(pci_read_config32(dev, entry->reg) & 1)) {
+		/* VTDBAR is not enabled */
+		*result = 0;
+		return;
+	}
+
 	mask = ((1ULL << entry->mask_bits) - 1);
 	mask = ~mask;
 
@@ -86,14 +129,17 @@ static void mc_report_map_entries(struct device *dev, uint64_t *values)
 {
 	int i;
 	for (i = 0; i < NUM_MAP_ENTRIES; i++) {
-		printk(BIOS_DEBUG, "MC MAP: %s: 0x%llx\n",
-		       memory_map[i].description, values[i]);
+		if (!memory_map[i].description)
+			continue;
+
+		printk(BIOS_DEBUG, "%s: MC MAP: %s: 0x%llx\n",
+		       dev_path(dev), memory_map[i].description, values[i]);
 	}
 }
 
 static void configure_dpr(struct device *dev)
 {
-	const uintptr_t cbmem_top_mb = ALIGN_UP((uintptr_t)cbmem_top(), MiB) / MiB;
+	const uintptr_t cbmem_top_mb = ALIGN_UP(cbmem_top(), MiB) / MiB;
 	union dpr_register dpr = { .raw = pci_read_config32(dev, VTD_LTDPR) };
 
 	/* The DPR lock bit has to be set sufficiently early. It looks like
@@ -146,6 +192,8 @@ static void configure_dpr(struct device *dev)
  * |     Tseg (relocatable)   | N x 8MB (0x70000000 - 0x77ffffff, 0x20000)
  * +--------------------------+
  * |     DPR                  |
+ * +--------------------------+ 1M aligned DPR base
+ * |     Unused memory        |
  * +--------------------------+ cbmem_top
  * |     Reserved - CBMEM     | (0x6fffe000 - 0x6fffffff, 0x2000)
  * +--------------------------+
@@ -168,46 +216,112 @@ static void mc_add_dram_resources(struct device *dev, int *res_count)
 {
 	const struct resource *res;
 	uint64_t mc_values[NUM_MAP_ENTRIES];
+	uint64_t top_of_ram;
 	int index = *res_count;
-
-	/* Only add dram resources once. */
-	if (dev->bus->secondary != 0)
-		return;
+	struct range_entry fsp_mem;
 
 	/* Read in the MAP registers and report their values. */
 	mc_read_map_entries(dev, &mc_values[0]);
 	mc_report_map_entries(dev, &mc_values[0]);
 
+	if (mc_values[VTDBAR_REG]) {
+		res = mmio_range(dev, VTD_BAR_CSR, mc_values[VTDBAR_REG],
+				vtd_probe_bar_size(dev));
+		LOG_RESOURCE("vtd_bar", dev, res);
+	}
+
+	/* Only add dram resources once. */
+	if (dev->upstream->secondary != 0 || dev->upstream->segment_group != 0)
+		return;
+
 	/* Conventional Memory (DOS region, 0x0 to 0x9FFFF) */
 	res = ram_from_to(dev, index++, 0, 0xa0000);
 	LOG_RESOURCE("legacy_ram", dev, res);
 
-	/* 1MB -> top_of_ram i.e., cbmem_top */
-	res = ram_from_to(dev, index++, 1 * MiB, (uintptr_t)cbmem_top());
+	/* 1MB -> top_of_ram */
+	fsp_find_reserved_memory(&fsp_mem);
+	top_of_ram = range_entry_base(&fsp_mem) - 1;
+	res = ram_from_to(dev, index++, 1 * MiB, top_of_ram);
 	LOG_RESOURCE("low_ram", dev, res);
 
+	/* top_of_ram -> cbmem_top */
+	res = ram_from_to(dev, index++, top_of_ram, cbmem_top());
+	LOG_RESOURCE("cbmem_ram", dev, res);
+
 	/* Mark TSEG/SMM region as reserved */
 	res = reserved_ram_from_to(dev, index++, mc_values[TSEG_BASE_REG],
 				   mc_values[TSEG_LIMIT_REG] + 1);
 	LOG_RESOURCE("mmio_tseg", dev, res);
 
-	/* Reserve and set up DPR */
-	configure_dpr(dev);
+	/* Reserve DPR region */
 	union dpr_register dpr = { .raw = pci_read_config32(dev, VTD_LTDPR) };
 	if (dpr.size) {
+		/*
+		 * cbmem_top -> DPR base:
+		 * DPR has a 1M granularity so it's possible if cbmem_top is not 1M
+		 * aligned that some memory does not get marked as assigned.
+		 */
+		res = reserved_ram_from_to(dev, index++, cbmem_top(),
+			(dpr.top - dpr.size) * MiB);
+		LOG_RESOURCE("unused_dram", dev, res);
+
+		/* DPR base -> DPR top */
 		res = reserved_ram_from_to(dev, index++, (dpr.top - dpr.size) * MiB,
 					   dpr.top * MiB);
 		LOG_RESOURCE("dpr", dev, res);
 	}
 
+	/* Mark TSEG/SMM region as reserved */
+	res = reserved_ram_from_to(dev, index++, mc_values[TSEG_BASE_REG],
+				   mc_values[TSEG_LIMIT_REG] + 1);
+	LOG_RESOURCE("mmio_tseg", dev, res);
+
 	/* Mark region between TSEG - TOLM (eg. MESEG) as reserved */
 	res = reserved_ram_from_to(dev, index++, mc_values[TSEG_LIMIT_REG] + 1,
 				   mc_values[TOLM_REG]);
 	LOG_RESOURCE("mmio_tolm", dev, res);
 
-	/* 4GiB -> TOHM */
-	res = upper_ram_end(dev, index++, mc_values[TOHM_REG] + 1);
-	LOG_RESOURCE("high_ram", dev, res);
+	if (CONFIG(SOC_INTEL_HAS_CXL)) {
+		/* 4GiB -> CXL Memory */
+		uint32_t gi_mem_size;
+		gi_mem_size = get_generic_initiator_mem_size(); /* unit: 64MB */
+		/*
+		 * Memory layout when there is CXL HDM (Host-managed Device Memory):
+		 * --------------  <- TOHM
+		 * CXL memory regions (pds global variable records the base/size of them)
+		 * Processor attached high memory
+		 * --------------  <- 0x100000000 (4GB)
+		 */
+		res = upper_ram_end(dev, index++,
+			mc_values[TOHM_REG] - ((uint64_t)gi_mem_size << 26) + 1);
+		LOG_RESOURCE("high_ram", dev, res);
+
+		/* CXL Memory */
+		uint8_t i;
+		for (i = 0; i < pds.num_pds; i++) {
+			if (pds.pds[i].pd_type != PD_TYPE_GENERIC_INITIATOR)
+				continue;
+
+			unsigned long flags = IORESOURCE_CACHEABLE;
+			int cxl_mode = get_cxl_mode();
+			if (cxl_mode == XEONSP_CXL_SP_MEM)
+				flags |= IORESOURCE_SOFT_RESERVE;
+			else
+				flags |= IORESOURCE_STORED;
+
+			res = fixed_mem_range_flags(dev, index++,
+				(uint64_t)pds.pds[i].base << 26,
+				(uint64_t)pds.pds[i].size << 26, flags);
+			if (cxl_mode == XEONSP_CXL_SP_MEM)
+				LOG_RESOURCE("specific_purpose_memory", dev, res);
+			else
+				LOG_RESOURCE("CXL_memory", dev, res);
+		}
+	} else {
+		/* 4GiB -> TOHM */
+		res = upper_ram_end(dev, index++, mc_values[TOHM_REG] + 1);
+		LOG_RESOURCE("high_ram", dev, res);
+	}
 
 	/* add MMIO CFG resource */
 	res = mmio_from_to(dev, index++, mc_values[MMCFG_BASE_REG],
@@ -225,7 +339,7 @@ static void mc_add_dram_resources(struct device *dev, int *res_count)
 	 * 0xa0000 - 0xbffff: legacy VGA
 	 * 0xc0000 - 0xfffff: RAM
 	 */
-	res = mmio_range(dev, index++, VGA_BASE_ADDRESS, VGA_BASE_SIZE);
+	res = mmio_range(dev, index++, VGA_MMIO_BASE, VGA_MMIO_SIZE);
 	LOG_RESOURCE("legacy_mmio", dev, res);
 
 	res = reserved_ram_from_to(dev, index++, 0xc0000, 1 * MiB);
@@ -241,6 +355,9 @@ static void mmapvtd_read_resources(struct device *dev)
 	/* Read standard PCI resources. */
 	pci_dev_read_resources(dev);
 
+	/* set up DPR */
+	configure_dpr(dev);
+
 	/* Calculate and add DRAM resources. */
 	mc_add_dram_resources(dev, &index);
 }
@@ -255,9 +372,6 @@ static struct device_operations mmapvtd_ops = {
 	.enable_resources  = pci_dev_enable_resources,
 	.init              = mmapvtd_init,
 	.ops_pci           = &soc_pci_ops,
-#if CONFIG(HAVE_ACPI_TABLES)
-	.acpi_inject_dsdt  = uncore_inject_dsdt,
-#endif
 };
 
 static const unsigned short mmapvtd_ids[] = {
@@ -271,6 +385,7 @@ static const struct pci_driver mmapvtd_driver __pci_driver = {
 	.devices  = mmapvtd_ids
 };
 
+#if !CONFIG(SOC_INTEL_MMAPVTD_ONLY_FOR_DPR)
 static void vtd_read_resources(struct device *dev)
 {
 	pci_dev_read_resources(dev);
@@ -291,6 +406,7 @@ static const struct pci_driver vtd_driver __pci_driver = {
 	.vendor   = PCI_VID_INTEL,
 	.device   = MMAP_VTD_STACK_CFG_REG_DEVID,
 };
+#endif
 
 static void dmi3_init(struct device *dev)
 {
