@@ -4,10 +4,32 @@
 #define _AMD_FW_TOOL_H_
 
 #include <commonlib/bsd/compiler.h>
+#include <commonlib/bsd/helpers.h>
 #include <openssl/sha.h>
 #include <stdint.h>
 #include <stdbool.h>
 
+#define ERASE_ALIGNMENT 0x1000U
+#define TABLE_ALIGNMENT 0x1000U
+#define BLOB_ALIGNMENT 0x100U
+#define TABLE_ERASE_ALIGNMENT _MAX(TABLE_ALIGNMENT, ERASE_ALIGNMENT)
+#define BLOB_ERASE_ALIGNMENT _MAX(BLOB_ALIGNMENT, ERASE_ALIGNMENT)
+
+enum platform {
+	PLATFORM_UNKNOWN,
+	PLATFORM_CARRIZO,
+	PLATFORM_STONEYRIDGE,
+	PLATFORM_RAVEN,
+	PLATFORM_PICASSO,
+	PLATFORM_RENOIR,
+	PLATFORM_CEZANNE,
+	PLATFORM_MENDOCINO,
+	PLATFORM_LUCIENNE,
+	PLATFORM_PHOENIX,
+	PLATFORM_GLINDA,
+	PLATFORM_GENOA,
+};
+
 typedef enum _amd_fw_type {
 	AMD_FW_PSP_PUBKEY = 0x00,
 	AMD_FW_PSP_BOOTLOADER = 0x01,
@@ -17,11 +39,16 @@ typedef enum _amd_fw_type {
 	AMD_FW_PSP_RTM_PUBKEY = 0x05,
 	AMD_FW_PSP_SMU_FIRMWARE = 0x08,
 	AMD_FW_PSP_SECURED_DEBUG = 0x09,
+	AMD_FW_ABL_PUBKEY = 0x0a,
 	AMD_PSP_FUSE_CHAIN = 0x0b,
 	AMD_FW_PSP_TRUSTLETS = 0x0c,
 	AMD_FW_PSP_TRUSTLETKEY = 0x0d,
 	AMD_FW_PSP_SMU_FIRMWARE2 = 0x12,
 	AMD_DEBUG_UNLOCK = 0x13,
+	AMD_BOOT_DRIVER = 0x1b,
+	AMD_SOC_DRIVER = 0x1c,
+	AMD_DEBUG_DRIVER = 0x1d,
+	AMD_INTERFACE_DRIVER = 0x1f,
 	AMD_HW_IPCFG = 0x20,
 	AMD_WRAPPED_IKEK = 0x21,
 	AMD_TOKEN_UNLOCK = 0x22,
@@ -29,6 +56,7 @@ typedef enum _amd_fw_type {
 	AMD_MP2_FW = 0x25,
 	AMD_DRIVER_ENTRIES = 0x28,
 	AMD_FW_KVM_IMAGE = 0x29,
+	AMD_FW_MP5 = 0x2a,
 	AMD_S0I3_DRIVER = 0x2d,
 	AMD_ABL0 = 0x30,
 	AMD_ABL1 = 0x31,
@@ -38,9 +66,12 @@ typedef enum _amd_fw_type {
 	AMD_ABL5 = 0x35,
 	AMD_ABL6 = 0x36,
 	AMD_ABL7 = 0x37,
+	AMD_SEV_DATA = 0x38,
+	AMD_SEV_CODE = 0x39,
 	AMD_FW_PSP_WHITELIST = 0x3a,
 	AMD_VBIOS_BTLOADER = 0x3c,
 	AMD_FW_L2_PTR = 0x40,
+	AMD_FW_DXIO = 0x42,
 	AMD_FW_USB_PHY = 0x44,
 	AMD_FW_TOS_SEC_POLICY = 0x45,
 	AMD_FW_DRTM_TA = 0x47,
@@ -57,10 +88,27 @@ typedef enum _amd_fw_type {
 	AMD_FW_DMCU_ISR = 0x59,
 	AMD_FW_MSMU = 0x5a,
 	AMD_FW_SPIROM_CFG = 0x5c,
-	AMD_FW_PSP_SMUSCS = 0x5f,
+	AMD_FW_MPIO = 0x5d,
+	AMD_FW_TPMLITE = 0x5f, /* family 17h & 19h */
+	AMD_FW_PSP_SMUSCS = 0x5f, /* family 15h & 16h */
 	AMD_FW_DMCUB = 0x71,
 	AMD_FW_PSP_BOOTLOADER_AB = 0x73,
+	AMD_RIB = 0x76,
+	AMD_FW_AMF_SRAM = 0x85,
+	AMD_FW_AMF_DRAM = 0x86,
+	AMD_FW_AMF_WLAN = 0x88,
+	AMD_FW_AMF_MFD = 0x89,
+	AMD_FW_MPDMA_TF = 0x8c,
 	AMD_TA_IKEK = 0x8d,
+	AMD_FW_MPCCX = 0x90,
+	AMD_FW_GMI3_PHY = 0x91,
+	AMD_FW_MPDMA_PM = 0x92,
+	AMD_FW_LSDMA = 0x94,
+	AMD_FW_C20_MP = 0x95,
+	AMD_FW_FCFG_TABLE = 0x98,
+	AMD_FW_MINIMSMU = 0x9a,
+	AMD_FW_SRAM_FW_EXT = 0x9d,
+	AMD_FW_UMSMU = 0xa2,
 	AMD_FW_IMC = 0x200,	/* Large enough to be larger than the top BHD entry type. */
 	AMD_FW_GEC,
 	AMD_FW_XHCI,
@@ -79,6 +127,7 @@ typedef enum _amd_bios_type {
 	AMD_BIOS_PMUD = 0x65,
 	AMD_BIOS_UCODE = 0x66,
 	AMD_BIOS_APCB_BK = 0x68,
+	AMD_BIOS_EARLY_VGA = 0x69,
 	AMD_BIOS_MP2_CFG = 0x6a,
 	AMD_BIOS_PSP_SHARED_MEM = 0x6b,
 	AMD_BIOS_L2_PTR =  0x70,
@@ -94,8 +143,8 @@ typedef enum _amd_addr_mode {
 } amd_addr_mode;
 
 struct second_gen_efs { /* todo: expand for Server products */
-	int gen:1; /* Client products only use bit 0 */
-	int reserved:31;
+	uint32_t gen:1; /* Client products only use bit 0 */
+	uint32_t reserved:31;
 } __attribute__((packed));
 
 #define EFS_SECOND_GEN 0
@@ -107,10 +156,7 @@ typedef struct _embedded_firmware {
 	uint32_t gec_entry;
 	uint32_t xhci_entry;
 	uint32_t psp_directory;
-	union {
-		uint32_t new_psp_directory;
-		uint32_t combo_psp_directory;
-	};
+	uint32_t new_psp_directory; /* also used as combo_psp_directory */
 	uint32_t bios0_entry; /* todo: add way to select correct entry */
 	uint32_t bios1_entry;
 	uint32_t bios2_entry;
@@ -155,7 +201,16 @@ typedef struct _psp_directory_header {
 typedef struct _psp_directory_entry {
 	uint8_t type;
 	uint8_t subprog;
-	uint16_t rsvd;
+	union {
+		uint16_t rsvd;
+		struct {
+			uint8_t rom_id:2;
+			uint8_t writable:1;
+			uint8_t inst:4;
+			uint8_t rsvd_1:1;
+			uint8_t rsvd_2:8;
+		} __attribute__((packed));
+	};
 	uint32_t size;
 	uint64_t addr:62; /* or a value in some cases */
 	uint64_t address_mode:2;
@@ -166,7 +221,7 @@ typedef struct _psp_directory_table {
 	psp_directory_entry entries[];
 } __attribute__((packed, aligned(16))) psp_directory_table;
 
-#define MAX_PSP_ENTRIES 0x2f
+#define MAX_PSP_ENTRIES 0xff
 
 typedef struct _psp_combo_header {
 	uint32_t cookie;
@@ -187,7 +242,7 @@ typedef struct _psp_combo_directory {
 	psp_combo_entry entries[];
 } __attribute__((packed, aligned(16))) psp_combo_directory;
 
-#define MAX_COMBO_ENTRIES 1
+#define MAX_COMBO_ENTRIES 2
 
 typedef struct _bios_directory_hdr {
 	uint32_t cookie;
@@ -267,6 +322,7 @@ typedef struct _ish_directory_table {
 #define PSP2_COOKIE 0x50535032		/* 'PSP2' */
 #define BHD_COOKIE 0x44484224		/* 'DHB$ */
 #define BHDL2_COOKIE 0x324c4224		/* '2LB$ */
+#define BHD2_COOKIE 0x44484232		/* 'DHB2' */
 
 #define PSP_LVL1 (1 << 0)
 #define PSP_LVL2 (1 << 1)
@@ -275,8 +331,18 @@ typedef struct _ish_directory_table {
 #define PSP_BOTH (PSP_LVL1 | PSP_LVL2)
 #define PSP_BOTH_AB (PSP_LVL1_AB | PSP_LVL2_AB)
 
+typedef enum _fwid_type {
+	FWID_TYPE_FWID = 0,
+	FWID_TYPE_UUID,
+} fwid_type_t;
+
+#define UUID_LEN_BYTES 16
 typedef struct _amd_fw_entry_hash {
-	uint16_t fw_id;
+	fwid_type_t fwid_type;
+	union {
+		uint16_t fw_id;
+		uint8_t uuid[UUID_LEN_BYTES];
+	};
 	uint16_t subtype;
 	uint32_t sha_len;
 	uint8_t sha[SHA384_DIGEST_LENGTH];
@@ -284,11 +350,9 @@ typedef struct _amd_fw_entry_hash {
 
 typedef struct _amd_fw_entry {
 	amd_fw_type type;
-	/* Mendocino and later SoCs use fw_id instead of fw_type. fw_type is still around
-	   for backwards compatibility. fw_id can be populated from the PSP binary file. */
-	uint16_t fw_id;
 	char *filename;
 	uint8_t subprog;
+	uint8_t inst;
 	uint64_t dest;
 	size_t size;
 	int level;
@@ -301,8 +365,11 @@ typedef struct _amd_fw_entry {
 	/* Some files that don't have amd_fw_header have to be skipped from hashing. These files
 	   include but not limited to: *iKek*, *.tkn, *.stkn */
 	bool skip_hashing;
+	uint8_t hash_tbl_id;
+	fwid_type_t fwid_type;
 	uint32_t num_hash_entries;
 	amd_fw_entry_hash *hash_entries;
+	bool generate_manifest;
 } amd_fw_entry;
 
 /* Most PSP binaries, if not all, have the following header format. */
@@ -320,7 +387,9 @@ struct amd_fw_header {
 	uint32_t comp_size;
 	/* Starting MDN fw_id is populated instead of fw_type. */
 	uint16_t fw_id;
-	uint8_t reserved_5a[18];
+	uint8_t reserved_5a[6];
+	uint8_t version[4];
+	uint8_t reserved_64[8];
 	uint32_t size_total;
 	uint8_t reserved_70[12];
 	/* Starting MDN fw_id is populated instead of fw_type. fw_type will still be around
@@ -332,6 +401,9 @@ struct amd_fw_header {
 	uint8_t reserved_80[128];
 } __packed;
 
+/* Based on the available PSP resources and increasing number of signed PSP binaries,
+   AMD recommends to split the hash table into 3 parts for now. */
+#define MAX_NUM_HASH_TABLES 3
 struct psp_fw_hash_table {
 	uint16_t version;
 	uint16_t no_of_entries_256;
@@ -353,16 +425,62 @@ typedef struct _amd_cb_config {
 	bool recovery_ab_single_copy;
 	bool need_ish;
 	bool use_combo;
+	bool have_apcb_bk;
+	enum platform soc_id;
+
+	uint8_t efs_spi_readmode, efs_spi_speed, efs_spi_micron_flag;
+	uint32_t body_location, efs_location;
+	uint64_t signed_start_addr;
+	char *manifest_file;
+	const char *signed_output_file;
+	char *output, *config;
+	char *combo_config[MAX_COMBO_ENTRIES];
+	int debug;
 } amd_cb_config;
 
-void register_fw_fuse(char *str);
-uint8_t process_config(FILE *config, amd_cb_config *cb_config, uint8_t print_deps);
+typedef struct _context {
+	char *rom;		/* target buffer, size of flash device */
+	uint32_t rom_size;	/* size of flash device */
+	uint32_t address_mode;	/* 0:abs address; 1:relative to flash; 2: relative to table */
+	uint32_t current;	/* pointer within flash & proxy buffer */
+	uint32_t current_pointer_saved;
+	uint32_t current_table;
+	void *amd_psp_fw_table_clean;
+	void *amd_bios_table_clean;
+	struct _combo_apcb {
+		char *filename;
+		uint8_t ins;
+		uint8_t sub;
+	} combo_apcb[MAX_COMBO_ENTRIES], combo_apcb_bk[MAX_COMBO_ENTRIES];
+	embedded_firmware *amd_romsig_ptr;
+	psp_directory_table *pspdir, *pspdir2, *pspdir2_b;
+	bios_directory_table *biosdir, *biosdir2, *biosdir2_b;
+	psp_combo_directory *psp_combo_dir, *bhd_combo_dir;
+	ish_directory_table *ish_a_dir, *ish_b_dir;
+} context;
 
+uint8_t process_config(FILE *config, amd_cb_config *cb_config);
+void process_signed_psp_firmwares(const char *signed_rom,
+		amd_fw_entry *fw_table,
+		uint64_t signed_start_addr,
+		enum platform soc_id);
+int find_bios_entry(amd_bios_type type);
+
+#define EFS_FILE_SUFFIX ".efs"
+#define TMP_FILE_SUFFIX ".tmp"
+#define BODY_FILE_SUFFIX ".body"
+
+void write_or_fail(int fd, void *ptr, size_t size);
+ssize_t read_from_file_to_buf(int fd, void *buf, size_t buf_size);
+ssize_t write_from_buf_to_file(int fd, const void *buf, size_t buf_size);
+ssize_t write_body(char *output, void *body_offset, ssize_t body_size);
+ssize_t copy_blob(void *dest, const char *src_file, size_t room);
 #define OK 0
 
 #define LINE_EOF (1)
 #define LINE_TOO_LONG (2)
 
-#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+int amdfwtool_getopt(int argc, char *argv[], amd_cb_config *cb_config, context *ctx);
+void register_apcb_combo(amd_cb_config *cb_config, int combo_index, context *ctx);
 
 #endif	/* _AMD_FW_TOOL_H_ */
