@@ -176,6 +176,9 @@ static uint8_t find_register_fw_filename_psp_dir(char *fw_name, char *filename,
 	} else if (strcmp(fw_name, "AMD_PUBKEY_FILE") == 0) {
 		fw_type = AMD_FW_PSP_PUBKEY;
 		subprog = 0;
+	} else if (strcmp(fw_name, "AMD_FUSE_CHAIN") == 0) {
+		fw_type = AMD_PSP_FUSE_CHAIN;
+		subprog = 0;
 	} else if (strcmp(fw_name, "PSPRCVR_FILE") == 0) {
 		fw_type = AMD_FW_PSP_RECOVERY;
 		subprog = 0;
@@ -499,11 +502,13 @@ static uint8_t find_register_fw_filename_psp_dir(char *fw_name, char *filename,
 			/* instance are not used in PSP table */
 			if (psp_tableptr->type == fw_type && psp_tableptr->subprog == subprog
 				&& psp_tableptr->inst  == instance) {
-				psp_tableptr->filename = filename;
+				if (psp_tableptr->type != AMD_PSP_FUSE_CHAIN) {
+					psp_tableptr->filename = filename;
+					psp_tableptr->hash_tbl_id = hash_tbl_id;
+					psp_tableptr->fwid_type = fwid_type;
+				}
 				SET_LEVEL(psp_tableptr, level_to_set, PSP,
 					cb_config->recovery_ab);
-				psp_tableptr->hash_tbl_id = hash_tbl_id;
-				psp_tableptr->fwid_type = fwid_type;
 				break;
 			}
 			psp_tableptr++;
@@ -613,11 +618,16 @@ static int is_valid_entry(char *oneline, regmatch_t match[N_MATCHES])
 		   match[4]: Optional directory level to be dropped
 		   match[6]: Optional hash table ID to put the hash for the entry
 		 */
-		oneline[match[FW_TYPE].rm_eo] = '\0';
-		oneline[match[FW_FILE].rm_eo] = '\0';
-		oneline[match[OPT_LEVEL].rm_eo] = '\0';
-		oneline[match[OPT_HASH_TABLE_ID].rm_eo] = '\0';
-		oneline[match[OPT_FWID_TYPE].rm_eo] = '\0';
+		if (match[FW_TYPE].rm_eo != -1)
+			oneline[match[FW_TYPE].rm_eo] = '\0';
+		if (match[FW_FILE].rm_eo != -1)
+			oneline[match[FW_FILE].rm_eo] = '\0';
+		if (match[OPT_LEVEL].rm_eo != -1)
+			oneline[match[OPT_LEVEL].rm_eo] = '\0';
+		if (match[OPT_HASH_TABLE_ID].rm_eo != -1)
+			oneline[match[OPT_HASH_TABLE_ID].rm_eo] = '\0';
+		if (match[OPT_FWID_TYPE].rm_eo != -1)
+			oneline[match[OPT_FWID_TYPE].rm_eo] = '\0';
 		retval = 1;
 	} else {
 		retval = 0;
