@@ -1,120 +1,12 @@
 package main
 
-import (
-	"fmt"
-	"os"
-)
+import "fmt"
 
 type bd82x6x struct {
 	variant string
 	node    *DevTreeNode
 }
 
-func (b bd82x6x) writeGPIOSet(ctx Context, sb *os.File,
-	val uint32, set uint, partno int, constraint uint32) {
-
-	max := uint(32)
-	if set == 3 {
-		max = 12
-	}
-
-	bits := [6][2]string{
-		{"GPIO_MODE_NATIVE", "GPIO_MODE_GPIO"},
-		{"GPIO_DIR_OUTPUT", "GPIO_DIR_INPUT"},
-		{"GPIO_LEVEL_LOW", "GPIO_LEVEL_HIGH"},
-		{"GPIO_RESET_PWROK", "GPIO_RESET_RSMRST"},
-		{"GPIO_NO_INVERT", "GPIO_INVERT"},
-		{"GPIO_NO_BLINK", "GPIO_BLINK"},
-	}
-
-	for i := uint(0); i < max; i++ {
-		if (constraint>>i)&1 == 1 {
-			fmt.Fprintf(sb, "	.gpio%d = %s,\n",
-				(set-1)*32+i,
-				bits[partno][(val>>i)&1])
-		}
-	}
-}
-
-func (b bd82x6x) GPIO(ctx Context, inteltool InteltoolData) {
-	var constraint uint32
-	gpio := Create(ctx, "gpio.c")
-	defer gpio.Close()
-
-	AddBootBlockFile("gpio.c", "")
-	AddROMStageFile("gpio.c", "")
-
-	Add_gpl(gpio)
-	gpio.WriteString("#include <southbridge/intel/common/gpio.h>\n\n")
-
-	addresses := [3][6]int{
-		{0x00, 0x04, 0x0c, 0x60, 0x2c, 0x18},
-		{0x30, 0x34, 0x38, 0x64, -1, -1},
-		{0x40, 0x44, 0x48, 0x68, -1, -1},
-	}
-
-	for set := 1; set <= 3; set++ {
-		for partno, part := range []string{"mode", "direction", "level", "reset", "invert", "blink"} {
-			addr := addresses[set-1][partno]
-			if addr < 0 {
-				continue
-			}
-			fmt.Fprintf(gpio, "static const struct pch_gpio_set%d pch_gpio_set%d_%s = {\n",
-				set, set, part)
-
-			constraint = 0xffffffff
-			switch part {
-			case "direction":
-				/* Ignored on native mode */
-				constraint = inteltool.GPIO[uint16(addresses[set-1][0])]
-			case "level":
-				/* Level doesn't matter for input */
-				constraint = inteltool.GPIO[uint16(addresses[set-1][0])]
-				constraint &^= inteltool.GPIO[uint16(addresses[set-1][1])]
-			case "reset":
-				/* Only show reset */
-				constraint = inteltool.GPIO[uint16(addresses[set-1][3])]
-			case "invert":
-				/* Only on input and only show inverted GPIO */
-				constraint = inteltool.GPIO[uint16(addresses[set-1][0])]
-				constraint &= inteltool.GPIO[uint16(addresses[set-1][1])]
-				constraint &= inteltool.GPIO[uint16(addresses[set-1][4])]
-			case "blink":
-				/* Only on output and only show blinking GPIO */
-				constraint = inteltool.GPIO[uint16(addresses[set-1][0])]
-				constraint &^= inteltool.GPIO[uint16(addresses[set-1][1])]
-				constraint &= inteltool.GPIO[uint16(addresses[set-1][5])]
-			}
-			b.writeGPIOSet(ctx, gpio, inteltool.GPIO[uint16(addr)], uint(set), partno, constraint)
-			gpio.WriteString("};\n\n")
-		}
-	}
-
-	gpio.WriteString(`const struct pch_gpio_map mainboard_gpio_map = {
-	.set1 = {
-		.mode		= &pch_gpio_set1_mode,
-		.direction	= &pch_gpio_set1_direction,
-		.level		= &pch_gpio_set1_level,
-		.blink		= &pch_gpio_set1_blink,
-		.invert		= &pch_gpio_set1_invert,
-		.reset		= &pch_gpio_set1_reset,
-	},
-	.set2 = {
-		.mode		= &pch_gpio_set2_mode,
-		.direction	= &pch_gpio_set2_direction,
-		.level		= &pch_gpio_set2_level,
-		.reset		= &pch_gpio_set2_reset,
-	},
-	.set3 = {
-		.mode		= &pch_gpio_set3_mode,
-		.direction	= &pch_gpio_set3_direction,
-		.level		= &pch_gpio_set3_level,
-		.reset		= &pch_gpio_set3_reset,
-	},
-};
-`)
-}
-
 func (b bd82x6x) IsPCIeHotplug(ctx Context, port int) bool {
 	portDev, ok := PCIMap[PCIAddr{Bus: 0, Dev: 0x1c, Func: port}]
 	if !ok {
@@ -168,7 +60,7 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 	SouthBridge = &b
 
 	inteltool := ctx.InfoSource.GetInteltool()
-	b.GPIO(ctx, inteltool)
+	GPIO(ctx, inteltool)
 
 	KconfigBool["SOUTHBRIDGE_INTEL_"+b.variant] = true
 	KconfigBool["SERIRQ_CONTINUOUS_MODE"] = true
@@ -194,9 +86,6 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 			Value: "\\_SB.PCI0.GFX0.DECB",
 		})
 
-	/* SPI init */
-	MainboardIncludes = append(MainboardIncludes, "southbridge/intel/bd82x6x/pch.h")
-
 	FADT := ctx.InfoSource.GetACPI()["FACP"]
 
 	pcieHotplugMap := "{ "
@@ -230,34 +119,34 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 
 			"sata_port_map": fmt.Sprintf("0x%x", PCIMap[PCIAddr{Bus: 0, Dev: 0x1f, Func: 2}].ConfigDump[0x92]&0x3f),
 
-			"docking_supported":          (FormatBool((FADT[113] & (1 << 1)) != 0)),
-			"spi_uvscc":                  fmt.Sprintf("0x%x", inteltool.RCBA[0x38c8]),
-			"spi_lvscc":                  fmt.Sprintf("0x%x", inteltool.RCBA[0x38c4]&^(1<<23)),
+			"docking_supported": (FormatBool((FADT[113] & (1 << 1)) != 0)),
+			"spi_uvscc":         fmt.Sprintf("0x%x", inteltool.RCBA[0x38c8]),
+			"spi_lvscc":         fmt.Sprintf("0x%x", inteltool.RCBA[0x38c4]&^(1<<23)),
 		},
 		PCISlots: []PCISlot{
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x14, Func: 0}, writeEmpty: false, additionalComment: "USB 3.0 Controller"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 0}, writeEmpty: true, additionalComment: "Management Engine Interface 1"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 1}, writeEmpty: true, additionalComment: "Management Engine Interface 2"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 2}, writeEmpty: true, additionalComment: "Management Engine IDE-R"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 3}, writeEmpty: true, additionalComment: "Management Engine KT"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x19, Func: 0}, writeEmpty: true, additionalComment: "Intel Gigabit Ethernet"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1a, Func: 0}, writeEmpty: true, additionalComment: "USB2 EHCI #2"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1b, Func: 0}, writeEmpty: true, additionalComment: "High Definition Audio"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 0}, writeEmpty: true, additionalComment: "PCIe Port #1"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 1}, writeEmpty: true, additionalComment: "PCIe Port #2"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 2}, writeEmpty: true, additionalComment: "PCIe Port #3"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 3}, writeEmpty: true, additionalComment: "PCIe Port #4"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 4}, writeEmpty: true, additionalComment: "PCIe Port #5"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 5}, writeEmpty: true, additionalComment: "PCIe Port #6"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 6}, writeEmpty: true, additionalComment: "PCIe Port #7"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 7}, writeEmpty: true, additionalComment: "PCIe Port #8"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1d, Func: 0}, writeEmpty: true, additionalComment: "USB2 EHCI #1"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1e, Func: 0}, writeEmpty: true, additionalComment: "PCI bridge"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 0}, writeEmpty: true, additionalComment: "LPC bridge"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 2}, writeEmpty: true, additionalComment: "SATA Controller 1"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 3}, writeEmpty: true, additionalComment: "SMBus"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 5}, writeEmpty: true, additionalComment: "SATA Controller 2"},
-			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 6}, writeEmpty: true, additionalComment: "Thermal"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x14, Func: 0}, writeEmpty: false, alias: "xhci", additionalComment: "USB 3.0 Controller"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 0}, writeEmpty: true, alias: "mei1", additionalComment: "Management Engine Interface 1"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 1}, writeEmpty: true, alias: "mei2", additionalComment: "Management Engine Interface 2"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 2}, writeEmpty: true, alias: "me_ide_r", additionalComment: "Management Engine IDE-R"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x16, Func: 3}, writeEmpty: true, alias: "me_kt", additionalComment: "Management Engine KT"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x19, Func: 0}, writeEmpty: true, alias: "gbe", additionalComment: "Intel Gigabit Ethernet"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1a, Func: 0}, writeEmpty: true, alias: "ehci2", additionalComment: "USB2 EHCI #2"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1b, Func: 0}, writeEmpty: true, alias: "hda", additionalComment: "High Definition Audio"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 0}, writeEmpty: true, alias: "pcie_rp1", additionalComment: "PCIe Port #1"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 1}, writeEmpty: true, alias: "pcie_rp2", additionalComment: "PCIe Port #2"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 2}, writeEmpty: true, alias: "pcie_rp3", additionalComment: "PCIe Port #3"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 3}, writeEmpty: true, alias: "pcie_rp4", additionalComment: "PCIe Port #4"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 4}, writeEmpty: true, alias: "pcie_rp5", additionalComment: "PCIe Port #5"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 5}, writeEmpty: true, alias: "pcie_rp6", additionalComment: "PCIe Port #6"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 6}, writeEmpty: true, alias: "pcie_rp7", additionalComment: "PCIe Port #7"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1c, Func: 7}, writeEmpty: true, alias: "pcie_rp8", additionalComment: "PCIe Port #8"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1d, Func: 0}, writeEmpty: true, alias: "ehci1", additionalComment: "USB2 EHCI #1"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1e, Func: 0}, writeEmpty: true, alias: "pci_bridge", additionalComment: "PCI bridge"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 0}, writeEmpty: true, alias: "lpc", additionalComment: "LPC bridge"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 2}, writeEmpty: true, alias: "sata1", additionalComment: "SATA Controller 1"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 3}, writeEmpty: true, alias: "smbus", additionalComment: "SMBus"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 5}, writeEmpty: true, alias: "sata2", additionalComment: "SATA Controller 2"},
+			PCISlot{PCIAddr: PCIAddr{Dev: 0x1f, Func: 6}, writeEmpty: true, alias: "thermal", additionalComment: "Thermal"},
 		},
 	}
 
@@ -278,7 +167,7 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 		File: "southbridge/intel/common/acpi/platform.asl",
 	})
 	DSDTIncludes = append(DSDTIncludes, DSDTInclude{
-		File:  "southbridge/intel/bd82x6x/acpi/globalnvs.asl",
+		File: "southbridge/intel/bd82x6x/acpi/globalnvs.asl",
 	})
 	DSDTIncludes = append(DSDTIncludes, DSDTInclude{
 		File: "southbridge/intel/common/acpi/sleepstates.asl",
@@ -294,14 +183,11 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 	defer sb.Close()
 	Add_gpl(sb)
 
-	sb.WriteString(`
-#include <bootblock_common.h>
+	sb.WriteString(`#include <bootblock_common.h>
 #include <device/pci_ops.h>
-#include <northbridge/intel/sandybridge/raminit_native.h>
 #include <southbridge/intel/bd82x6x/pch.h>
-
 `)
-	sb.WriteString("const struct southbridge_usb_port mainboard_usb_ports[] = {\n")
+	usbPortConfig := "{\n"
 
 	currentMap := map[uint32]int{
 		0x20000153: 0,
@@ -309,6 +195,10 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 		0x2000055b: 2,
 		0x20000f51: 3,
 		0x2000094a: 4,
+		0x2000035f: 5,
+		0x20000f53: 6,
+		0x20000f5b: 7,
+		0x20000553: 9,
 	}
 
 	for port := uint(0); port < 14; port++ {
@@ -327,14 +217,21 @@ func (b bd82x6x) Scan(ctx Context, addr PCIDevData) {
 				}
 			}
 		}
-		fmt.Fprintf(sb, "\t{ %d, %d, %d },\n",
-			((inteltool.RCBA[0x359c]>>port)&1)^1,
-			currentMap[inteltool.RCBA[uint16(0x3500+4*port)]],
-			OCPin)
+		current, ok := currentMap[inteltool.RCBA[uint16(0x3500+4*port)]]
+		if !ok {
+			usbPortConfig += fmt.Sprintf("\t\t\t\t{%d, 0x%x, %d},\n",
+				((inteltool.RCBA[0x359c]>>port)&1)^1,
+				inteltool.RCBA[uint16(0x3500+4*port)] & 0xfff,
+				OCPin)
+		} else {
+			usbPortConfig += fmt.Sprintf("\t\t\t\t{%d, %d, %d},\n",
+				((inteltool.RCBA[0x359c]>>port)&1)^1,
+				current,
+				OCPin)
+		}
 	}
-	sb.WriteString("};\n")
-
-	guessedMap := GuessSPDMap(ctx)
+	usbPortConfig += "\t\t\t}"
+	cur.Registers["usb_port_config"] = usbPortConfig
 
 	sb.WriteString(`
 void bootblock_mainboard_early_init(void)
@@ -344,15 +241,6 @@ void bootblock_mainboard_early_init(void)
 
 	RestorePCI16Simple(sb, addr, 0x80)
 
-	sb.WriteString(`}
-
-/* FIXME: Put proper SPD map here. */
-void mainboard_get_spd(spd_raw_data *spd, bool id_only)
-{
-`)
-	for i, spd := range guessedMap {
-		fmt.Fprintf(sb, "\tread_spd(&spd[%d], 0x%02x, id_only);\n", i, spd)
-	}
 	sb.WriteString("}\n")
 
 	gnvs := Create(ctx, "acpi_tables.c")
