@@ -8,6 +8,7 @@ import (
 	"fmt"
 	"log"
 	"os"
+	"path/filepath"
 	"sort"
 	"strings"
 )
@@ -30,9 +31,12 @@ type PCIDevice interface {
 }
 
 type InteltoolData struct {
-	GPIO map[uint16]uint32
-	RCBA map[uint16]uint32
-	IGD  map[uint32]uint32
+	GPIO   map[uint16]uint32
+	RCBA   map[uint16]uint32
+	IOBP   map[uint32]uint32
+	IGD    map[uint32]uint32
+	MCHBAR map[uint16]uint32
+	PMBASE map[uint16]uint32
 }
 
 type DMIData struct {
@@ -77,6 +81,7 @@ type SouthBridger interface {
 }
 
 var SouthBridge SouthBridger
+var BootBlockFiles map[string]string = map[string]string{}
 var ROMStageFiles map[string]string = map[string]string{}
 var RAMStageFiles map[string]string = map[string]string{}
 var SMMFiles map[string]string = map[string]string{}
@@ -94,16 +99,9 @@ type Context struct {
 	SaneVendor    string
 }
 
-type IOAPICIRQ struct {
-	APICID int
-	IRQNO  [4]int
-}
-
-var IOAPICIRQs map[PCIAddr]IOAPICIRQ = map[PCIAddr]IOAPICIRQ{}
 var KconfigBool map[string]bool = map[string]bool{}
 var KconfigComment map[string]string = map[string]string{}
 var KconfigString map[string]string = map[string]string{}
-var KconfigStringUnquoted map[string]string = map[string]string{}
 var KconfigHex map[string]uint32 = map[string]uint32{}
 var KconfigInt map[string]int = map[string]int{}
 var ROMSizeKB = 0
@@ -147,12 +145,17 @@ func sanitize(inp string) string {
 	result := strings.ToLower(inp)
 	result = strings.Replace(result, " ", "_", -1)
 	result = strings.Replace(result, ",", "_", -1)
+	result = strings.Replace(result, "-", "_", -1)
 	for strings.HasSuffix(result, ".") {
 		result = result[0 : len(result)-1]
 	}
 	return result
 }
 
+func AddBootBlockFile(Name string, Condition string) {
+	BootBlockFiles[Name] = Condition
+}
+
 func AddROMStageFile(Name string, Condition string) {
 	ROMStageFiles[Name] = Condition
 }
@@ -189,8 +192,7 @@ func writeMF(mf *os.File, files map[string]string, category string) {
 		if condition == "" {
 			fmt.Fprintf(mf, "%s-y += %s\n", category, file)
 		} else {
-			fmt.Fprintf(mf, "%s-$(%s) += %s\n", category,
-				condition, file)
+			fmt.Fprintf(mf, "%s-$(%s) += %s\n", category, condition, file)
 		}
 	}
 }
@@ -207,25 +209,9 @@ func Create(ctx Context, name string) *os.File {
 	return mf
 }
 
-func Add_gpl(fp *os.File) {
-	fp.WriteString(`/*
- * This file is part of the coreboot project.
- *
- * Copyright (C) 2008-2009 coresystems GmbH
- * Copyright (C) 2014 Vladimir Serbinenko
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 of
- * the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
-
-`)
+func Add_gpl(f *os.File) {
+	fmt.Fprintln(f, "/* SPDX-License-Identifier: GPL-2.0-only */")
+	fmt.Fprintln(f)
 }
 
 func RestorePCI16Simple(f *os.File, pcidev PCIDevData, addr uint16) {
@@ -250,6 +236,7 @@ func RestoreRCBA32(f *os.File, inteltool InteltoolData, addr uint16) {
 
 type PCISlot struct {
 	PCIAddr
+	alias             string
 	additionalComment string
 	writeEmpty        bool
 }
@@ -269,6 +256,7 @@ type DevTreeNode struct {
 	SubVendor     uint16
 	SubSystem     uint16
 	Chip          string
+	Ops           string
 	Comment       string
 }
 
@@ -288,34 +276,6 @@ func MatchDev(dev *DevTreeNode) {
 		MatchDev(&dev.Children[idx])
 	}
 
-	for _, slot := range dev.PCISlots {
-		slotChip, ok := unmatchedPCIChips[slot.PCIAddr]
-
-		if !ok {
-			continue
-		}
-
-		if slot.additionalComment != "" && slotChip.Comment != "" {
-			slotChip.Comment = slot.additionalComment + " " + slotChip.Comment
-		} else {
-			slotChip.Comment = slot.additionalComment + slotChip.Comment
-		}
-
-		delete(unmatchedPCIChips, slot.PCIAddr)
-		MatchDev(&slotChip)
-		dev.Children = append(dev.Children, slotChip)
-	}
-
-	if dev.PCIController {
-		for slot, slotDev := range unmatchedPCIChips {
-			if slot.Bus == dev.ChildPCIBus {
-				delete(unmatchedPCIChips, slot)
-				MatchDev(&slotDev)
-				dev.Children = append(dev.Children, slotDev)
-			}
-		}
-	}
-
 	for _, slot := range dev.PCISlots {
 		slotDev, ok := unmatchedPCIDevices[slot.PCIAddr]
 		if !ok {
@@ -346,6 +306,34 @@ func MatchDev(dev *DevTreeNode) {
 		delete(unmatchedPCIDevices, slot.PCIAddr)
 	}
 
+	for _, slot := range dev.PCISlots {
+		slotChip, ok := unmatchedPCIChips[slot.PCIAddr]
+
+		if !ok {
+			continue
+		}
+
+		if slot.additionalComment != "" && slotChip.Comment != "" {
+			slotChip.Comment = slot.additionalComment + " " + slotChip.Comment
+		} else {
+			slotChip.Comment = slot.additionalComment + slotChip.Comment
+		}
+
+		delete(unmatchedPCIChips, slot.PCIAddr)
+		MatchDev(&slotChip)
+		dev.Children = append(dev.Children, slotChip)
+	}
+
+	if dev.PCIController {
+		for slot, slotDev := range unmatchedPCIChips {
+			if slot.Bus == dev.ChildPCIBus {
+				delete(unmatchedPCIChips, slot)
+				MatchDev(&slotDev)
+				dev.Children = append(dev.Children, slotDev)
+			}
+		}
+	}
+
 	if dev.MissingParent != "" {
 		for _, child := range MissingChildren[dev.MissingParent] {
 			MatchDev(&child)
@@ -373,14 +361,18 @@ func writeOn(dt *os.File, dev DevTreeNode) {
 	}
 }
 
-func WriteDev(dt *os.File, offset int, dev DevTreeNode) {
+func WriteDev(dt *os.File, offset int, alias string, dev DevTreeNode) {
 	Offset(dt, offset)
 	switch dev.Chip {
-	case "cpu_cluster", "lapic", "domain", "ioapic":
+	case "cpu_cluster", "domain":
 		fmt.Fprintf(dt, "device %s 0x%x ", dev.Chip, dev.Dev)
 		writeOn(dt, dev)
 	case "pci", "pnp":
-		fmt.Fprintf(dt, "device %s %02x.%x ", dev.Chip, dev.Dev, dev.Func)
+		if alias != "" {
+			fmt.Fprintf(dt, "device ref %s ", alias)
+		} else {
+			fmt.Fprintf(dt, "device %s %02x.%x ", dev.Chip, dev.Dev, dev.Func)
+		}
 		writeOn(dt, dev)
 	case "i2c":
 		fmt.Fprintf(dt, "device %s %02x ", dev.Chip, dev.Dev)
@@ -392,21 +384,15 @@ func WriteDev(dt *os.File, offset int, dev DevTreeNode) {
 		fmt.Fprintf(dt, " # %s", dev.Comment)
 	}
 	fmt.Fprintf(dt, "\n")
+	if dev.Ops != "" {
+		Offset(dt, offset+1)
+		fmt.Fprintf(dt, "ops %s\n", dev.Ops)
+	}
 	if dev.Chip == "pci" && dev.SubSystem != 0 && dev.SubVendor != 0 {
 		Offset(dt, offset+1)
 		fmt.Fprintf(dt, "subsystemid 0x%04x 0x%04x\n", dev.SubVendor, dev.SubSystem)
 	}
 
-	ioapic, ok := IOAPICIRQs[PCIAddr{Bus: dev.Bus, Dev: dev.Dev, Func: dev.Func}]
-	if dev.Chip == "pci" && ok {
-		for pin, irq := range ioapic.IRQNO {
-			if irq != 0 {
-				Offset(dt, offset+1)
-				fmt.Fprintf(dt, "ioapic_irq %d INT%c 0x%x\n", ioapic.APICID, 'A'+pin, irq)
-			}
-		}
-	}
-
 	keys := []string{}
 	for reg, _ := range dev.Registers {
 		keys = append(keys, reg)
@@ -434,7 +420,14 @@ func WriteDev(dt *os.File, offset int, dev DevTreeNode) {
 	}
 
 	for _, child := range dev.Children {
-		WriteDev(dt, offset+1, child)
+		alias = ""
+		for _, slot := range dev.PCISlots {
+			if slot.PCIAddr.Bus == child.Bus &&
+				slot.PCIAddr.Dev == child.Dev && slot.PCIAddr.Func == child.Func {
+				alias = slot.alias
+			}
+		}
+		WriteDev(dt, offset+1, alias, child)
 	}
 
 	Offset(dt, offset)
@@ -516,14 +509,14 @@ func (g GenericPCI) Scan(ctx Context, addr PCIDevData) {
 	PutPCIDevParent(addr, g.Comment, g.MissingParent)
 }
 
+var IGDEnabled bool = false
+
 func (g GenericVGA) Scan(ctx Context, addr PCIDevData) {
 	KconfigString["VGA_BIOS_ID"] = fmt.Sprintf("%04x,%04x",
 		addr.PCIVenID,
 		addr.PCIDevID)
-	KconfigString["VGA_BIOS_FILE"] = fmt.Sprintf("pci%04x,%04x.rom",
-		addr.PCIVenID,
-		addr.PCIDevID)
 	PutPCIDevParent(addr, g.Comment, g.MissingParent)
+	IGDEnabled = true
 }
 
 func makeKconfigName(ctx Context) {
@@ -547,7 +540,7 @@ func makeKconfig(ctx Context) {
 
 	fmt.Fprintf(kc, "if %s\n\n", ctx.KconfigName)
 
-	fmt.Fprintf(kc, "config BOARD_SPECIFIC_OPTIONS # dummy\n\tdef_bool y\n")
+	fmt.Fprintf(kc, "config BOARD_SPECIFIC_OPTIONS\n\tdef_bool y\n")
 	keys := []string{}
 	for name, val := range KconfigBool {
 		if val {
@@ -578,21 +571,6 @@ config %s%s
 `, name, makeComment(name))
 	}
 
-	keys = nil
-	for name, _ := range KconfigStringUnquoted {
-		keys = append(keys, name)
-	}
-
-	sort.Strings(keys)
-
-	for _, name := range keys {
-		fmt.Fprintf(kc, `
-config %s%s
-	string
-	default %s
-`, name, makeComment(name), KconfigStringUnquoted[name])
-	}
-
 	keys = nil
 	for name, _ := range KconfigString {
 		keys = append(keys, name)
@@ -738,7 +716,7 @@ func main() {
 	ctx.MoboID = ctx.SaneVendor + "/" + sanitize(ctx.Model)
 	ctx.KconfigName = "BOARD_" + strings.ToUpper(ctx.SaneVendor+"_"+sanitize(ctx.Model))
 	ctx.BaseDirectory = *FlagOutDir + MoboDir + ctx.MoboID
-	KconfigStringUnquoted["MAINBOARD_DIR"] = ctx.MoboID
+	KconfigString["MAINBOARD_DIR"] = ctx.MoboID
 	KconfigString["MAINBOARD_PART_NUMBER"] = ctx.Model
 
 	os.MkdirAll(ctx.BaseDirectory, 0700)
@@ -747,9 +725,16 @@ func main() {
 
 	ScanRoot(ctx)
 
-	if len(ROMStageFiles) > 0 || len(RAMStageFiles) > 0 || len(SMMFiles) > 0 {
-		mf := Create(ctx, "Makefile.inc")
+	if IGDEnabled {
+		KconfigBool["MAINBOARD_HAS_LIBGFXINIT"] = true
+		KconfigComment["MAINBOARD_HAS_LIBGFXINIT"] = "FIXME: check this"
+		AddRAMStageFile("gma-mainboard.ads", "CONFIG_MAINBOARD_USE_LIBGFXINIT")
+	}
+
+	if len(BootBlockFiles) > 0 || len(ROMStageFiles) > 0 || len(RAMStageFiles) > 0 || len(SMMFiles) > 0 {
+		mf := Create(ctx, "Makefile.mk")
 		defer mf.Close()
+		writeMF(mf, BootBlockFiles, "bootblock")
 		writeMF(mf, ROMStageFiles, "romstage")
 		writeMF(mf, RAMStageFiles, "ramstage")
 		writeMF(mf, SMMFiles, "smm")
@@ -759,23 +744,24 @@ func main() {
 	defer devtree.Close()
 
 	MatchDev(&DevTree)
-	WriteDev(devtree, 0, DevTree)
+	WriteDev(devtree, 0, "", DevTree)
 
 	if MainboardInit != "" || MainboardEnable != "" || MainboardIncludes != nil {
 		mainboard := Create(ctx, "mainboard.c")
 		defer mainboard.Close()
+		Add_gpl(mainboard)
 		mainboard.WriteString("#include <device/device.h>\n")
 		for _, include := range MainboardIncludes {
 			mainboard.WriteString("#include <" + include + ">\n")
 		}
 		mainboard.WriteString("\n")
 		if MainboardInit != "" {
-			mainboard.WriteString(`static void mainboard_init(device_t dev)
+			mainboard.WriteString(`static void mainboard_init(struct device *dev)
 {
 ` + MainboardInit + "}\n\n")
 		}
 		if MainboardInit != "" || MainboardEnable != "" {
-			mainboard.WriteString("static void mainboard_enable(device_t dev)\n{\n")
+			mainboard.WriteString("static void mainboard_enable(struct device *dev)\n{\n")
 			if MainboardInit != "" {
 				mainboard.WriteString("\tdev->ops->init = mainboard_init;\n\n")
 			}
@@ -788,10 +774,6 @@ func main() {
 		}
 	}
 
-	at := Create(ctx, "acpi_tables.c")
-	defer at.Close()
-	at.WriteString("/* dummy */\n")
-
 	bi := Create(ctx, "board_info.txt")
 	defer bi.Close()
 
@@ -826,10 +808,6 @@ func main() {
 		fmt.Fprintf(bi, "FIXME: put %s\n", missing)
 	}
 
-	rs := Create(ctx, "romstage.c")
-	defer rs.Close()
-	rs.WriteString("/* dummy file */\n")
-
 	if ROMSizeKB == 0 {
 		KconfigBool["BOARD_ROMSIZE_KB_2048"] = true
 		KconfigComment["BOARD_ROMSIZE_KB_2048"] = "FIXME: correct this"
@@ -842,50 +820,82 @@ func main() {
 
 	dsdt := Create(ctx, "dsdt.asl")
 	defer dsdt.Close()
+	Add_gpl(dsdt)
 
 	for _, define := range DSDTDefines {
 		if define.Comment != "" {
-			fmt.Fprintf(dsdt, "\t/* %s.  */\n", define.Comment)
+			fmt.Fprintf(dsdt, "\t/* %s. */\n", define.Comment)
 		}
 		dsdt.WriteString("#define " + define.Key + " " + define.Value + "\n")
 	}
 
 	dsdt.WriteString(
-		`DefinitionBlock(
+`#include <acpi/acpi.h>
+
+DefinitionBlock(
 	"dsdt.aml",
 	"DSDT",
-	0x03,		// DSDT revision: ACPI v3.0
-	"COREv4",	// OEM id
-	"COREBOOT",	// OEM table id
-	0x20141018	// OEM revision
+	ACPI_DSDT_REV_2,
+	OEM_ID,
+	ACPI_TABLE_CREATOR,
+	0x20141018
 )
 {
-	// Some generic macros
+	#include <acpi/dsdt_top.asl>
 	#include "acpi/platform.asl"
 `)
 
 	for _, x := range DSDTIncludes {
 		if x.Comment != "" {
-			fmt.Fprintf(dsdt, "\t/* %s.  */\n", x.Comment)
+			fmt.Fprintf(dsdt, "\t/* %s. */\n", x.Comment)
 		}
 		fmt.Fprintf(dsdt, "\t#include <%s>\n", x.File)
 	}
 
 	dsdt.WriteString(`
-	Scope (\_SB) {
-		Device (PCI0)
-		{
+	Device (\_SB.PCI0)
+	{
 `)
 	for _, x := range DSDTPCI0Includes {
 		if x.Comment != "" {
-			fmt.Fprintf(dsdt, "\t/* %s.  */\n", x.Comment)
+			fmt.Fprintf(dsdt, "\t/* %s. */\n", x.Comment)
 		}
 		fmt.Fprintf(dsdt, "\t\t#include <%s>\n", x.File)
 	}
 	dsdt.WriteString(
-		`		}
-	}
+		`	}
 }
 `)
 
+	if IGDEnabled {
+		gma := Create(ctx, "gma-mainboard.ads")
+		defer gma.Close()
+
+		gma.WriteString(`-- SPDX-License-Identifier: GPL-2.0-or-later
+
+with HW.GFX.GMA;
+with HW.GFX.GMA.Display_Probing;
+
+use HW.GFX.GMA;
+use HW.GFX.GMA.Display_Probing;
+
+private package GMA.Mainboard is
+
+   -- FIXME: check this
+   ports : constant Port_List :=
+     (DP1,
+      DP2,
+      DP3,
+      HDMI1,
+      HDMI2,
+      HDMI3,
+      Analog,
+      LVDS,
+      eDP);
+
+end GMA.Mainboard;
+`)
+	}
+	outputPath, _ := filepath.Abs(ctx.BaseDirectory)
+	fmt.Printf("Done! Generated sources are in %s\n", outputPath)
 }
