@@ -11,7 +11,11 @@ import (
 	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/lbg"
 	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/apl"
 	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/cnl"
+	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/tgl"
 	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/adl"
+	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/jsl"
+	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/mtl"
+	"review.coreboot.org/coreboot.git/util/intelp2m/platforms/ebg"
 	"review.coreboot.org/coreboot.git/util/intelp2m/config"
 )
 
@@ -149,7 +153,16 @@ func (parser *ParserData) PlatformSpecificInterfaceSet() {
 		config.CannonType    : cnl.PlatformSpecific{
 			InheritanceTemplate : snr.PlatformSpecific{},
 		},
-		config.AlderType    : adl.PlatformSpecific{},
+		config.TigerType     : tgl.PlatformSpecific{},
+		config.AlderType     : adl.PlatformSpecific{},
+		config.JasperType    : jsl.PlatformSpecific{},
+		config.MeteorType    : mtl.PlatformSpecific{},
+		// See platforms/ebg/macro.go
+		config.EmmitsburgType : ebg.PlatformSpecific{
+			InheritanceTemplate : cnl.PlatformSpecific{
+				InheritanceTemplate : snr.PlatformSpecific{},
+			},
+		},
 	}
 	parser.platform = platform[config.PlatformGet()]
 }
