@@ -4,12 +4,15 @@
 #ifndef INTELTOOL_H
 #define INTELTOOL_H 1
 
+#if defined(__linux__)
+#include <linux/stddef.h>
+#endif
 #include <arch/mmio.h>
 #include <commonlib/helpers.h>
 
 #include <stdint.h>
 
-#if defined(__GLIBC__)
+#if defined(__linux__)
 #include <sys/io.h>
 #endif
 #if (defined(__MACH__) && defined(__APPLE__))
@@ -17,6 +20,7 @@
 #define __DARWIN__
 #include <DirectHW/DirectHW.h>
 #endif
+
 #ifdef __NetBSD__
 #include <pciutils/pci.h>
 #else
@@ -229,11 +233,29 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_Q670		0x7a83
 #define PCI_DEVICE_ID_INTEL_Z690		0x7a84
 #define PCI_DEVICE_ID_INTEL_W680		0x7a88
-#define PCI_DEVICE_ID_INTEL_W685		0x7a8a
 #define PCI_DEVICE_ID_INTEL_WM690		0x7a8d
 #define PCI_DEVICE_ID_INTEL_HM670		0x7a8c
-#define PCI_DEVICE_ID_INTEL_WM790		0x7a0d
+#define PCI_DEVICE_ID_INTEL_R680E		0x7a90
+#define PCI_DEVICE_ID_INTEL_Q670E		0x7a91
+#define PCI_DEVICE_ID_INTEL_H610E		0x7a92
+
+#define PCI_DEVICE_ID_INTEL_W790		0x7a8a
+#define PCI_DEVICE_ID_INTEL_Z790		0x7a04
+#define PCI_DEVICE_ID_INTEL_H770		0x7a05
+#define PCI_DEVICE_ID_INTEL_B760		0x7a06
 #define PCI_DEVICE_ID_INTEL_HM770		0x7a0c
+#define PCI_DEVICE_ID_INTEL_WM790		0x7a0d
+#define PCI_DEVICE_ID_INTEL_C262		0x7a14
+#define PCI_DEVICE_ID_INTEL_C266		0x7a13
+
+#define PCI_DEVICE_ID_INTEL_MTL_0		0x7e00
+#define PCI_DEVICE_ID_INTEL_MTL_1		0x7e01
+#define PCI_DEVICE_ID_INTEL_MTL_2		0x7e02
+#define PCI_DEVICE_ID_INTEL_MTL_3		0x7e03
+#define PCI_DEVICE_ID_INTEL_MTL_4		0x7e04
+#define PCI_DEVICE_ID_INTEL_MTL_5		0x7e05
+#define PCI_DEVICE_ID_INTEL_MTL_6		0x7e06
+#define PCI_DEVICE_ID_INTEL_MTL_7		0x7e07
 
 #define PCI_DEVICE_ID_INTEL_82810		0x7120
 #define PCI_DEVICE_ID_INTEL_82810_DC	0x7122
@@ -296,6 +318,10 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_ADL_P		0x5182
 #define PCI_DEVICE_ID_INTEL_ADL_M		0x5187
 #define PCI_DEVICE_ID_INTEL_RPL_P		0x519d
+#define PCI_DEVICE_ID_INTEL_ADL_N		0x5481
+
+#define PCI_DEVICE_ID_INTEL_EHL			0x4b00
+#define PCI_DEVICE_ID_INTEL_JSL			0x4d87
 
 /* untested, but almost identical to D-series */
 #define PCI_DEVICE_ID_INTEL_ATOM_NXXX	0xa010
@@ -320,7 +346,9 @@ static inline uint32_t inl(unsigned port)
 
 /* Intel starts counting these generations with the integration of the DRAM controller */
 #define PCI_DEVICE_ID_INTEL_CORE_0TH_GEN	0xd132 /* Nehalem */
-#define PCI_DEVICE_ID_INTEL_CORE_1ST_GEN	0x0044 /* Westmere */
+#define PCI_DEVICE_ID_INTEL_CORE_1ST_GEN_D	0x0040 /* Clarkdale (Westmere Desktop) */
+#define PCI_DEVICE_ID_INTEL_CORE_1ST_GEN_M	0x0044 /* Arrandale (Westmere Mobile) */
+#define PCI_DEVICE_ID_INTEL_CORE_1ST_GEN_0048	0x0048 /* Unknown Westmere */
 #define PCI_DEVICE_ID_INTEL_CORE_2ND_GEN_D	0x0100 /* Sandy Bridge (Desktop) */
 #define PCI_DEVICE_ID_INTEL_CORE_2ND_GEN_M	0x0104 /* Sandy Bridge (Mobile) */
 #define PCI_DEVICE_ID_INTEL_CORE_2ND_GEN_E3	0x0108 /* Sandy Bridge (Xeon E3) */
@@ -372,6 +400,16 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_P_6_8	0x4641 /* Alderlake P 6+8 */
 #define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_P_4_8	0x4621 /* Alderlake P 4+8 */
 #define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_P_2_8	0x4601 /* Alderlake P 2+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_N_0_8	0x4617 /* Alderlake N 0+8 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_N_0_4	0x461b /* Alderlake N 0+4 */
+#define PCI_DEVICE_ID_INTEL_CORE_ADL_ID_N_0_4_1	0x461c /* Alderlake N 0+4 */
+#define PCI_DEVICE_ID_INTEL_CORE_MTL_ID_M	0x7D00 /* Meteorlake M */
+#define PCI_DEVICE_ID_INTEL_CORE_MTL_ID_P_1	0x7D01 /* Meteorlake P */
+#define PCI_DEVICE_ID_INTEL_CORE_MTL_ID_P_2	0x7D02 /* Meteorlake P */
+#define PCI_DEVICE_ID_INTEL_CORE_MTL_ID_P_3	0x7d14 /* Meteorlake P */
+#define PCI_DEVICE_ID_INTEL_CORE_MTL_ID_P_4	0x7d15 /* Meteorlake P */
+#define PCI_DEVICE_ID_INTEL_CORE_MTL_ID_P_5	0x7d16 /* Meteorlake P */
+#define PCI_DEVICE_ID_INTEL_CORE_RPL_ID_H_8_6	0xa706 /* Raptorlake H 8+6 */
 
 /* Intel GPUs */
 #define PCI_DEVICE_ID_INTEL_G35_EXPRESS		0x2982
@@ -452,6 +490,15 @@ static inline uint32_t inl(unsigned port)
 #define PCI_DEVICE_ID_INTEL_ADL_S_GT1_2		0x4682
 #define PCI_DEVICE_ID_INTEL_ADL_S_GT1_3		0x4690
 #define PCI_DEVICE_ID_INTEL_ADL_S_GT1_4		0x4692
+#define PCI_DEVICE_ID_INTEL_ADL_N_UHD		0x46D0
+#define PCI_DEVICE_ID_INTEL_ADL_N_UHD_2		0x46D1
+#define PCI_DEVICE_ID_INTEL_ADL_N_UHD_3		0x46D2
+#define PCI_DEVICE_ID_INTEL_MTL_M_GT2		0x7d40
+#define PCI_DEVICE_ID_INTEL_MTL_P_GT2_1		0x7d45
+#define PCI_DEVICE_ID_INTEL_MTL_P_GT2_2		0x7d50
+#define PCI_DEVICE_ID_INTEL_MTL_P_GT2_3		0x7d55
+#define PCI_DEVICE_ID_INTEL_MTL_P_GT2_4		0x7d60
+#define PCI_DEVICE_ID_INTEL_RPL_H_IRIS_XE	0xa7a0
 
 #if !defined(__DARWIN__) && !defined(__FreeBSD__)
 typedef struct { uint32_t hi, lo; } msr_t;
@@ -496,6 +543,7 @@ int print_gfx(struct pci_dev *gfx);
 int print_ahci(struct pci_dev *ahci);
 int print_sgx(void);
 void print_tme(void);
+void print_keylocker(void);
 void ivybridge_dump_timings(const char *dump_spd_file);
 
 #endif
