@@ -81,7 +81,6 @@ static struct device override_root_dev;
 static struct chip_instance mainboard_instance;
 
 static struct bus base_root_bus = {
-	.id = 0,
 	.dev = &base_root_dev,
 };
 
@@ -95,7 +94,6 @@ static struct device base_root_dev = {
 };
 
 static struct bus chipset_root_bus = {
-	.id = 0,
 	.dev = &chipset_root_dev,
 };
 
@@ -109,7 +107,6 @@ static struct device chipset_root_dev = {
 };
 
 static struct bus override_root_bus = {
-	.id = 0,
 	.dev = &override_root_dev,
 };
 
@@ -317,7 +314,7 @@ static struct chip *get_chip(char *path)
 	sprintf(chip_h, "src/%s", path);
 	if ((stat(chip_h, &st) == -1) && (errno == ENOENT)) {
 		/* root_complex gets away without a separate directory, but
-		 * exists on on pretty much all AMD chipsets.
+		 * exists on pretty much all AMD chipsets.
 		 */
 		if (!strstr(path, "/root_complex")) {
 			fprintf(stderr, "ERROR: Chip component %s does not exist.\n",
@@ -584,6 +581,11 @@ void add_fw_config_probe(struct bus *bus, const char *field, const char *option)
 	append_fw_config_probe_to_dev(bus->dev, probe);
 }
 
+void probe_unprovisioned_fw_config(struct bus *bus)
+{
+	bus->dev->enable_on_unprovisioned_fw_config = true;
+}
+
 static uint64_t compute_fw_config_mask(const struct fw_config_field_bits *bits)
 {
 	uint64_t mask = 0;
@@ -723,30 +725,13 @@ void add_device_ops(struct bus *bus, char *ops_id)
 	bus->dev->ops_id = ops_id;
 }
 
-/*
- * Allocate a new bus for the provided device.
- *   - If this is the first bus being allocated under this device, then its id
- *     is set to 0 and bus and last_bus are pointed to the newly allocated bus.
- *   - If this is not the first bus under this device, then its id is set to 1
- *     plus the id of last bus and newly allocated bus is added to the list of
- *     buses under the device. last_bus is updated to point to the newly
- *     allocated bus.
- */
+/* Allocate a new bus for the provided device. */
 static void alloc_bus(struct device *dev)
 {
 	struct bus *bus = S_ALLOC(sizeof(*bus));
 
 	bus->dev = dev;
-
-	if (dev->last_bus == NULL)  {
-		bus->id = 0;
-		dev->bus = bus;
-	} else {
-		bus->id = dev->last_bus->id + 1;
-		dev->last_bus->next_bus = bus;
-	}
-
-	dev->last_bus = bus;
+	dev->bus = bus;
 }
 
 /*
@@ -814,14 +799,15 @@ static const struct device *find_alias(const struct device *const parent,
 	if (parent->alias && !strcmp(parent->alias, alias))
 		return parent;
 
-	const struct bus *bus;
-	for (bus = parent->bus; bus; bus = bus->next_bus) {
-		const struct device *child;
-		for (child = bus->children; child; child = child->sibling) {
-			const struct device *const ret = find_alias(child, alias);
-			if (ret)
-				return ret;
-		}
+	const struct bus *bus = parent->bus;
+	if (!bus)
+		return NULL;
+
+	const struct device *child;
+	for (child = bus->children; child; child = child->sibling) {
+		const struct device *const ret = find_alias(child, alias);
+		if (ret)
+			return ret;
 	}
 
 	return NULL;
@@ -834,11 +820,11 @@ static struct device *new_device_with_path(struct bus *parent,
 {
 	struct device *new_d;
 
-	/* If device is found under parent, no need to allocate new device. */
+	/* We don't allow duplicate devices in devicetree. */
 	new_d = get_dev(parent, path_a, path_b, bustype, chip_instance);
 	if (new_d) {
-		alloc_bus(new_d);
-		return new_d;
+		printf("ERROR: Duplicate device! %s\n", new_d->name);
+		exit(1);
 	}
 
 	new_d = alloc_dev(parent);
@@ -869,10 +855,6 @@ static struct device *new_device_with_path(struct bus *parent,
 		new_d->path = ".type=DEVICE_PATH_I2C,{.i2c={ .device = 0x%x, .mode_10bit = %d }}";
 		break;
 
-	case APIC:
-		new_d->path = ".type=DEVICE_PATH_APIC,{.apic={ .apic_id = 0x%x }}";
-		break;
-
 	case CPU_CLUSTER:
 		new_d->path = ".type=DEVICE_PATH_CPU_CLUSTER,{.cpu_cluster={ .cluster = 0x%x }}";
 		break;
@@ -882,11 +864,7 @@ static struct device *new_device_with_path(struct bus *parent,
 		break;
 
 	case DOMAIN:
-		new_d->path = ".type=DEVICE_PATH_DOMAIN,{.domain={ .domain = 0x%x }}";
-		break;
-
-	case IOAPIC:
-		new_d->path = ".type=DEVICE_PATH_IOAPIC,{.ioapic={ .ioapic_id = 0x%x }}";
+		new_d->path = ".type=DEVICE_PATH_DOMAIN,{.domain={ .domain_id = 0x%x }}";
 		break;
 
 	case GENERIC:
@@ -908,6 +886,10 @@ static struct device *new_device_with_path(struct bus *parent,
 	case GPIO:
 		new_d->path = ".type=DEVICE_PATH_GPIO,{.gpio={ .id = 0x%x }}";
 		break;
+
+	case MDIO:
+		new_d->path = ".type=DEVICE_PATH_MDIO,{.mdio={ .addr = 0x%x }}";
+		break;
 	}
 
 	return new_d;
@@ -1084,42 +1066,12 @@ void add_pci_subsystem_ids(struct bus *bus, int vendor, int device,
 	dev->inherit_subsystem = inherit;
 }
 
-void add_ioapic_info(struct bus *bus, int apicid, const char *_srcpin,
-		     int irqpin)
-{
-	int srcpin;
-	struct device *dev = bus->dev;
-
-	if (!_srcpin || strlen(_srcpin) < 4 || strncasecmp(_srcpin, "INT", 3) ||
-	    _srcpin[3] < 'A' || _srcpin[3] > 'D') {
-		printf("ERROR: malformed ioapic_irq args: %s\n", _srcpin);
-		exit(1);
-	}
-
-	srcpin = _srcpin[3] - 'A';
-
-	if (dev->bustype != PCI && dev->bustype != DOMAIN) {
-		printf("ERROR: ioapic config only allowed for PCI devices\n");
-		exit(1);
-	}
-
-	if (srcpin > 3) {
-		printf("ERROR: srcpin '%d' invalid\n", srcpin);
-		exit(1);
-	}
-	dev->pci_irq_info[srcpin].ioapic_irq_pin = irqpin;
-	dev->pci_irq_info[srcpin].ioapic_dst_id = apicid;
-}
-
 static int dev_has_children(struct device *dev)
 {
 	struct bus *bus = dev->bus;
 
-	while (bus) {
-		if (bus->children)
-			return 1;
-		bus = bus->next_bus;
-	}
+	if (bus && bus->children)
+		return 1;
 
 	return 0;
 }
@@ -1129,7 +1081,7 @@ static void pass0(FILE *fil, FILE *head, struct device *ptr, struct device *next
 	static int dev_id;
 
 	if (ptr == &base_root_dev) {
-		fprintf(fil, "STORAGE struct bus %s_links[];\n",
+		fprintf(fil, "STORAGE struct bus %s_bus;\n",
 			ptr->name);
 		return;
 	}
@@ -1151,7 +1103,7 @@ static void pass0(FILE *fil, FILE *head, struct device *ptr, struct device *next
 		fprintf(fil, "STORAGE struct resource %s_res[];\n",
 			ptr->name);
 	if (dev_has_children(ptr))
-		fprintf(fil, "STORAGE struct bus %s_links[];\n",
+		fprintf(fil, "STORAGE struct bus %s_bus;\n",
 			ptr->name);
 
 	if (next)
@@ -1223,35 +1175,18 @@ static void emit_resources(FILE *fil, struct device *ptr)
 	fprintf(fil, "\t };\n");
 }
 
-static void emit_bus(FILE *fil, struct bus *bus)
+static void emit_dev_bus(FILE *fil, struct device *ptr)
 {
-	fprintf(fil, "\t\t[%d] = {\n", bus->id);
-	fprintf(fil, "\t\t\t.link_num = %d,\n", bus->id);
-	fprintf(fil, "\t\t\t.dev = &%s,\n", bus->dev->name);
-	if (bus->children)
-		fprintf(fil, "\t\t\t.children = &%s,\n", bus->children->name);
-
-	if (bus->next_bus)
-		fprintf(fil, "\t\t\t.next=&%s_links[%d],\n", bus->dev->name,
-			bus->id + 1);
-	else
-		fprintf(fil, "\t\t\t.next = NULL,\n");
-	fprintf(fil, "\t\t},\n");
-}
-
-static void emit_dev_links(FILE *fil, struct device *ptr)
-{
-	fprintf(fil, "STORAGE struct bus %s_links[] = {\n",
+	fprintf(fil, "STORAGE struct bus %s_bus = {\n",
 		ptr->name);
 
+	assert(ptr->bus && ptr->bus->children);
 	struct bus *bus = ptr->bus;
 
-	while (bus) {
-		emit_bus(fil, bus);
-		bus = bus->next_bus;
-	}
+	fprintf(fil, "\t.dev = &%s,\n", bus->dev->name);
+	fprintf(fil, "\t.children = &%s,\n", bus->children->name);
 
-	fprintf(fil, "\t};\n");
+	fprintf(fil, "};\n");
 }
 
 static struct chip_instance *get_chip_instance(const struct device *dev)
@@ -1269,7 +1204,6 @@ static struct chip_instance *get_chip_instance(const struct device *dev)
 
 static void pass1(FILE *fil, FILE *head, struct device *ptr, struct device *next)
 {
-	int pin;
 	struct chip_instance *chip_ins = get_chip_instance(ptr);
 	int has_children = dev_has_children(ptr);
 
@@ -1300,8 +1234,7 @@ static void pass1(FILE *fil, FILE *head, struct device *ptr, struct device *next
 	else
 		fprintf(fil, "\t.ops = NULL,\n");
 	fprintf(fil, "#endif\n");
-	fprintf(fil, "\t.bus = &%s_links[%d],\n", ptr->parent->dev->name,
-		ptr->parent->id);
+	fprintf(fil, "\t.upstream = &%s_bus,\n", ptr->parent->dev->name);
 	fprintf(fil, "\t.path = {");
 	fprintf(fil, ptr->path, ptr->path_a, ptr->path_b);
 	fprintf(fil, "},\n");
@@ -1322,28 +1255,19 @@ static void pass1(FILE *fil, FILE *head, struct device *ptr, struct device *next
 			ptr->name);
 	}
 	if (has_children)
-		fprintf(fil, "\t.link_list = &%s_links[0],\n",
+		fprintf(fil, "\t.downstream = &%s_bus,\n",
 			ptr->name);
 	else
-		fprintf(fil, "\t.link_list = NULL,\n");
+		fprintf(fil, "\t.downstream = NULL,\n");
 	if (ptr->sibling)
 		fprintf(fil, "\t.sibling = &%s,\n", ptr->sibling->name);
 	else
 		fprintf(fil, "\t.sibling = NULL,\n");
 	if (ptr->probe)
 		fprintf(fil, "\t.probe_list = %s_probe_list,\n", ptr->name);
+	fprintf(fil, "\t.enable_on_unprovisioned_fw_config = %d,\n",
+		ptr->enable_on_unprovisioned_fw_config);
 	fprintf(fil, "#if !DEVTREE_EARLY\n");
-	for (pin = 0; pin < 4; pin++) {
-		if (ptr->pci_irq_info[pin].ioapic_irq_pin > 0)
-			fprintf(fil,
-				"\t.pci_irq_info[%d].ioapic_irq_pin = %d,\n",
-				pin, ptr->pci_irq_info[pin].ioapic_irq_pin);
-
-		if (ptr->pci_irq_info[pin].ioapic_dst_id > 0)
-			fprintf(fil,
-				"\t.pci_irq_info[%d].ioapic_dst_id = %d,\n",
-				pin, ptr->pci_irq_info[pin].ioapic_dst_id);
-	}
 	fprintf(fil, "\t.chip_ops = &%s_ops,\n",
 		chip_ins->chip->name_underscore);
 	if (chip_ins == &mainboard_instance)
@@ -1362,7 +1286,7 @@ static void pass1(FILE *fil, FILE *head, struct device *ptr, struct device *next
 	emit_resources(fil, ptr);
 
 	if (has_children)
-		emit_dev_links(fil, ptr);
+		emit_dev_bus(fil, ptr);
 }
 
 static void expose_device_names(FILE *fil, FILE *head, struct device *ptr, struct device *next)
@@ -1371,20 +1295,29 @@ static void expose_device_names(FILE *fil, FILE *head, struct device *ptr, struc
 
 	/* Only devices on root bus here. */
 	if (ptr->bustype == PCI && ptr->parent->dev->bustype == DOMAIN) {
-		fprintf(head, "extern DEVTREE_CONST struct device *const __pci_0_%02x_%d;\n",
-			ptr->path_a, ptr->path_b);
-		fprintf(fil, "DEVTREE_CONST struct device *const __pci_0_%02x_%d = &%s;\n",
-			ptr->path_a, ptr->path_b, ptr->name);
+		if (ptr->alias) {
+			fprintf(head, "static const pci_devfn_t _sdev_%s = PCI_DEV(%d, %d, %d);\n",
+				ptr->alias, ptr->parent->dev->path_a, ptr->path_a, ptr->path_b);
+		}
+		fprintf(head, "extern DEVTREE_CONST struct device *const __pci_%d_%02x_%d;\n",
+			ptr->parent->dev->path_a, ptr->path_a, ptr->path_b);
+		fprintf(fil, "DEVTREE_CONST struct device *const __pci_%d_%02x_%d = &%s;\n",
+			ptr->parent->dev->path_a, ptr->path_a, ptr->path_b, ptr->name);
 
 		if (chip_ins->chip->chiph_exists) {
-			fprintf(head, "extern DEVTREE_CONST void *const __pci_0_%02x_%d_config;\n",
-				ptr->path_a, ptr->path_b);
-			fprintf(fil, "DEVTREE_CONST void *const __pci_0_%02x_%d_config = &%s_info_%d;\n",
-				ptr->path_a, ptr->path_b, chip_ins->chip->name_underscore, chip_ins->id);
+			fprintf(head, "extern DEVTREE_CONST void *const __pci_%d_%02x_%d_config;\n",
+				ptr->parent->dev->path_a, ptr->path_a, ptr->path_b);
+			fprintf(fil, "DEVTREE_CONST void *const __pci_%d_%02x_%d_config = &%s_info_%d;\n",
+				ptr->parent->dev->path_a, ptr->path_a, ptr->path_b,
+				chip_ins->chip->name_underscore, chip_ins->id);
 		}
 	}
 
 	if (ptr->bustype == PNP) {
+		if (ptr->alias) {
+			fprintf(head, "static const pnp_devfn_t _sdev_%s = PNP_DEV(0x%02x, 0x%04x);\n",
+				ptr->alias, ptr->path_a, ptr->path_b);
+		}
 		fprintf(head, "extern DEVTREE_CONST struct device *const __pnp_%04x_%02x;\n",
 			ptr->path_a, ptr->path_b);
 		fprintf(fil, "DEVTREE_CONST struct device *const __pnp_%04x_%02x = &%s;\n",
@@ -1412,11 +1345,8 @@ static void add_children_to_queue(struct queue_entry **bfs_q_head,
 {
 	struct bus *bus = d->bus;
 
-	while (bus) {
-		if (bus->children)
-			add_siblings_to_queue(bfs_q_head, bus->children);
-		bus = bus->next_bus;
-	}
+	if (dev_has_children(d))
+		add_siblings_to_queue(bfs_q_head, bus->children);
 }
 
 static void walk_device_tree(FILE *fil, FILE *head, struct device *ptr,
@@ -1536,20 +1466,6 @@ static void inherit_subsystem_ids(FILE *file, FILE *head, struct device *dev,
 	}
 }
 
-static void usage(void)
-{
-	printf("usage: sconfig <options>\n");
-	printf("  -c | --output_c          : Path to output static.c file (required)\n");
-	printf("  -r | --output_h          : Path to header static.h file (required)\n");
-	printf("  -d | --output_d          : Path to header static_devices.h file (required)\n");
-	printf("  -f | --output_f          : Path to header static_fw_config.h file (required)\n");
-	printf("  -m | --mainboard_devtree : Path to mainboard devicetree file (required)\n");
-	printf("  -o | --override_devtree  : Path to override devicetree file (optional)\n");
-	printf("  -p | --chipset_devtree   : Path to chipset/SOC devicetree file (optional)\n");
-
-	exit(1);
-}
-
 static void parse_devicetree(const char *file, struct bus *parent)
 {
 	FILE *filec = fopen(file, "r");
@@ -1780,12 +1696,9 @@ static void override_devicetree(struct bus *base_parent,
  * +-----------------------------------------------------------------+
  * |                    |                                            |
  * | bus                | Recursively call override_devicetree on    |
- * | last_bus           | each bus of override device. It is assumed |
+ * |                    | each bus of override device. It is assumed |
  * |                    | that bus with id X under base device       |
- * |                    | to bus with id X under override device. If |
- * |                    | override device has more buses than base   |
- * |                    | device, then new buses are allocated under |
- * |                    | base device.                               |
+ * |                    | to bus with id X under override device.    |
  * |                    |                                            |
  * +-----------------------------------------------------------------+
  */
@@ -1877,6 +1790,8 @@ static void update_device(struct device *base_dev, struct device *override_dev)
 	 * to allow an override to remove a probe from the base device.
 	 */
 	base_dev->probe = override_dev->probe;
+	base_dev->enable_on_unprovisioned_fw_config =
+		override_dev->enable_on_unprovisioned_fw_config;
 
 	/* Copy SMBIOS slot information from base device */
 	base_dev->smbios_slot_type = override_dev->smbios_slot_type;
@@ -1897,29 +1812,20 @@ static void update_device(struct device *base_dev, struct device *override_dev)
 	/*
 	 * Now that the device properties are all copied over, look at each bus
 	 * of the override device and run override_devicetree in a recursive
-	 * manner. The assumption here is that first bus of override device
-	 * corresponds to first bus of base device and so on. If base device has
-	 * lesser buses than override tree, then new buses are allocated for it.
+	 * manner. If base device has no bus but the override tree has, then a new
+	 * bus is allocated for it.
 	 */
 	struct bus *override_bus = override_dev->bus;
 	struct bus *base_bus = base_dev->bus;
 
-	while (override_bus) {
+	/*
+	 * If we have more buses in override tree device, then allocate
+	 * a new bus for the base tree device as well.
+	 */
+	if (!base_bus)
+		alloc_bus(base_dev);
 
-		/*
-		 * If we have more buses in override tree device, then allocate
-		 * a new bus for the base tree device as well.
-		 */
-		if (!base_bus) {
-			alloc_bus(base_dev);
-			base_bus = base_dev->last_bus;
-		}
-
-		override_devicetree(base_dev->bus, override_dev->bus);
-
-		override_bus = override_bus->next_bus;
-		base_bus = base_bus->next_bus;
-	}
+	override_devicetree(base_dev->bus, override_dev->bus);
 }
 
 /*
@@ -2008,6 +1914,7 @@ static void generate_outputc(FILE *f, const char *static_header)
 	fprintf(f, "#include <device/device.h>\n");
 	fprintf(f, "#include <device/pci.h>\n");
 	fprintf(f, "#include <fw_config.h>\n");
+	fprintf(f, "#include <identity.h>\n");
 	fprintf(f, "#include <%s>\n", static_header);
 	emit_chip_headers(f, chip_header.next);
 	emit_identifiers(f, "struct device_operations", device_operations);
@@ -2027,6 +1934,8 @@ static void generate_outputd(FILE *gen, FILE *dev)
 {
 	fprintf(dev, "#ifndef __STATIC_DEVICES_H\n");
 	fprintf(dev, "#define __STATIC_DEVICES_H\n\n");
+	fprintf(dev, "#include <device/pci_type.h>\n");
+	fprintf(dev, "#include <device/pnp_type.h>\n");
 	fprintf(dev, "#include <device/device.h>\n\n");
 	fprintf(dev, "/* expose_device_names */\n");
 	walk_device_tree(gen, dev, &base_root_dev, expose_device_names);
@@ -2041,17 +1950,31 @@ static void generate_outputf(FILE *f)
 	fprintf(f, "\n#endif /* __STATIC_FW_CONFIG_H */\n");
 }
 
+static void usage(void)
+{
+	printf("Usage: sconfig <options>\n");
+	printf("  -c | --output_c          : Path to output static.c file (required)\n");
+	printf("  -r | --output_h          : Path to header static.h file (required)\n");
+	printf("  -d | --output_d          : Path to header static_devices.h file (required)\n");
+	printf("  -f | --output_f          : Path to header static_fw_config.h file (required)\n");
+	printf("  -m | --mainboard_devtree : Path to mainboard devicetree file (required)\n");
+	printf("  -o | --override_devtree  : Path to override devicetree file (optional)\n");
+	printf("  -p | --chipset_devtree   : Path to chipset/SOC devicetree file (optional)\n");
+
+	exit(1);
+}
+
 int main(int argc, char **argv)
 {
 	static const struct option long_options[] = {
-		{ "mainboard_devtree", 1, NULL, 'm' },
-		{ "override_devtree", 1, NULL, 'o' },
-		{ "chipset_devtree", 1, NULL, 'p' },
-		{ "output_c", 1, NULL, 'c' },
-		{ "output_h", 1, NULL, 'r' },
-		{ "output_d", 1, NULL, 'd' },
-		{ "output_f", 1, NULL, 'f' },
-		{ "help", 1, NULL, 'h' },
+		{ "mainboard_devtree", required_argument, NULL, 'm' },
+		{ "override_devtree", required_argument, NULL, 'o' },
+		{ "chipset_devtree", required_argument, NULL, 'p' },
+		{ "output_c", required_argument, NULL, 'c' },
+		{ "output_h", required_argument, NULL, 'r' },
+		{ "output_d", required_argument, NULL, 'd' },
+		{ "output_f", required_argument, NULL, 'f' },
+		{ "help", no_argument, NULL, 'h' },
 		{ }
 	};
 	const char *override_devtree = NULL;
@@ -2067,25 +1990,25 @@ int main(int argc, char **argv)
 				  &option_index)) != EOF) {
 		switch (opt) {
 		case 'm':
-			base_devtree = strdup(optarg);
+			base_devtree = optarg;
 			break;
 		case 'o':
-			override_devtree = strdup(optarg);
+			override_devtree = optarg;
 			break;
 		case 'p':
-			chipset_devtree = strdup(optarg);
+			chipset_devtree = optarg;
 			break;
 		case 'c':
-			outputc = strdup(optarg);
+			outputc = optarg;
 			break;
 		case 'r':
-			outputh = strdup(optarg);
+			outputh = optarg;
 			break;
 		case 'd':
-			outputd = strdup(optarg);
+			outputd = optarg;
 			break;
 		case 'f':
-			outputf = strdup(optarg);
+			outputf = optarg;
 			break;
 		case 'h':
 		default:
